const DELIMITER = /[:;]/;
export class RationalTime {
    constructor(value = 0, rate = 1) {
        this.epsilon = 1e-6;
        this.value = value;
        this.rate = rate;
    }
    /**
     * Determines if the current `RationalTime` instance is almost equal to another `RationalTime` instance
     * within a specified delta.
     *
     * @param other - The other `RationalTime` instance to compare against.
     * @param delta - The maximum allowable difference in seconds for the two times to be considered almost equal.
     *                 Defaults to the value of `this.epsilon`.
     * @returns `true` if the absolute difference between the two times is less than or equal to `delta`, otherwise `false`.
     */
    almostEqual(other, delta = this.epsilon) {
        const diff = Math.abs(this.toSeconds() - other.toSeconds());
        return diff <= delta;
    }
    /**
     * Rounds the value of the current `RationalTime` instance up to the nearest integer.
     *
     * @returns {RationalTime} A new `RationalTime` instance with the value rounded up.
     */
    ceil() {
        return new RationalTime(Math.ceil(this.value), this.rate);
    }
    /**
     * Calculates the duration between two RationalTime instances.
     *
     * @param startTime - The starting time.
     * @param endTimeExclusive - The ending time (exclusive).
     * @returns A new RationalTime instance representing the duration between the start and end times.
     */
    static durationFromStartEndTime(startTime, endTimeExclusive) {
        const durationValue = endTimeExclusive.value - startTime.value;
        return new RationalTime(durationValue, startTime.rate);
    }
    /**
     * Calculates the duration between two RationalTime instances, inclusive of the end time.
     *
     * @param startTime - The starting RationalTime instance.
     * @param endTimeInclusive - The ending RationalTime instance, inclusive.
     * @returns A new RationalTime instance representing the duration from start to end, inclusive.
     */
    static durationFromStartEndTimeInclusive(startTime, endTimeInclusive) {
        const durationValue = endTimeInclusive.value - startTime.value + 1;
        return new RationalTime(durationValue, startTime.rate);
    }
    /**
     * Rounds the value of the current `RationalTime` instance down to the nearest integer.
     *
     * @returns A new `RationalTime` instance with the floored value and the same rate.
     */
    floor() {
        return new RationalTime(Math.floor(this.value), this.rate);
    }
    /**
     * Creates a `RationalTime` instance from the given frame number and rate.
     *
     * @param frame - The frame number to be converted to `RationalTime`.
     * @param rate - The rate at which the frames are being played.
     * @returns A new `RationalTime` instance representing the given frame and rate.
     */
    static fromFrames(frame, rate) {
        return new RationalTime(frame, rate);
    }
    static fromSeconds(seconds, rate = 1) {
        return new RationalTime(seconds * rate, rate);
    }
    // Converts a time string (HH:MM:ss[.ff]) to a RationalTime.
    static fromTimeString(timeString, rate) {
        const seconds = RationalTime.parseTimeStringToSeconds(timeString);
        return RationalTime.fromSeconds(seconds, rate);
    }
    // Converts a timecode string (HH:MM:SS;FF or HH:MM:SS:FF) to a RationalTime.
    static fromTimecode(timecode, rate) {
        const frames = RationalTime.parseTimecodeToFrames(timecode, rate);
        return RationalTime.fromFrames(frames, rate);
    }
    // Returns true if the time is invalid.
    isInvalidTime() {
        return isNaN(this.value) || isNaN(this.rate) || this.rate <= 0;
    }
    // Checks if the rate is valid for use with timecode.
    static isValidTimecodeRate(rate) {
        const validRates = [23.976, 24, 25, 29.97, 30, 48, 50, 59.94, 60];
        return validRates.includes(rate);
    }
    // Returns the nearest valid timecode rate to the given value.
    static nearestValidTimecodeRate(rate) {
        const validRates = [23.976, 24, 25, 29.97, 30, 48, 50, 59.94, 60];
        let nearestRate = validRates[0];
        let minDiff = Math.abs(rate - validRates[0]);
        for (const validRate of validRates) {
            const diff = Math.abs(rate - validRate);
            if (diff < minDiff) {
                minDiff = diff;
                nearestRate = validRate;
            }
        }
        return nearestRate;
    }
    // Returns a new RationalTime rescaled to the new rate.
    rescaledTo(newRate) {
        if (newRate instanceof RationalTime) {
            newRate = newRate.rate;
        }
        const newValue = (this.value * newRate) / this.rate;
        return new RationalTime(newValue, newRate);
    }
    // Returns a new RationalTime with the value rounded to the nearest whole number.
    round() {
        return new RationalTime(Math.round(this.value), this.rate);
    }
    // Checks if this RationalTime is exactly equal to another.
    strictlyEqual(other) {
        return this.value === other.value && this.rate === other.rate;
    }
    // Converts the time to frame number based on the current rate or provided rate.
    toFrames(rate) {
        const r = rate !== null && rate !== void 0 ? rate : this.rate;
        return Math.round(this.valueRescaledTo(r));
    }
    // Converts the time to the nearest timecode string.
    toNearestTimecode(rate = this.rate, dropFrame = undefined) {
        const frames = this.toFrames(rate);
        return RationalTime.framesToTimecode(frames, rate, dropFrame);
    }
    // Converts the time to seconds.
    toSeconds() {
        return this.value / this.rate;
    }
    // Converts the time to a time string (HH:MM:ss.ff).
    toTimeString() {
        const totalSeconds = this.toSeconds();
        const hours = Math.floor(totalSeconds / 3600);
        const minutes = Math.floor((totalSeconds % 3600) / 60);
        const secondsNumber = totalSeconds % 60;
        const secondsIntegerPart = Math.floor(secondsNumber);
        const secondsFractionPart = (secondsNumber - secondsIntegerPart)
            .toFixed(2)
            .substring(1);
        const secondsStr = `${String(secondsIntegerPart).padStart(2, "0")}${secondsFractionPart}`;
        return `${String(hours).padStart(2, "0")}:${String(minutes).padStart(2, "0")}:${secondsStr}`;
    }
    /**
     * Converts the current RationalTime instance to a timecode string.
     *
     * @param rate - The frame rate to use for the conversion. Defaults to the instance's rate.
     * @param dropFrame - Optional boolean indicating whether to use drop-frame timecode. If undefined, the default behavior is used.
     * @returns The timecode string representation of the current RationalTime instance.
     */
    toTimecode(rate = this.rate, dropFrame = undefined) {
        const frames = this.toFrames(rate);
        return RationalTime.framesToTimecode(frames, rate, dropFrame);
    }
    // Returns the time value rescaled to the new rate.
    valueRescaledTo(newRate) {
        if (newRate instanceof RationalTime) {
            newRate = newRate.rate;
        }
        return (this.value * newRate) / this.rate;
    }
    // Helper method to parse time strings to seconds.
    /**
     * Parses a time string in the format "HH:MM:SS" and converts it to the total number of seconds.
     *
     * @param timeString - The time string to parse, expected in the format "HH:MM:SS".
     * @returns The total number of seconds represented by the time string.
     * @throws Will throw an error if the time string format is invalid.
     */
    static parseTimeStringToSeconds(timeString) {
        const parts = timeString.split(":");
        if (parts.length !== 3) {
            throw new Error("Invalid time string format");
        }
        const hours = parseFloat(parts[0]);
        const minutes = parseFloat(parts[1]);
        const seconds = parseFloat(parts[2]);
        return hours * 3600 + minutes * 60 + seconds;
    }
    // Helper method to parse timecode strings to frames.
    static parseTimecodeToFrames(timecode, rate) {
        const parts = timecode.split(DELIMITER);
        if (parts.length !== 4) {
            throw new Error("Invalid timecode format");
        }
        const hours = parseInt(parts[0], 10);
        const minutes = parseInt(parts[1], 10);
        const seconds = parseInt(parts[2], 10);
        const frames = parseInt(parts[3], 10);
        const totalSeconds = hours * 3600 + minutes * 60 + seconds;
        return Math.round(totalSeconds * rate + frames);
    }
    // Helper method to convert frames to a timecode string.
    static framesToTimecode(frames, rate, dropFrame) {
        const frameRate = rate;
        const totalSeconds = Math.floor(frames / frameRate);
        const remainingFrames = frames % frameRate;
        const hours = Math.floor(totalSeconds / 3600);
        const minutes = Math.floor((totalSeconds % 3600) / 60);
        const seconds = totalSeconds % 60;
        const delimiter = dropFrame ? ";" : ":";
        const hoursStr = String(hours).padStart(2, "0");
        const minutesStr = String(minutes).padStart(2, "0");
        const secondsStr = String(seconds).padStart(2, "0");
        const framesStr = String(Math.round(remainingFrames)).padStart(2, "0");
        return `${hoursStr}:${minutesStr}:${secondsStr}${delimiter}${framesStr}`;
    }
}
//# sourceMappingURL=RationalTime.js.map