import { describe, it, expect } from "vitest";
import { backgroundTasksRisingEdge, getTasksByAction, } from "./ICollectionDocument";
import { WorkerAction, WorkerState, } from "./ICollectionDocument";
describe("backgroundTasksRisingEdge", () => {
    const createTask = (action, state) => ({
        id: "1",
        action,
        state,
        name: "",
        progress: 0,
        pipeConfig: "",
        createdAt: undefined,
    });
    const createDocument = (tasks) => ({
        backgroundTasks: tasks,
        id: "",
    });
    it("should return true if a task has transitioned to the specified state", () => {
        const action = WorkerAction.None;
        const stateRise = WorkerState.Completed;
        const priorTask = createTask(action, WorkerState.Running);
        const posteriorTask = createTask(action, stateRise);
        const prior = createDocument([priorTask]);
        const posterior = createDocument([posteriorTask]);
        const result = backgroundTasksRisingEdge(action, stateRise, posterior, prior);
        expect(result).toBeTruthy();
    });
    it("should return false if no task has transitioned to the specified state", () => {
        const action = WorkerAction.None;
        const stateRise = WorkerState.Completed;
        const priorTask = createTask(action, WorkerState.Running);
        const posteriorTask = createTask(action, WorkerState.Running);
        const prior = createDocument([priorTask]);
        const posterior = createDocument([posteriorTask]);
        const result = backgroundTasksRisingEdge(action, stateRise, posterior, prior);
        expect(result).toBeFalsy();
    });
    it("should return false if the task is blocked by a blocking state", () => {
        const action = WorkerAction.None;
        const stateRise = WorkerState.Completed;
        const blockingState = WorkerState.Failed;
        const priorTask = createTask(action, blockingState);
        const posteriorTask = createTask(action, stateRise);
        const prior = createDocument([priorTask]);
        const posterior = createDocument([posteriorTask]);
        const result = backgroundTasksRisingEdge(action, stateRise, posterior, prior, [blockingState]);
        expect(result).toBeFalsy();
    });
    it("should return false if onlyOnce is true and the task has already transitioned", () => {
        const action = WorkerAction.None;
        const stateRise = WorkerState.Completed;
        const priorTask = createTask(action, stateRise);
        const posteriorTask = createTask(action, stateRise);
        const prior = createDocument([priorTask]);
        const posterior = createDocument([posteriorTask]);
        const result = backgroundTasksRisingEdge(action, stateRise, posterior, prior, [], true);
        expect(result).toBeFalsy();
    });
    it("should return true if onlyOnce is false and the task has transitioned", () => {
        const action = WorkerAction.None;
        const stateRise = WorkerState.Completed;
        const priorTask = createTask(action, WorkerState.Running);
        const posteriorTask = createTask(action, stateRise);
        const prior = createDocument([priorTask]);
        const posterior = createDocument([posteriorTask]);
        const result = backgroundTasksRisingEdge(action, stateRise, posterior, prior, [], false);
        expect(result).toBeTruthy();
    });
    it("should return true if there are no prior tasks", () => {
        const action = WorkerAction.None;
        const stateRise = WorkerState.Completed;
        const posteriorTask = createTask(action, stateRise);
        const prior = createDocument([]);
        const posterior = createDocument([posteriorTask]);
        const result = backgroundTasksRisingEdge(action, stateRise, posterior, prior);
        expect(result).toBeTruthy();
    });
    it("should return false if there are no posterior tasks", () => {
        const action = WorkerAction.None;
        const stateRise = WorkerState.Completed;
        const priorTask = createTask(action, WorkerState.Running);
        const prior = createDocument([priorTask]);
        const posterior = createDocument([]);
        const result = backgroundTasksRisingEdge(action, stateRise, posterior, prior);
        expect(result).toBeFalsy();
    });
});
describe("getTasksByAction", () => {
    const createTask = (id, action, state) => ({
        id,
        action,
        state,
        name: "",
        progress: 0,
        pipeConfig: "",
        createdAt: undefined,
    });
    const createDocument = (tasks) => ({
        backgroundTasks: tasks,
        id: "",
    });
    it("should return tasks filtered by the specified action", () => {
        const action = WorkerAction.Render;
        const task1 = createTask("1", action, WorkerState.Pending);
        const task2 = createTask("2", WorkerAction.Transcode, WorkerState.Running);
        const task3 = createTask("3", action, WorkerState.Completed);
        const document = createDocument([task1, task2, task3]);
        const result = getTasksByAction(action, document);
        expect(result).toHaveLength(2);
        expect(result).toContain(task1);
        expect(result).toContain(task3);
    });
    it("should return an empty array if no tasks match the specified action", () => {
        const action = WorkerAction.Render;
        const task1 = createTask("1", WorkerAction.Transcode, WorkerState.Pending);
        const task2 = createTask("2", WorkerAction.Transcode, WorkerState.Running);
        const document = createDocument([task1, task2]);
        const result = getTasksByAction(action, document);
        expect(result).toHaveLength(0);
    });
    it("should sort tasks with in-progress states first", () => {
        const action = WorkerAction.Render;
        const task1 = createTask("1", action, WorkerState.Completed);
        const task2 = createTask("2", action, WorkerState.Running);
        const task3 = createTask("3", action, WorkerState.Pending);
        const document = createDocument([task1, task2, task3]);
        const result = getTasksByAction(action, document);
        expect(result).toHaveLength(3);
        expect(result[0]).toBe(task2);
        expect(result[1]).toBe(task3);
        expect(result[2]).toBe(task1);
    });
    it("should return an empty array if actualData is undefined", () => {
        const action = WorkerAction.Render;
        const result = getTasksByAction(action, undefined);
        expect(result).toHaveLength(0);
    });
    it("should return an empty array if actualData has no backgroundTasks", () => {
        const action = WorkerAction.Render;
        const document = createDocument([]);
        const result = getTasksByAction(action, document);
        expect(result).toHaveLength(0);
    });
});
//# sourceMappingURL=ICollectionDocument.spec.js.map