var _a;
/* eslint-disable default-case */
/* eslint-disable camelcase */
/* eslint-disable capitalized-comments */
import { vec2, vec3, mat3 } from "gl-matrix";
import { immerable } from "immer";
import { AnchorType, Aspect } from "../../enums/index.js";
/**
 * Represents a 2D box model with position, size, anchor properties.
 * The Coordinate system is based on x=center y=center of the box.
 * The interval of the local space is -1 to 1 with respect to the size of the box.
 * The box model can be nested within a parent box model, allowing for hierarchical transformations.
 *                         y
 *                         ↑
 *                         |
 *                    +----+----+
 *                    |    |    |
 *                    |    +----+-----→ x
 *                    |         |
 *                    +---------+
 **/
export class Box {
    get positionTranslationMatrix() {
        const translationMatrix = mat3.fromTranslation(mat3.create(), this.position);
        return translationMatrix;
    }
    get rotate() {
        return this.rotation > 0 || this.rotation < 0;
    }
    get rotationMatrix() {
        const rotationRad = (this.rotation * Math.PI) / 180;
        // Create rotation matrix around this._position
        const translationMatrix = mat3.fromTranslation(mat3.create(), vec2.negate(vec2.create(), this.position));
        const rotationMatrix = mat3.fromRotation(mat3.create(), rotationRad);
        const reverseTranslationMatrix = mat3.fromTranslation(mat3.create(), this.position);
        // Combine transformations: M = T2 * R * T1
        const matrix = mat3.create();
        mat3.multiply(matrix, rotationMatrix, translationMatrix);
        mat3.multiply(matrix, reverseTranslationMatrix, matrix);
        return matrix;
    }
    get scaleToParentSpaceMatrix() {
        var _b;
        const parentSize = ((_b = this.parent) === null || _b === void 0 ? void 0 : _b.size) || Box.unitSize;
        const scaleSize = vec2.divide(vec2.create(), this.size, parentSize);
        // Create scaling matrix
        return mat3.fromScaling(mat3.create(), scaleSize);
    }
    get sizeInParentSpace() {
        return this.scaleToParentSpace(vec2.fromValues(1, 1));
    }
    constructor(options) {
        var _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
        this[_a] = true; // Option 1
        /**
         * Represents the scaling effect applied to the box model.
         *
         * @type {Aspect}
         * @default Aspect.None
         */
        this.scalingEffect = Aspect.None;
        this.parent = options === null || options === void 0 ? void 0 : options.parent;
        this.position = [(_c = (_b = options === null || options === void 0 ? void 0 : options.position) === null || _b === void 0 ? void 0 : _b[0]) !== null && _c !== void 0 ? _c : 0, (_e = (_d = options === null || options === void 0 ? void 0 : options.position) === null || _d === void 0 ? void 0 : _d[1]) !== null && _e !== void 0 ? _e : 0];
        this.size = [(_g = (_f = options === null || options === void 0 ? void 0 : options.size) === null || _f === void 0 ? void 0 : _f[0]) !== null && _g !== void 0 ? _g : 1, (_j = (_h = options === null || options === void 0 ? void 0 : options.size) === null || _h === void 0 ? void 0 : _h[1]) !== null && _j !== void 0 ? _j : 1];
        this.anchor = [(_l = (_k = options === null || options === void 0 ? void 0 : options.anchor) === null || _k === void 0 ? void 0 : _k[0]) !== null && _l !== void 0 ? _l : 0, (_o = (_m = options === null || options === void 0 ? void 0 : options.anchor) === null || _m === void 0 ? void 0 : _m[1]) !== null && _o !== void 0 ? _o : 0];
        this.rotation = (options === null || options === void 0 ? void 0 : options.rotation) || 0;
        if (options === null || options === void 0 ? void 0 : options.ratio) {
            this.ratio = [options.ratio[0], options.ratio[1]];
        }
        else {
            this.ratio = reduceFraction(this.size[0], this.size[1]);
        }
        this.zIndex = options === null || options === void 0 ? void 0 : options.zIndex;
        this.scalingEffect = (options === null || options === void 0 ? void 0 : options.scalingEffect) || Aspect.None;
    }
    /**
     * Scales a given position vector from local space (-0.5 to 0.5) to parent space.
     *
     * @param A - The position vector in local space.
     * @returns The scaled position vector in parent space.
     */
    // scaleToParentSpace(A: vec2): vec2 {
    //   const parentSize = this.parent?._size || Box.unitSize;
    //   const scaleSize = vec2.divide(vec2.create(), this._size, parentSize);
    //   console.log("scaleSize", scaleSize);
    //   const scaledA = vec2.multiply(vec2.create(), A, scaleSize);
    //   return scaledA;
    // }
    scaleToParentSpace(A) {
        const scalingMatrix = this.scaleToParentSpaceMatrix;
        const A3 = vec3.fromValues(A[0], A[1], 1);
        const scaledA3 = vec3.transformMat3(vec3.create(), A3, scalingMatrix);
        return vec2.fromValues(scaledA3[0], scaledA3[1]);
    }
    /**
     * Translates a given vector to the parent's coordinate system.
     *
     * @param A - The vector to be translated.
     * @returns The translated vector in the parent's coordinate system.
     */
    // translateToParent(A: vec2): vec2 {
    //   const translatedA = vec2.add(vec2.create(), this._position, A);
    //   return translatedA;
    // }
    translateToParent(A) {
        const A3 = vec3.fromValues(A[0], A[1], 1);
        const translatedA3 = vec3.transformMat3(vec3.create(), A3, this.positionTranslationMatrix);
        return vec2.fromValues(translatedA3[0], translatedA3[1]);
    }
    /**
     * Rotates a given vector `A` to the parent's coordinate system based on the current object's rotation and position.
     *
     * @param A - The vector to be rotated.
     * @returns The rotated vector in the parent's coordinate system.
     */
    // rotateToParent(A: vec2): vec2 {
    //   const rotationRad = (this.rotation * Math.PI) / 180;
    //   // console.log("rotate", A, this.position, rotationRad);
    //   return vec2.rotate(vec2.create(), A, this._position, rotationRad);
    // }
    rotateToParent(A) {
        const A3 = vec3.fromValues(A[0], A[1], 1);
        const rotatedA3 = vec3.transformMat3(vec3.create(), A3, this.rotationMatrix);
        return vec2.fromValues(rotatedA3[0], rotatedA3[1]);
    }
    /**
     * Projects a given point `A` to the parent space of the current box model.
     *
     * This method performs the following transformations in sequence:
     * 1. Translates the point `A` with respect to the anchor of the current box.
     * 2. Scales the translated point to the parent space.
     * 3. Translates the scaled point to the parent space.
     * 4. Rotates the translated point to the parent space.
     *
     * @param A - The point to be projected, represented as a `vec2`.
     * @returns The projected point in the parent space, represented as a `vec2`.
     */
    projectToParentSpace(A) {
        // console.log("A", A);
        const wrtAncor = vec2.sub(vec2.create(), A, this.anchor);
        // console.log("wrtAncor", wrtAncor);
        const scaledA = this.scaleToParentSpace(wrtAncor);
        // console.log("scaledA", scaledA);
        const translatedA = this.translateToParent(scaledA);
        // console.log("translatedA", translatedA);
        const rotatedA = this.rotateToParent(translatedA);
        // console.log("rotatedA", rotatedA);
        return rotatedA;
    }
    /**
     * Adjusts the size of the current box to fit within its parent's dimensions while maintaining the aspect ratio.
     *
     * This method scales the box's size based on the smaller scaling factor between the width and height ratios.
     * If the parent or ratio properties are not defined, the method will return without making any changes.
     *
     * @remarks
     * - The `parent` property should have a `size` attribute representing the dimensions of the parent container.
     * - The `ratio` property should be a tuple representing the aspect ratio of the box.
     * - The `size` property will be scaled in place to fit within the parent's dimensions.
     */
    fit() {
        if (!this.parent)
            return;
        const parentSize = this.parent.size;
        const scaleX = parentSize[0] / this.ratio[0];
        const scaleY = parentSize[1] / this.ratio[1];
        const scale = Math.min(scaleX, scaleY);
        vec2.scale(this.size, this.ratio, scale);
    }
    /**
     * Adjusts the size of the box to fill its parent container while maintaining the aspect ratio.
     *
     * This method scales the box's size based on the ratio and the size of its parent container.
     * It ensures that the box fills the parent container as much as possible without distorting
     * the aspect ratio.
     *
     * @remarks
     * - If the box does not have a parent or a defined ratio, the method returns immediately.
     * - The scaling factor is determined by the larger of the two ratios (width or height).
     */
    fill() {
        if (!this.parent)
            return;
        const parentSize = this.parent.size;
        const scaleX = parentSize[0] / this.ratio[0];
        const scaleY = parentSize[1] / this.ratio[1];
        const scale = Math.max(scaleX, scaleY);
        vec2.scale(this.size, this.ratio, scale);
    }
    /**
     * Converts the current Box instance to a BoxData object.
     *
     * @returns {BoxData} The BoxData representation of the current Box instance.
     */
    toData() {
        return {
            position: [this.position[0], this.position[1]],
            size: [this.size[0], this.size[1]],
            anchor: [this.anchor[0], this.anchor[1]],
            ratio: [this.ratio[0], this.ratio[1]],
            rotation: this.rotation,
            scalingEffect: this.scalingEffect,
            zIndex: this.zIndex,
        };
    }
    /**
     * Inverts the Y-coordinate of a given 2D vector.
     *
     * @param A - The input vector to be inverted.
     * @returns A new vector with the Y-coordinate inverted.
     */
    static invertY(A) {
        return vec2.multiply(vec2.create(), A, vec2.fromValues(1, -1));
    }
    /**
     * Converts a vector from a center-origin coordinate system to a top-left-origin coordinate system.
     *
     * @param A - The vector in the center-origin coordinate system.
     * @returns The vector in the top-left-origin coordinate system.
     */
    static toPixelCoordinateSystem(A) {
        // convert from center origin to top left origin
        const translated = vec2.add(vec2.create(), A, Box.topLeftToOrigin);
        const inverted = Box.invertY(translated);
        return inverted;
    }
    /**
     * Creates and returns a new instance of the `Box` class.
     *
     * @returns {Box} A new `Box` instance.
     */
    static default() {
        return new Box();
    }
    /**
     * Creates a new `Box` instance from the provided data.
     *
     * @param data - The data to create the `Box` from.
     * @param parent - An optional parent `Box`.
     * @returns A new `Box` instance.
     */
    static fromData(data, parent) {
        return new Box({
            parent,
            position: data.position,
            size: data.size,
            anchor: data.anchor,
            rotation: data.rotation,
            ratio: data.ratio,
            scalingEffect: data.scalingEffect,
            zIndex: data.zIndex,
        });
    }
    /**
     * Creates a new `Box` instance from the provided image settings.
     *
     * @param settings - The image settings to create the `Box` from.
     * @param parent - Optional parent image settings to create a parent `Box`.
     * @param scalingEffect - Optional scaling effect to apply. Defaults to `Aspect.Fit`.
     * @returns A new `Box` instance configured with the provided settings.
     */
    static fromIImageSettings(settings, parent, scalingEffect) {
        const result = new Box({
            size: [settings.width, settings.height],
            ratio: settings.ratioFraction,
            scalingEffect: scalingEffect !== null && scalingEffect !== void 0 ? scalingEffect : Aspect.Fit,
        });
        if (parent) {
            result.parent = new Box({
                size: [parent.width, parent.height],
                ratio: parent.ratioFraction,
            });
        }
        return result;
    }
}
_a = immerable;
Box.unitSize = [1, 1];
Box.originToTopLeft = [-0.5, 0.5];
Box.topLeftToOrigin = [0.5, -0.5];
Box.originToTopRight = [0.5, 0.5];
Box.originToBottomLeft = [-0.5, -0.5];
Box.originToBottomRight = [0.5, -0.5];
/**
 * Reduces a fraction to its simplest form.
 *
 * @param numerator - The numerator of the fraction. If not provided, defaults to 0.
 * @param denominator - The denominator of the fraction. If not provided, defaults to 1.
 * @returns A tuple containing the simplified numerator and denominator.
 */
export function reduceFraction(numerator, denominator) {
    if (!numerator || !denominator)
        return [0, 1];
    if (denominator === 0) {
        throw new Error("Denominator cannot be zero");
    }
    numerator = Math.abs(numerator);
    denominator = Math.abs(denominator);
    const commonDivisor = greatestCommonDivisor(numerator, denominator);
    return [numerator / commonDivisor, denominator / commonDivisor];
}
/**
 * Calculates the greatest common divisor (GCD) of two numbers using the Euclidean algorithm.
 *
 * @param a - The first number.
 * @param b - The second number.
 * @returns The greatest common divisor of the two numbers.
 */
export function greatestCommonDivisor(a, b) {
    a = Math.abs(a);
    b = Math.abs(b);
    while (b) {
        const t = b;
        b = a % b;
        a = t;
    }
    return a;
}
export function parseAnchorType(anchorX, anchorY) {
    let x = 0;
    let y = 0;
    switch (anchorX) {
        case AnchorType.Left:
            x = -0.5;
            break;
        case AnchorType.Center:
            x = 0;
            break;
        case AnchorType.Right:
            x = 0.5;
            break;
    }
    switch (anchorY) {
        case AnchorType.Top:
            y = 0.5;
            break;
        case AnchorType.Center:
            y = 0;
            break;
        case AnchorType.Bottom:
            y = -0.5;
            break;
    }
    return [x, y];
}
export function toAnchorType(A) {
    let anchorX;
    let anchorY;
    if (A[0] === -0.5) {
        anchorX = AnchorType.Left;
    }
    else if (A[0] === 0.5) {
        anchorX = AnchorType.Right;
    }
    else {
        anchorX = AnchorType.Center;
    }
    if (A[1] === -0.5) {
        anchorY = AnchorType.Bottom;
    }
    else if (A[1] === 0.5) {
        anchorY = AnchorType.Top;
    }
    else {
        anchorY = AnchorType.Center;
    }
    return [anchorX, anchorY];
}
//# sourceMappingURL=box.js.map