import { describe, it, expect, beforeEach } from "vitest";
import { spec } from "../../data";
import { SelectorType } from "../../enums/index.js";
import { getVideoStripLength, calculateClips, calculateClipSequence, } from "./video-strip";
import { EditSpecification } from "../editSpecification";
import { FrameRange } from "../frameRange";
describe("video-strip", () => {
    let editSpecificationJson;
    let editSpecification;
    let videoSettings;
    const selectors = {
        [SelectorType.IsPlayer]: false,
        [SelectorType.CalculateClips]: true,
        [SelectorType.CalculateFromJson]: false,
        [SelectorType.Language]: "de",
    };
    beforeEach(() => {
        console.log(FrameRange.fromIFrameRange);
        editSpecificationJson = JSON.stringify(spec);
        editSpecification = EditSpecification.fromJSON(editSpecificationJson);
        videoSettings = editSpecification.videoSettings;
        if (!editSpecification.selectors)
            editSpecification.selectors = selectors;
    });
    describe("getVideoStripLength", () => {
        it("should return the length of the video-strip in frames", () => {
            const videoStripLength = getVideoStripLength(editSpecification, videoSettings);
            expect(videoStripLength).toBe(100);
        });
    });
    describe("calculateClips", () => {
        it("should calculate from JSON and return an array of clips", () => {
            const videoStripProps = {
                editSpecificationJson,
                editSpecification,
                videoSettings,
                clips: [],
            };
            const clips = calculateClips(videoStripProps);
            expect(clips).toHaveLength(1);
        });
        it("should calculate from editSpecification and return an array of clips", () => {
            const videoStripProps = {
                editSpecificationJson,
                editSpecification,
                videoSettings,
                clips: [],
            };
            const clips = calculateClips(videoStripProps);
            expect(clips).toHaveLength(1);
        });
    });
    describe("calculateClipSequence", () => {
        it("should return correct message when clips is null", () => {
            // Arrange
            const videoStripProps = {
                editSpecificationJson,
                editSpecification,
                videoSettings,
                clips: [],
            };
            const clips = calculateClips(videoStripProps);
            const selectors = {}; // Assuming an empty object for selectors as it's not the focus of this test
            // Act
            const result = calculateClipSequence(clips, selectors);
            // Assert
            expect(result).toHaveLength(1);
            expect(result[0].key).toBe("ClipSequence-0");
            expect(result[0].props.clip).toBeTruthy();
        });
        it("should return correct message when clips is null", () => {
            // Arrange
            const clips = null;
            const selectors = {}; // Assuming an empty object for selectors as it's not the focus of this test
            // Act
            const result = calculateClipSequence(clips, selectors);
            // Assert
            expect(result).toHaveLength(1);
            expect(result[0].props.children).toBe('"clips[] is null"');
        });
        it("should return correct message when clips is not an Array", () => {
            // Arrange
            const clips = {};
            const selectors = {}; // Assuming an empty object for selectors as it's not the focus of this test
            // Act
            const result = calculateClipSequence(clips, selectors);
            // Assert
            expect(result).toHaveLength(1);
            expect(result[0].props.children).toBe('"clips[] is not an Array"');
        });
    });
});
//# sourceMappingURL=video-strip.spec.js.map