import { describe, it, expect } from "vitest";
import { TimingAction, Anchor } from "../enums/index.js";
import { FrameRange } from "./frameRange.js"; // Adjust the import based on your file structure
describe("FrameRange", () => {
    describe("constructor", () => {
        it("should initialize with default values", () => {
            const frameRange = new FrameRange();
            expect(frameRange.start).toBe(0);
            expect(frameRange.end).toBe(1);
            expect(frameRange.anchor).toBeNull();
        });
        it("should initialize with provided values", () => {
            const frameRange = new FrameRange(5, 10, Anchor.Center);
            expect(frameRange.start).toBe(5);
            expect(frameRange.end).toBe(10);
            expect(frameRange.anchor).toBe(Anchor.Center);
        });
        it("should adjust end value if it is less than start", () => {
            const frameRange = new FrameRange(10, 5);
            expect(frameRange.start).toBe(10);
            expect(frameRange.end).toBeGreaterThan(10);
        });
    });
    describe("getters", () => {
        it("should correctly calculate center", () => {
            const frameRange = new FrameRange(2, 4);
            expect(frameRange.center).toBe(3);
        });
        it("should correctly calculate duration", () => {
            const frameRange = new FrameRange(1, 5);
            expect(frameRange.duration).toBe(4);
        });
    });
    describe("FrameRange.constructor", () => {
        it("should initialize with default values when no arguments are provided", () => {
            const frameRange = new FrameRange();
            expect(frameRange.start).toEqual(0);
            expect(frameRange.end).toEqual(1);
            expect(frameRange.anchor).toBeNull();
        });
        it("should initialize with provided start and end values", () => {
            const start = 5;
            const end = 10;
            const frameRange = new FrameRange(start, end);
            expect(frameRange.start).toEqual(start);
            expect(frameRange.end).toEqual(end);
            expect(frameRange.anchor).toBeNull();
        });
        it("should initialize with provided start, end, and anchor values", () => {
            const start = 5;
            const end = 10;
            const anchor = Anchor.Start;
            const frameRange = new FrameRange(start, end, anchor);
            expect(frameRange.start).toEqual(start);
            expect(frameRange.end).toEqual(end);
            expect(frameRange.anchor).toEqual(anchor);
        });
        it("should set end to start + 1 when end is not provided", () => {
            const start = 5;
            const frameRange = new FrameRange(start);
            expect(frameRange.start).toEqual(start);
            expect(frameRange.end).toEqual(start + 1);
        });
        it("should set end to start + 1 when end is less than start", () => {
            const start = 5;
            const end = 3;
            const frameRange = new FrameRange(start, end);
            expect(frameRange.start).toEqual(start);
            expect(frameRange.end).toEqual(start + 1);
        });
        it("should handle null values for start and end", () => {
            const frameRange = new FrameRange(undefined, undefined);
            expect(frameRange.start).toEqual(0);
            expect(frameRange.end).toEqual(1);
            expect(frameRange.anchor).toBeNull();
        });
    });
    // #region SUBSET TESTS
    describe("FrameRange.fromMilliseconds", () => {
        const fps = 30; // Example frames per second
        it("should correctly convert time in milliseconds to frame numbers", () => {
            const startInMilliseconds = 1000; // 1 second
            const endInMilliseconds = 2000; // 2 seconds
            const startInFrames = Math.round((startInMilliseconds / 1000) * fps);
            const endInFrames = Math.round((endInMilliseconds / 1000) * fps);
            const result = FrameRange.fromMilliseconds(startInMilliseconds, endInMilliseconds, fps);
            expect(result.start).toEqual(startInFrames);
            expect(result.end).toEqual(endInFrames);
        });
        it("should handle zero milliseconds correctly", () => {
            const result = FrameRange.fromMilliseconds(0, 0, fps);
            expect(result.start).toEqual(0);
            expect(result.end).toEqual(1);
        });
        it("should handle non-integer milliseconds correctly", () => {
            const startInMilliseconds = 1500; // 1.5 seconds
            const endInMilliseconds = 2500; // 2.5 seconds
            const startInFrames = Math.round((startInMilliseconds / 1000) * fps);
            const endInFrames = Math.round((endInMilliseconds / 1000) * fps);
            const result = FrameRange.fromMilliseconds(startInMilliseconds, endInMilliseconds, fps);
            expect(result.start).toEqual(startInFrames);
            expect(result.end).toEqual(endInFrames);
        });
        it("should apply the specified anchor", () => {
            const anchor = Anchor.Start;
            const result = FrameRange.fromMilliseconds(1000, 2000, fps, anchor);
            expect(result.anchor).toEqual(anchor);
        });
        it("should handle negative milliseconds correctly", () => {
            const startInMilliseconds = -1000; // -1 second
            const endInMilliseconds = 1000; // 1 second
            const startInFrames = Math.round((startInMilliseconds / 1000) * fps);
            const endInFrames = Math.round((endInMilliseconds / 1000) * fps);
            const result = FrameRange.fromMilliseconds(startInMilliseconds, endInMilliseconds, fps);
            expect(result.start).toEqual(startInFrames);
            expect(result.end).toEqual(endInFrames);
        });
    });
    describe("FrameRange.fromSeconds", () => {
        const fps = 30; // Example frames per second
        it("should correctly convert time in seconds to frame numbers", () => {
            const startInSeconds = 1;
            const endInSeconds = 2;
            const startInFrames = (Math.round(startInSeconds * 1000) * fps) / 1000;
            const endInFrames = (Math.round(endInSeconds * 1000) * fps) / 1000;
            const result = FrameRange.fromSeconds(startInSeconds, endInSeconds, fps);
            expect(result.start).toEqual(startInFrames);
            expect(result.end).toEqual(endInFrames);
        });
        it("should handle zero seconds correctly", () => {
            const result = FrameRange.fromSeconds(0, 0, fps);
            expect(result.start).toEqual(0);
            expect(result.end).toEqual(1);
        });
        it("should handle non-integer seconds correctly", () => {
            const startInSeconds = 1.5;
            const endInSeconds = 2.5;
            const startInFrames = (Math.round(startInSeconds * 1000) * fps) / 1000;
            const endInFrames = (Math.round(endInSeconds * 1000) * fps) / 1000;
            const result = FrameRange.fromSeconds(startInSeconds, endInSeconds, fps);
            expect(result.start).toEqual(startInFrames);
            expect(result.end).toEqual(endInFrames);
        });
        it("should apply the specified anchor", () => {
            const anchor = Anchor.Start;
            const result = FrameRange.fromSeconds(1, 2, fps, anchor);
            expect(result.anchor).toEqual(anchor);
        });
        it("should handle negative seconds correctly", () => {
            const startInSeconds = -1;
            const endInSeconds = 1;
            const startInFrames = (Math.round(startInSeconds * 1000) * fps) / 1000;
            const endInFrames = (Math.round(endInSeconds * 1000) * fps) / 1000;
            const result = FrameRange.fromSeconds(startInSeconds, endInSeconds, fps);
            expect(result.start).toEqual(startInFrames);
            expect(result.end).toEqual(endInFrames);
        });
    });
    describe("FrameRange.aAndNotB", () => {
        it("should return the original frame when there is no overlap", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(20, 30);
            const expected = [a];
            const result = a.aAndNotB(b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle a partially overlapping frame", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(5, 15);
            const expected = [new FrameRange(0, 5)];
            const result = a.aAndNotB(b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should return an empty array for full overlap", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(0, 10);
            const expected = [];
            const result = a.aAndNotB(b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle the case where 'b' is inside 'a'", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(2, 8);
            const expected = [new FrameRange(0, 2), new FrameRange(8, 10)];
            const result = a.aAndNotB(b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
    });
    describe("FrameRange.aIntersectB", () => {
        it("should return the intersection of two overlapping frames", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(5, 15);
            const expected = new FrameRange(5, 10);
            const result = a.aIntersectB(b);
            expect(FrameRange.areEqual(result, expected)).toBeTruthy();
        });
        it("should return the intersection of two overlapping frames flipped", () => {
            const b = new FrameRange(0, 10);
            const a = new FrameRange(5, 15);
            const expected = new FrameRange(5, 10);
            const result = a.aIntersectB(b);
            expect(FrameRange.areEqual(result, expected)).toBeTruthy();
        });
        it("should return an empty frame when there is no overlap", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(20, 30);
            const expected = FrameRange.empty();
            const result = a.aIntersectB(b);
            expect(FrameRange.areEqual(result, expected)).toBeTruthy();
        });
        it("should handle the case where one frame is completely within the other", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(2, 8);
            const expected = new FrameRange(2, 8);
            const result = a.aIntersectB(b);
            expect(FrameRange.areEqual(result, expected)).toBeTruthy();
        });
        it("should return an empty frame when frames are adjacent but not overlapping", () => {
            const a = new FrameRange(0, 5);
            const b = new FrameRange(5, 10);
            const expected = FrameRange.empty();
            const result = a.aIntersectB(b);
            expect(FrameRange.areEqual(result, expected)).toBeTruthy();
        });
    });
    describe("FrameRange.aXorB", () => {
        it("should return the symmetric difference of two overlapping frames", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(5, 15);
            const expected = [new FrameRange(0, 5), new FrameRange(10, 15)];
            const result = a.aXorB(b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should return both frames when there is no overlap", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(20, 30);
            const expected = [a, b];
            const result = a.aXorB(b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should return an empty array when frames are identical", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(0, 10);
            const expected = [];
            const result = a.aXorB(b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle the case where one frame is completely within the other", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(2, 8);
            const expected = [new FrameRange(0, 2), new FrameRange(8, 10)];
            const result = a.aXorB(b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should return both frames when they are adjacent but not overlapping", () => {
            const a = new FrameRange(0, 5);
            const b = new FrameRange(5, 10);
            const expected = [a, b];
            const result = a.aXorB(b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
    });
    describe("FrameRange.union", () => {
        // Testing when 'b' is a single FrameRange
        describe("when 'b' is a FrameRange", () => {
            it("should return a single frame when frames are overlapping", () => {
                const a = new FrameRange(0, 10);
                const b = new FrameRange(5, 15);
                const expected = [new FrameRange(0, 15)];
                const result = a.union(b);
                expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
            });
            it("should return two separate frames when frames are not overlapping", () => {
                const a = new FrameRange(0, 5);
                const b = new FrameRange(10, 15);
                const expected = [a.clone(), b.clone()];
                const result = a.union(b);
                expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
            });
        });
        // Testing when 'b' is an array of FrameRange
        describe("when 'b' is an array of FrameRange", () => {
            it("should return the correct union for multiple overlapping frames", () => {
                const a = new FrameRange(0, 10);
                const b = [new FrameRange(5, 15), new FrameRange(20, 25)];
                const expected = [new FrameRange(0, 15), new FrameRange(20, 25)];
                const result = a.union(b);
                expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
            });
            it("should return the correct union for non-overlapping frames", () => {
                const a = new FrameRange(0, 5);
                const b = [new FrameRange(10, 15), new FrameRange(20, 25)];
                const expected = [
                    a.clone(),
                    new FrameRange(10, 15),
                    new FrameRange(20, 25),
                ];
                const result = a.union(b);
                expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
            });
        });
    });
    describe("FrameRange.unionOfAll", () => {
        it("should return the correct union for multiple overlapping frames", () => {
            const ranges = [
                new FrameRange(0, 10),
                new FrameRange(5, 15),
                new FrameRange(12, 20),
            ];
            const expected = [new FrameRange(0, 20)];
            const result = FrameRange.unionOfAll(ranges);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should return the correct union for non-overlapping frames", () => {
            const ranges = [
                new FrameRange(0, 5),
                new FrameRange(10, 15),
                new FrameRange(20, 25),
            ];
            const expected = [
                new FrameRange(0, 5),
                new FrameRange(10, 15),
                new FrameRange(20, 25),
            ];
            const result = FrameRange.unionOfAll(ranges);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle an array with a single frame", () => {
            const ranges = [new FrameRange(0, 10)];
            const expected = [new FrameRange(0, 10)];
            const result = FrameRange.unionOfAll(ranges);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should eliminate duplicate frames", () => {
            const ranges = [new FrameRange(0, 10), new FrameRange(0, 10)];
            const expected = [new FrameRange(0, 10)];
            const result = FrameRange.unionOfAll(ranges);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should eliminate frames that are inside other frames", () => {
            const ranges = [new FrameRange(0, 15), new FrameRange(5, 10)];
            const expected = [new FrameRange(0, 15)];
            const result = FrameRange.unionOfAll(ranges);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
    });
    describe("FrameRange.differences", () => {
        it("should return the original frame when there is no overlap", () => {
            const a = new FrameRange(0, 10);
            const bItems = [new FrameRange(20, 30)];
            const expected = [a];
            const result = FrameRange.differences(a, bItems);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle a single partially overlapping frame", () => {
            const a = new FrameRange(0, 10);
            const bItems = [new FrameRange(5, 15)];
            const expected = [new FrameRange(0, 5)];
            const result = FrameRange.differences(a, bItems);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should return an empty array for full overlap", () => {
            const a = new FrameRange(0, 10);
            const bItems = [new FrameRange(0, 10)];
            const expected = [];
            const result = FrameRange.differences(a, bItems);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle multiple overlapping frames", () => {
            const a = new FrameRange(0, 10);
            const bItems = [new FrameRange(2, 4), new FrameRange(6, 8)];
            const expected = [
                new FrameRange(0, 2),
                new FrameRange(4, 6),
                new FrameRange(8, 10),
            ];
            const result = FrameRange.differences(a, bItems);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle non-contiguous overlapping frames", () => {
            const a = new FrameRange(0, 15);
            const bItems = [new FrameRange(3, 6), new FrameRange(10, 12)];
            const expected = [
                new FrameRange(0, 3),
                new FrameRange(6, 10),
                new FrameRange(12, 15),
            ];
            const result = FrameRange.differences(a, bItems);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle overlapping frames that exceed 'a'", () => {
            const a = new FrameRange(5, 15);
            const bItems = [new FrameRange(0, 10), new FrameRange(10, 20)];
            const expected = [];
            const result = FrameRange.differences(a, bItems);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle an empty bItems array", () => {
            const a = new FrameRange(0, 10);
            const bItems = [];
            const expected = [a];
            const result = FrameRange.differences(a, bItems);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle a case where 'a' is completely within a range in bItems", () => {
            const a = new FrameRange(5, 15);
            const bItems = [new FrameRange(0, 20)];
            const expected = [];
            const result = FrameRange.differences(a, bItems);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle overlapping frames at the edges of 'a'", () => {
            const a = new FrameRange(0, 10);
            const bItems = [new FrameRange(0, 2), new FrameRange(8, 12)];
            const expected = [new FrameRange(2, 8)];
            const result = FrameRange.differences(a, bItems);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle non-overlapping frames within 'a'", () => {
            const a = new FrameRange(0, 20);
            const bItems = [new FrameRange(5, 8), new FrameRange(12, 15)];
            const expected = [
                new FrameRange(0, 5),
                new FrameRange(8, 12),
                new FrameRange(15, 20),
            ];
            const result = FrameRange.differences(a, bItems);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
    });
    describe("FrameRange.difference", () => {
        it("should return the original frame when there is no overlap", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(20, 30);
            const expected = [a];
            const result = FrameRange.difference(a, b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle a partially overlapping frame", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(5, 15);
            const expected = [new FrameRange(0, 5)];
            const result = FrameRange.difference(a, b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should return an empty array for full overlap", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(0, 10);
            const expected = [];
            const result = FrameRange.difference(a, b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle a frame that fully contains the other", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(2, 8);
            const expected = [new FrameRange(0, 2), new FrameRange(8, 10)];
            const result = FrameRange.difference(a, b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle a frame that starts before and ends within the other", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(-5, 5);
            const expected = [new FrameRange(5, 10)];
            const result = FrameRange.difference(a, b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle a frame that starts within and ends after the other", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(5, 15);
            const expected = [new FrameRange(0, 5)];
            const result = FrameRange.difference(a, b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should return the original frame when the second frame is after the first", () => {
            const a = new FrameRange(0, 10);
            const b = new FrameRange(10, 20);
            const expected = [a];
            const result = FrameRange.difference(a, b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should return the original frame when the second frame is before the first", () => {
            const a = new FrameRange(10, 20);
            const b = new FrameRange(0, 10);
            const expected = [a];
            const result = FrameRange.difference(a, b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should return empty array if a is inside b", () => {
            const a = new FrameRange(10, 20);
            const b = new FrameRange(0, 30);
            const expected = [];
            const result = FrameRange.difference(a, b);
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
    });
    describe("FrameRange.distinctPartition", () => {
        it("should create distinct partitions from a list of segments", () => {
            const segments = [
                { absoluteRange: new FrameRange(0, 5) },
                { absoluteRange: new FrameRange(3, 10) },
                { absoluteRange: new FrameRange(7, 12) },
            ];
            const expected = [
                new FrameRange(0, 3),
                new FrameRange(3, 5),
                new FrameRange(5, 7),
                new FrameRange(7, 10),
                new FrameRange(10, 12),
            ];
            const result = FrameRange.distinctPartition(segments, "absoluteRange");
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle overlapping ranges correctly", () => {
            const segments = [
                { absoluteRange: new FrameRange(0, 10) },
                { absoluteRange: new FrameRange(5, 15) },
            ];
            const expected = [
                new FrameRange(0, 5),
                new FrameRange(5, 10),
                new FrameRange(10, 15),
            ];
            const result = FrameRange.distinctPartition(segments, "absoluteRange");
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should return an empty array when segments are empty", () => {
            const segments = [];
            const expected = [];
            const result = FrameRange.distinctPartition(segments, "absoluteRange");
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle non-overlapping ranges", () => {
            const segments = [
                { absoluteRange: new FrameRange(0, 3) },
                { absoluteRange: new FrameRange(5, 8) },
            ];
            const expected = [new FrameRange(0, 3), new FrameRange(5, 8)];
            const result = FrameRange.distinctPartition(segments, "absoluteRange");
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle a single segment", () => {
            const segments = [{ absoluteRange: new FrameRange(0, 5) }];
            const expected = [new FrameRange(0, 5)];
            const result = FrameRange.distinctPartition(segments, "absoluteRange");
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
        it("should handle segments with identical ranges", () => {
            const segments = [
                { absoluteRange: new FrameRange(0, 5) },
                { absoluteRange: new FrameRange(0, 5) },
            ];
            const expected = [new FrameRange(0, 5)];
            const result = FrameRange.distinctPartition(segments, "absoluteRange");
            expect(FrameRange.arrayEquals(result, expected)).toBeTruthy();
        });
    });
    // #endregion SUBSET TESTS
    // #region DERIVED TESTS
    describe("FrameRange.toIFrameRange", () => {
        it("should correctly convert FrameRange to IFrameRange", () => {
            const frameRange = new FrameRange(5, 10, Anchor.Start);
            const expected = {
                start: 5,
                end: 10,
                anchor: Anchor.Start,
            };
            const result = frameRange.toIFrameRange();
            expect(result).toEqual(expected);
        });
        it("should handle FrameRange without an anchor", () => {
            const frameRange = new FrameRange(0, 5);
            const expected = {
                start: 0,
                end: 5,
                anchor: null,
            };
            const result = frameRange.toIFrameRange();
            expect(result).toEqual(expected);
        });
        it("should handle FrameRange with null anchor", () => {
            const frameRange = new FrameRange(0, 5, null);
            const expected = {
                start: 0,
                end: 5,
                anchor: null,
            };
            const result = frameRange.toIFrameRange();
            expect(result).toEqual(expected);
        });
    });
    describe("FrameRange.withPadding", () => {
        it("should correctly apply equal padding to both start and end", () => {
            const frameRange = new FrameRange(0, 10);
            const padding = 2;
            const expected = new FrameRange(2, 8);
            const result = frameRange.withPadding(padding);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should correctly apply different padding to start and end", () => {
            const frameRange = new FrameRange(0, 10);
            const paddingStart = 1;
            const paddingEnd = 3;
            const expected = new FrameRange(1, 7);
            const result = frameRange.withPadding(paddingStart, paddingEnd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should handle the case where padding exceeds the duration", () => {
            const frameRange = new FrameRange(0, 4);
            const paddingStart = 3;
            const paddingEnd = 3; // Total padding exceeds the duration
            const expected = new FrameRange(1, 3); // Adjusted padding to fit within duration
            const result = frameRange.withPadding(paddingStart, paddingEnd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should return a clone if the duration is less than or equal to 1", () => {
            const frameRange = new FrameRange(0, 1);
            const padding = 1;
            const result = frameRange.withPadding(padding);
            expect(FrameRange.areEqual(result, frameRange)).toBeTruthy();
        });
        it("should use the same padding for end if paddingEnd is not provided", () => {
            const frameRange = new FrameRange(0, 10);
            const padding = 2;
            const expected = new FrameRange(2, 8);
            const result = frameRange.withPadding(padding);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
    });
    describe("FrameRange.withDurationFromStart", () => {
        it("should correctly set the duration from start when limit is true", () => {
            const frameRange = new FrameRange(0, 10);
            const duration = 8;
            const expected = new FrameRange(0, 8); // Limited by duration
            const result = frameRange.withDurationFromStart(duration);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should correctly set the duration from start when limit is false", () => {
            const frameRange = new FrameRange(0, 10);
            const duration = 15;
            const expected = new FrameRange(0, 15); // Not limited, exceeding original end
            const result = frameRange.withDurationFromStart(duration, false);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should not exceed the original end when limit is true", () => {
            const frameRange = new FrameRange(0, 10);
            const duration = 12;
            const expected = new FrameRange(0, 10); // Limited by the original end
            const result = frameRange.withDurationFromStart(duration);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should handle zero duration", () => {
            const frameRange = new FrameRange(2, 10);
            const duration = 0;
            const expected = new FrameRange(2, 2); // Duration set to 0
            const result = frameRange.withDurationFromStart(duration);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
    });
    describe("FrameRange.scale", () => {
        it("should correctly scale the frame range", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 2;
            const expected = new FrameRange(4, 20);
            const result = frameRange.scale(factor);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should handle a scale factor of zero", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 0; // Should be adjusted to Number.EPSILON
            const expected = new FrameRange(2 * Number.EPSILON, 10 * Number.EPSILON);
            const result = frameRange.scale(factor);
            expect(result.start).toBeCloseTo(expected.start);
            expect(result.end).toBeCloseTo(expected.end);
        });
        it("should handle a negative scale factor", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = -2; // Should be adjusted to Number.EPSILON
            const expected = new FrameRange(2 * Number.EPSILON, 10 * Number.EPSILON);
            const result = frameRange.scale(factor);
            expect(result.start).toBeCloseTo(expected.start);
            expect(result.end).toBeCloseTo(expected.end);
        });
        it("should handle a very small scale factor", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 1e-50; // Should be adjusted to Number.EPSILON
            const expected = new FrameRange(2 * Number.EPSILON, 10 * Number.EPSILON);
            const result = frameRange.scale(factor);
            expect(result.start).toBeCloseTo(expected.start);
            expect(result.end).toBeCloseTo(expected.end);
        });
    });
    describe("FrameRange.scaleDuration", () => {
        const factor = 2;
        it("should scale duration from the start by default", () => {
            const frameRange = new FrameRange(2, 10);
            const expected = frameRange.scaleDurationFromStart(factor);
            const result = frameRange.scaleDuration(factor);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should scale duration from the center when anchor is center", () => {
            const frameRange = new FrameRange(2, 10, Anchor.Center);
            const expected = frameRange.scaleDurationFromCenter(factor);
            const result = frameRange.scaleDuration(factor);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should scale duration from the end when anchor is end", () => {
            const frameRange = new FrameRange(2, 10, Anchor.End);
            const expected = frameRange.scaleDurationFromEnd(factor);
            const result = frameRange.scaleDuration(factor);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
    });
    describe("FrameRange.scaleDurationFromStart", () => {
        it("should correctly scale the frame range duration from the start", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 2;
            const expectedEnd = frameRange.start + frameRange.duration * factor;
            const result = frameRange.scaleDurationFromStart(factor);
            expect(result.start).toEqual(frameRange.start);
            expect(result.end).toEqual(expectedEnd);
        });
        it("should handle a scale factor of zero", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 0; // Should be adjusted to Number.EPSILON
            const expectedEnd = frameRange.start + frameRange.duration * Number.EPSILON;
            const result = frameRange.scaleDurationFromStart(factor);
            expect(result.start).toEqual(frameRange.start);
            expect(result.end).toBeCloseTo(expectedEnd);
        });
        it("should handle a negative scale factor", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = -2; // Should be adjusted to Number.EPSILON
            const expectedEnd = frameRange.start + frameRange.duration * Number.EPSILON;
            const result = frameRange.scaleDurationFromStart(factor);
            expect(result.start).toEqual(frameRange.start);
            expect(result.end).toBeCloseTo(expectedEnd);
        });
        it("should handle a very small scale factor", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 1e-50; // Should be adjusted to Number.EPSILON
            const expectedEnd = frameRange.start + frameRange.duration * Number.EPSILON;
            const result = frameRange.scaleDurationFromStart(factor);
            expect(result.start).toEqual(frameRange.start);
            expect(result.end).toBeCloseTo(expectedEnd);
        });
    });
    describe("FrameRange.scaleDurationFromCenter", () => {
        it("should correctly scale the frame range duration from the center", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 2;
            const center = (frameRange.start + frameRange.end) / 2;
            const newHalfDuration = (frameRange.duration / 2) * factor;
            const expectedStart = center - newHalfDuration;
            const expectedEnd = center + newHalfDuration;
            const result = frameRange.scaleDurationFromCenter(factor);
            expect(result.start).toEqual(expectedStart);
            expect(result.end).toEqual(expectedEnd);
        });
        it("should handle a scale factor of zero", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 0; // Should be adjusted to Number.EPSILON
            const center = (frameRange.start + frameRange.end) / 2;
            const newHalfDuration = (frameRange.duration / 2) * Number.EPSILON;
            const expectedStart = center - newHalfDuration;
            const expectedEnd = center + newHalfDuration;
            const result = frameRange.scaleDurationFromCenter(factor);
            expect(result.start).toBeCloseTo(expectedStart);
            expect(result.end).toBeCloseTo(expectedEnd);
        });
        it("should handle a negative scale factor", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = -2; // Should be adjusted to Number.EPSILON
            const center = (frameRange.start + frameRange.end) / 2;
            const newHalfDuration = (frameRange.duration / 2) * Number.EPSILON;
            const expectedStart = center - newHalfDuration;
            const expectedEnd = center + newHalfDuration;
            const result = frameRange.scaleDurationFromCenter(factor);
            expect(result.start).toBeCloseTo(expectedStart);
            expect(result.end).toBeCloseTo(expectedEnd);
        });
        it("should handle a very small scale factor", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 1e-50; // Should be adjusted to Number.EPSILON
            const center = (frameRange.start + frameRange.end) / 2;
            const newHalfDuration = (frameRange.duration / 2) * Number.EPSILON;
            const expectedStart = center - newHalfDuration;
            const expectedEnd = center + newHalfDuration;
            const result = frameRange.scaleDurationFromCenter(factor);
            expect(result.start).toBeCloseTo(expectedStart);
            expect(result.end).toBeCloseTo(expectedEnd);
        });
    });
    describe("FrameRange.scaleDurationFromEnd", () => {
        it("should correctly scale the frame range duration from the end", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 2;
            const expectedDuration = frameRange.duration * factor;
            const expectedStart = frameRange.end - expectedDuration;
            const result = frameRange.scaleDurationFromEnd(factor);
            expect(result.start).toEqual(expectedStart);
            expect(result.end).toEqual(frameRange.end);
        });
        it("should handle a scale factor of zero", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 0; // Should be adjusted to Number.EPSILON
            const expectedDuration = frameRange.duration * Number.EPSILON;
            const expectedStart = frameRange.end - expectedDuration;
            const result = frameRange.scaleDurationFromEnd(factor);
            expect(result.start).toBeCloseTo(expectedStart);
            expect(result.end).toEqual(frameRange.end);
        });
        it("should handle a negative scale factor", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = -2; // Should be adjusted to Number.EPSILON
            const expectedDuration = frameRange.duration * Number.EPSILON;
            const expectedStart = frameRange.end - expectedDuration;
            const result = frameRange.scaleDurationFromEnd(factor);
            expect(result.start).toBeCloseTo(expectedStart);
            expect(result.end).toEqual(frameRange.end);
        });
        it("should handle a very small scale factor", () => {
            const frameRange = new FrameRange(2, 10);
            const factor = 1e-50; // Should be adjusted to Number.EPSILON
            const expectedDuration = frameRange.duration * Number.EPSILON;
            const expectedStart = frameRange.end - expectedDuration;
            const result = frameRange.scaleDurationFromEnd(factor);
            expect(result.start).toBeCloseTo(expectedStart);
            expect(result.end).toEqual(frameRange.end);
        });
    });
    describe("shiftLeft", () => {
        it("should correctly shift the range to the left by a given value", () => {
            // Arrange
            const start = 10;
            const end = 20;
            const shiftValue = 5;
            const expectedStart = 5;
            const expectedEnd = 15;
            const frameRange = new FrameRange(start, end);
            // Act
            const shiftedRange = frameRange.shiftLeft(shiftValue);
            // Assert
            expect(shiftedRange.start).toBe(expectedStart);
            expect(shiftedRange.end).toBe(expectedEnd);
        });
        it("should not change the range when shift value is zero", () => {
            // Arrange
            const start = 10;
            const end = 20;
            const shiftValue = 0;
            const frameRange = new FrameRange(start, end);
            // Act
            const shiftedRange = frameRange.shiftLeft(shiftValue);
            // Assert
            expect(shiftedRange.start).toBe(start);
            expect(shiftedRange.end).toBe(end);
        });
        it("should correctly handle negative shift value", () => {
            // Arrange
            const start = 10;
            const end = 20;
            const shiftValue = -3;
            const expectedStart = 13;
            const expectedEnd = 23;
            const frameRange = new FrameRange(start, end);
            // Act
            const shiftedRange = frameRange.shiftLeft(shiftValue);
            // Assert
            expect(shiftedRange.start).toBe(expectedStart);
            expect(shiftedRange.end).toBe(expectedEnd);
        });
        it("should handle large shift values that make the start negative", () => {
            // Arrange
            const start = 10;
            const end = 20;
            const shiftValue = 15;
            const expectedStart = -5; // Depending on the implementation, this might need to be clamped to 0.
            const expectedEnd = 5;
            const frameRange = new FrameRange(start, end);
            // Act
            const shiftedRange = frameRange.shiftLeft(shiftValue);
            // Assert
            expect(shiftedRange.start).toBe(expectedStart);
            expect(shiftedRange.end).toBe(expectedEnd);
        });
        // ... Additional tests for other edge cases
    });
    describe("shiftRight", () => {
        it("should correctly shift the range to the right by a given value", () => {
            // Arrange
            const start = 5;
            const end = 15;
            const shiftValue = 3;
            const expectedStart = 8;
            const expectedEnd = 18;
            const frameRange = new FrameRange(start, end);
            // Act
            const shiftedRange = frameRange.shiftRight(shiftValue);
            // Assert
            expect(shiftedRange.start).toBe(expectedStart);
            expect(shiftedRange.end).toBe(expectedEnd);
        });
        it("should not change the range when shift value is zero", () => {
            // Arrange
            const start = 5;
            const end = 15;
            const shiftValue = 0;
            const frameRange = new FrameRange(start, end);
            // Act
            const shiftedRange = frameRange.shiftRight(shiftValue);
            // Assert
            expect(shiftedRange.start).toBe(start);
            expect(shiftedRange.end).toBe(end);
        });
        it("should correctly handle negative shift value", () => {
            // Arrange
            const start = 10;
            const end = 20;
            const shiftValue = -3;
            const expectedStart = 7;
            const expectedEnd = 17;
            const frameRange = new FrameRange(start, end);
            // Act
            const shiftedRange = frameRange.shiftRight(shiftValue);
            // Assert
            expect(shiftedRange.start).toBe(expectedStart);
            expect(shiftedRange.end).toBe(expectedEnd);
        });
        it("should handle large shift values that exceed frame limits", () => {
            // Arrange
            const start = 10;
            const end = 20;
            const shiftValue = 100000; // Large value for testing boundary condition
            const expectedStart = 100010;
            const expectedEnd = 100020;
            const frameRange = new FrameRange(start, end);
            // Act
            const shiftedRange = frameRange.shiftRight(shiftValue);
            // Assert
            expect(shiftedRange.start).toBe(expectedStart);
            expect(shiftedRange.end).toBe(expectedEnd);
        });
        // ... Additional tests for other edge cases
    });
    describe("trimmLeft", () => {
        it("should correctly trim the range from the left by a given value", () => {
            // Arrange
            const start = 5;
            const end = 15;
            const trimValue = 3;
            const expectedStart = 8;
            const expectedEnd = 15; // End is also reduced by trimValue
            const frameRange = new FrameRange(start, end);
            // Act
            const trimmedRange = frameRange.trimmLeft(trimValue);
            // Assert
            expect(trimmedRange.start).toBe(expectedStart);
            expect(trimmedRange.end).toBe(expectedEnd);
        });
        // ... Additional tests for other scenarios
    });
    describe("trimmRight", () => {
        it("should correctly trim the range from the right by a given value", () => {
            // Arrange
            const start = 5;
            const end = 15;
            const trimValue = 3;
            const expectedEnd = 12; // Only the end is reduced by trimValue
            const frameRange = new FrameRange(start, end);
            // Act
            const trimmedRange = frameRange.trimmRight(trimValue);
            // Assert
            expect(trimmedRange.start).toBe(start); // Start remains the same
            expect(trimmedRange.end).toBe(expectedEnd);
        });
        // ... Additional tests for other scenarios
    });
    describe("FrameRange.add", () => {
        it("should add a value to both start and end", () => {
            const frameRange = new FrameRange(2, 10);
            const valueToAdd = 5;
            const expected = new FrameRange(7, 15);
            const result = frameRange.add(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should limit the frame range within minStart and maxEnd", () => {
            const frameRange = new FrameRange(2, 10);
            const valueToAdd = 5;
            const minStart = 0;
            const maxEnd = 12;
            const expected = new FrameRange(4, 12); // Limited by maxEnd
            const result = frameRange.add(valueToAdd, minStart, maxEnd, TimingAction.Shift);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should limit the frame range within minStart and maxEnd", () => {
            const frameRange = new FrameRange(2, 10);
            const valueToAdd = 5;
            const minStart = 0;
            const maxEnd = 12;
            const expected = new FrameRange(7, 12); // Limited by maxEnd
            const result = frameRange.add(valueToAdd, minStart, maxEnd, TimingAction.Trim);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should not limit the frame range if minStart and maxEnd are not provided", () => {
            const frameRange = new FrameRange(2, 10);
            const valueToAdd = 5;
            const expected = new FrameRange(7, 15);
            const result = frameRange.add(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should handle negative values", () => {
            const frameRange = new FrameRange(10, 20);
            const valueToAdd = -5;
            const expected = new FrameRange(5, 15);
            const result = frameRange.add(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
    });
    describe("FrameRange.addStart", () => {
        it("should correctly add a value to the start within the duration limit", () => {
            const frameRange = new FrameRange(0, 10);
            const valueToAdd = 3;
            const expected = new FrameRange(3, 10);
            const result = frameRange.addStart(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should not allow the start to exceed end minus one", () => {
            const frameRange = new FrameRange(0, 10);
            const valueToAdd = 15; // Exceeds the limit
            const expected = new FrameRange(9, 10); // Limited to end - 1
            const result = frameRange.addStart(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should handle negative values", () => {
            const frameRange = new FrameRange(5, 10);
            const valueToAdd = -3;
            const expected = new FrameRange(2, 10); // Adjusted to start - 3
            const result = frameRange.addStart(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should handle zero value", () => {
            const frameRange = new FrameRange(5, 10);
            const valueToAdd = 0; // No change
            const expected = frameRange.clone();
            const result = frameRange.addStart(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
    });
    describe("FrameRange.addEnd", () => {
        it("should correctly add a value to the end", () => {
            const frameRange = new FrameRange(0, 10);
            const valueToAdd = 5;
            const expected = new FrameRange(0, 15);
            const result = frameRange.addEnd(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should handle negative values", () => {
            const frameRange = new FrameRange(0, 10);
            const valueToAdd = -3;
            const expected = new FrameRange(0, 7); // Decreasing the end
            const result = frameRange.addEnd(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should limit negative values to start +1", () => {
            const frameRange = new FrameRange(0, 10);
            const valueToAdd = -10;
            const expected = new FrameRange(0, 1); // Decreasing the end
            const result = frameRange.addEnd(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should handle zero value", () => {
            const frameRange = new FrameRange(0, 10);
            const valueToAdd = 0; // No change
            const expected = frameRange.clone();
            const result = frameRange.addEnd(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
        it("should allow the end to be less than the start", () => {
            const frameRange = new FrameRange(5, 10);
            const valueToAdd = -10; // Resulting end less than start
            const expected = new FrameRange(5, 0); // Adjusted to end < start
            const result = frameRange.addEnd(valueToAdd);
            expect(result.start).toEqual(expected.start);
            expect(result.end).toEqual(expected.end);
        });
    });
    describe("addRelative", () => {
        // Test when TimingAction is 'Trim' (default case)
        it("should correctly add the relative range and trim the duration if necessary", () => {
            // Arrange
            const baseRange = new FrameRange(10, 20);
            const relativeRange = new FrameRange(5, 10);
            // Act
            const result = baseRange.addRelative(relativeRange);
            // Assert
            expect(result.start).toBe(15); // 10 (base start) + 5 (relative start)
            expect(result.end).toBe(20); // 20 (base end) is unchanged since we trim
        });
        it("should correctly add the relative range and expand the duration if necessary", () => {
            // Arrange
            const baseRange = new FrameRange(10, 15);
            const relativeRange = new FrameRange(0, 10);
            const timingAction = TimingAction.Expand;
            // Act
            const result = baseRange.addRelative(relativeRange, timingAction);
            // Assert
            expect(result.start).toBe(10); // 10 (base start) is unchanged
            expect(result.end).toBe(20); // 10 (base start) + 10 (relative duration)
        });
    });
    describe("FrameRange.subtract", () => {
        it("should correctly subtract a value from start and end of the frame range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 20);
            const subtractValue = 5;
            // Act
            const result = frameRange.subtract(subtractValue);
            // Assert
            expect(result.start).toBe(5);
            expect(result.end).toBe(15);
            expect(result.isValid()).toBe(true); // Ensuring the frame range is still valid
        });
    });
    describe("FrameRange.subtractStart", () => {
        it("should correctly subtract a value from the start of the frame range without changing the end", () => {
            // Arrange
            const frameRange = new FrameRange(10, 20);
            const subtractValue = 3;
            // Act
            const result = frameRange.subtractStart(subtractValue);
            // Assert
            expect(result.start).toBe(7); // 10 - 3
            expect(result.end).toBe(20); // Unchanged
            expect(result.isValid()).toBe(true); // Ensuring the frame range is still valid
        });
    });
    describe("FrameRange.subtractEnd", () => {
        it("should correctly subtract a value from the end of the frame range without changing the start", () => {
            // Arrange
            const frameRange = new FrameRange(10, 20);
            const subtractValue = 4;
            // Act
            const result = frameRange.subtractEnd(subtractValue);
            // Assert
            expect(result.start).toBe(10); // Unchanged
            expect(result.end).toBe(16); // 20 - 4
            expect(result.isValid()).toBe(true); // Ensuring the frame range is still valid
        });
    });
    describe("FrameRange.split", () => {
        it("should correctly split the frame range into two equal parts", () => {
            // Arrange
            const frameRange = new FrameRange(10, 20);
            // Act
            const splitResult = frameRange.split();
            const midpoint = (frameRange.start + frameRange.end) / 2;
            // Assert
            expect(splitResult.first.start).toBe(10);
            expect(splitResult.first.end).toBe(midpoint);
            expect(splitResult.last.start).toBe(midpoint);
            expect(splitResult.last.end).toBe(20);
            expect(splitResult.first.isValid()).toBe(true);
            expect(splitResult.last.isValid()).toBe(true);
        });
    });
    describe("withDurationFromEnd", () => {
        it("should adjust start keeping end constant for a positive duration", () => {
            const frameRange = new FrameRange(5, 10);
            const duration = 3;
            const expectedStart = 7; // 10 - 3
            const result = frameRange.withDurationFromEnd(duration);
            expect(result.start).toBe(expectedStart);
            expect(result.end).toBe(frameRange.end);
        });
        it("should set start to end for a zero duration", () => {
            const frameRange = new FrameRange(5, 10);
            const duration = 0;
            const result = frameRange.withDurationFromEnd(duration);
            expect(result.start).toBe(frameRange.end - 1);
            expect(result.end).toBe(frameRange.end);
        });
        it("should handle negative duration", () => {
            const frameRange = new FrameRange(5, 10);
            const duration = -3;
            const result = frameRange.withDurationFromEnd(duration);
            // Define behavior for negative duration
            expect(result.start).toBeLessThan(frameRange.end);
        });
        it("should handle duration longer than current range", () => {
            const frameRange = new FrameRange(5, 10);
            const duration = 10;
            const result = frameRange.withDurationFromEnd(duration);
            // Define behavior for longer duration
            expect(result.start).toBeLessThanOrEqual(5);
        });
        // Add more test cases here with the '++' command
    });
    describe("withAnchor", () => {
        it("should set the anchor to Start", () => {
            const frameRange = new FrameRange(5, 10);
            const anchor = Anchor.Start;
            const result = frameRange.withAnchor(anchor);
            expect(result.anchor).toBe(Anchor.Start);
        });
        it("should set the anchor to Center", () => {
            const frameRange = new FrameRange(5, 10);
            const anchor = Anchor.Center;
            const result = frameRange.withAnchor(anchor);
            expect(result.anchor).toBe(Anchor.Center);
        });
        it("should set the anchor to End", () => {
            const frameRange = new FrameRange(5, 10);
            const anchor = Anchor.End;
            const result = frameRange.withAnchor(anchor);
            expect(result.anchor).toBe(Anchor.End);
        });
        it("should handle null anchor", () => {
            const frameRange = new FrameRange(5, 10);
            const anchor = null;
            const result = frameRange.withAnchor(anchor);
            expect(result.anchor).toBeNull();
        });
        it("should handle undefined anchor", () => {
            const frameRange = new FrameRange(5, 10);
            const result = frameRange.withAnchor(undefined);
            expect(result.anchor).toBeNull();
        });
        // Add more test cases here with the '++' command
    });
    describe("clearAnchor", () => {
        it("should clear the anchor when initially set to Start", () => {
            const frameRange = new FrameRange(5, 10, Anchor.Start);
            const result = frameRange.clearAnchor();
            expect(result.anchor).toBeNull();
        });
        it("should clear the anchor when initially set to Center", () => {
            const frameRange = new FrameRange(5, 10, Anchor.Center);
            const result = frameRange.clearAnchor();
            expect(result.anchor).toBeNull();
        });
        it("should clear the anchor when initially set to End", () => {
            const frameRange = new FrameRange(5, 10, Anchor.End);
            const result = frameRange.clearAnchor();
            expect(result.anchor).toBeNull();
        });
        it("should remain null if anchor is already null", () => {
            const frameRange = new FrameRange(5, 10, null);
            const result = frameRange.clearAnchor();
            expect(result.anchor).toBeNull();
        });
        it("should remain unchanged if anchor is undefined", () => {
            const frameRange = new FrameRange(5, 10); // No anchor specified
            const result = frameRange.clearAnchor();
            expect(result.anchor).toBeNull();
        });
        // Add more test cases here with the '++' command
    });
    describe("getFadeIn", () => {
        it("should return correct start points with positive fade duration and no padding", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 3;
            const result = frameRange.getFadeIn(fadeDuration);
            expect(result).toEqual([10, 13]); // Start of frame range and start after fade
        });
        it("should return correct start points with fade duration and positive padding", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 3;
            const padding = 2;
            const result = frameRange.getFadeIn(fadeDuration, padding);
            expect(result).toEqual([12, 15]); // Adjusted for padding
        });
        it("should handle zero fade duration", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 0;
            const result = frameRange.getFadeIn(fadeDuration);
            expect(result).toEqual([10, 10]); // No fade effect
        });
        it("should handle negative fade duration", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = -3;
            const result = frameRange.getFadeIn(fadeDuration);
            expect(result).toEqual([10, 7]);
        });
        it("should handle fade duration longer than the frame range", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 15;
            const result = frameRange.getFadeIn(fadeDuration);
            expect(result).toEqual([10, 12.5]);
        });
        it("should handle negative padding", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 3;
            const padding = -2;
            const result = frameRange.getFadeIn(fadeDuration, padding);
            expect(result).toEqual([8, 11]);
        });
        // Add more test cases here with the '++' command
    });
    describe("getFadeOut", () => {
        it("should return correct end points with positive fade duration and no padding", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 3;
            const result = frameRange.getFadeOut(fadeDuration);
            expect(result).toEqual([17, 20]); // End after fade and end of frame range
        });
        it("should return correct end points with fade duration and positive padding", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 3;
            const padding = 2;
            const result = frameRange.getFadeOut(fadeDuration, padding);
            expect(result).toEqual([15, 18]); // Adjusted for padding
        });
        it("should handle zero fade duration", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 0;
            const result = frameRange.getFadeOut(fadeDuration);
            expect(result).toEqual([20, 20]); // No fade effect
        });
        it("should handle negative fade duration", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = -3;
            const result = frameRange.getFadeOut(fadeDuration);
            expect(result).toEqual([23, 20]);
        });
        it("should handle fade duration longer than the frame range", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 15;
            const result = frameRange.getFadeOut(fadeDuration);
            expect(result).toEqual([17.5, 20]);
        });
        it("should handle negative padding", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 3;
            const padding = -2;
            const result = frameRange.getFadeOut(fadeDuration, padding);
            expect(result).toEqual([19, 22]);
        });
        // Add more test cases here with the '++' command
    });
    describe("getFadeInOut", () => {
        it("should return correct points for fade-in and fade-out with positive duration and no padding", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 3;
            const result = frameRange.getFadeInOut(fadeDuration);
            expect(result).toEqual([10, 13, 17, 20]); // Start, start after fade-in, end before fade-out, end
        });
        it("should return correct points with fade duration and positive padding", () => {
            const frameRange = new FrameRange(10, 22);
            const fadeDuration = 3;
            const padding = 2;
            const result = frameRange.getFadeInOut(fadeDuration, padding);
            expect(result).toEqual([12, 15, 17, 20]); // Adjusted for padding
        });
        it("should handle zero fade duration", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 0;
            const result = frameRange.getFadeInOut(fadeDuration);
            expect(result).toEqual([10, 10, 20, 20]); // No fade effect
        });
        it("should handle negative fade duration", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = -3;
            const result = frameRange.getFadeInOut(fadeDuration);
            expect(result).toEqual([10, 7, 23, 20]);
        });
        it("should handle fade duration longer than the frame range", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 15;
            const result = frameRange.getFadeInOut(fadeDuration);
            expect(result).toEqual([10, 12.5, 17.5, 20]);
        });
        it("should handle negative padding", () => {
            const frameRange = new FrameRange(10, 20);
            const fadeDuration = 3;
            const padding = -2;
            const result = frameRange.getFadeInOut(fadeDuration, padding);
            expect(result).toEqual([8, 11, 19, 22]);
        });
        // Add more test cases here with the '++' command
    });
    describe("getFadeInOutAtStart", () => {
        it("should calculate fade in and out points with valid durations and padding", () => {
            // Arrange
            const frameRange = new FrameRange(10, 60); // Arbitrary start and end values
            const fadeDuration = 6;
            const visibleDuration = 20;
            const padding = 3;
            // Expected values after applying fadeDuration, visibleDuration, and padding
            const expected = [13, 19, 27, 33]; // Calculated based on the method's logic
            // Act
            const result = frameRange.getFadeInOutAtStart(fadeDuration, visibleDuration, padding);
            // Assert
            expect(result).toEqual(expected);
        });
        it("throws an error when fadeDuration is less than 2", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const fadeDuration = 1; // Invalid fade duration
            const visibleDuration = 20;
            const padding = 3;
            // Act and Assert
            expect(() => {
                frameRange.getFadeInOutAtStart(fadeDuration, visibleDuration, padding);
            }).toThrowError("fadeDuration must be at least 2 frame");
        });
        it("throws an error when visibleDuration is less than 2", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const fadeDuration = 5;
            const visibleDuration = 1; // Invalid visible duration
            const padding = 3;
            // Act and Assert
            expect(() => {
                frameRange.getFadeInOutAtStart(fadeDuration, visibleDuration, padding);
            }).toThrowError("visibleDuration must be at least 2 frame");
        });
        it("should return correct points for standard fade duration and visible duration with no padding", () => {
            const frameRange = new FrameRange(10, 60);
            const fadeDuration = 5;
            const visibleDuration = 20;
            const result = frameRange.getFadeInOutAtStart(fadeDuration, visibleDuration);
            expect(result).toEqual([10, 15, 25, 30]); // Start, start after fade-in, end before fade-out, end
        });
        it("should adjust points correctly with positive padding", () => {
            const frameRange = new FrameRange(10, 60);
            const fadeDuration = 5;
            const visibleDuration = 20;
            const padding = 3;
            const result = frameRange.getFadeInOutAtStart(fadeDuration, visibleDuration, padding);
            expect(result).toEqual([13, 18, 28, 33]); // Adjusted for padding
        });
        it("should handle fade duration longer than visible duration", () => {
            const frameRange = new FrameRange(10, 50);
            const fadeDuration = 25;
            const visibleDuration = 20;
            const result = frameRange.getFadeInOutAtStart(fadeDuration, visibleDuration);
            expect(result).toEqual([10, 15, 25, 30]); // Overlapping fade
        });
        // Add more test cases here with the '++' command
    });
    describe("getFadeInOutAtEnd", () => {
        it("should calculate fade in and out points with valid durations and padding", () => {
            // Arrange
            const frameRange = new FrameRange(10, 60); // Arbitrary start and end values
            const fadeDuration = 6;
            const visibleDuration = 20;
            const padding = 3;
            const expected = [37, 43, 51, 57]; // Adjust as needed
            // Act
            const result = frameRange.getFadeInOutAtEnd(fadeDuration, visibleDuration, padding);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle zero padding", () => {
            // Arrange
            const frameRange = new FrameRange(10, 60);
            const fadeDuration = 5;
            const visibleDuration = 20;
            const padding = 0;
            const expected = [40, 45, 55, 60]; // Adjust as needed
            // Act
            const result = frameRange.getFadeInOutAtEnd(fadeDuration, visibleDuration, padding);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle maximum possible padding", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const fadeDuration = 5;
            const visibleDuration = 20;
            const padding = 50; // Large padding -> padding is limited to 10
            // Expected values for maximum padding
            const expected = [20, 25, 35, 40]; // Adjust as needed
            // Act
            const result = frameRange.getFadeInOutAtEnd(fadeDuration, visibleDuration, padding);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle fade duration equal to visible duration", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const fadeDuration = 10;
            const visibleDuration = 20;
            const padding = 0;
            // Expected values for equal fade and visible durations
            const expected = [30, 40, 40, 50]; // Adjust as needed
            // Act
            const result = frameRange.getFadeInOutAtEnd(fadeDuration, visibleDuration, padding);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle fade duration greater than visible duration", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const fadeDuration = 25;
            const visibleDuration = 20;
            const padding = 0;
            const expected = [30, 35, 45, 50]; // Adjust as needed
            // Act
            const result = frameRange.getFadeInOutAtEnd(fadeDuration, visibleDuration, padding);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle very short frame range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 13);
            const fadeDuration = 5;
            const visibleDuration = 5;
            const padding = 0;
            // Expected values for short frame range
            const expected = [10, 10.75, 12.25, 13]; // Adjust as needed
            // Act
            const result = frameRange.getFadeInOutAtEnd(fadeDuration, visibleDuration, padding);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle negative values", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const fadeDuration = -5;
            const visibleDuration = -20;
            const padding = -3;
            // Act and Assert
            expect(() => {
                frameRange.getFadeInOutAtEnd(fadeDuration, visibleDuration, padding);
            }).toThrow(); // Expecting an error
        });
        // Additional test cases can be added here to cover different scenarios
        // such as different values of fadeDuration, visibleDuration, and padding
    });
    describe("getFadeInOutWithOther", () => {
        it("should handle normal case with valid distinct frame ranges", () => {
            // Arrange
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(30, 40);
            // Expected values for normal case
            const expected = [10, 20, 30, 40];
            // Act
            const result = frameRange1.getFadeInOutWithOther(frameRange2);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle identical frame ranges", () => {
            // Arrange
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(10, 20);
            // Expected values for identical frame ranges
            const expected = [10, 20, 10, 20];
            // Act
            const result = frameRange1.getFadeInOutWithOther(frameRange2);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle overlapping frame ranges", () => {
            // Arrange
            const frameRange1 = new FrameRange(10, 30);
            const frameRange2 = new FrameRange(20, 40);
            // Expected values for overlapping frame ranges
            const expected = [10, 30, 20, 40];
            // Act
            const result = frameRange1.getFadeInOutWithOther(frameRange2);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle one frame range inside another", () => {
            // Arrange
            const frameRange1 = new FrameRange(10, 40);
            const frameRange2 = new FrameRange(20, 30);
            // Expected values for one frame range inside another
            const expected = [10, 40, 20, 30];
            // Act
            const result = frameRange1.getFadeInOutWithOther(frameRange2);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle zero-length frame range", () => {
            // Arrange
            const frameRange1 = new FrameRange(10, 10, null, true);
            const frameRange2 = new FrameRange(20, 20, null, true);
            // Expected values for zero-length frame range
            const expected = [10, 10, 20, 20];
            // Act
            const result = frameRange1.getFadeInOutWithOther(frameRange2);
            // Assert
            expect(result).toEqual(expected);
        });
    });
    describe("getFade", () => {
        it("should return start and end for a normal frame range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            // Expected values for a normal frame range
            const expected = [10, 50];
            // Act
            const result = frameRange.getFade();
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle zero-length frame range", () => {
            // Arrange
            const frameRange = new FrameRange(25, 25, null, true);
            // Expected values for a zero-length frame range
            const expected = [25, 25];
            // Act
            const result = frameRange.getFade();
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle frame range with negative values", () => {
            // Arrange
            const frameRange = new FrameRange(-20, 10);
            // Expected values for a frame range with negative values
            const expected = [-20, 10];
            // Act
            const result = frameRange.getFade();
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle very large frame values", () => {
            // Arrange
            const frameRange = new FrameRange(1000000, 2000000);
            // Expected values for a very large frame range
            const expected = [1000000, 2000000];
            // Act
            const result = frameRange.getFade();
            // Assert
            expect(result).toEqual(expected);
        });
    });
    describe("getCutCenter", () => {
        it("should return center and next frame for a normal frame range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 20);
            // Expected values for center and next frame
            const expected = [15, 16]; // Center is at 15
            // Act
            const result = frameRange.getCutCenter();
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle zero-length frame range", () => {
            // Arrange
            const frameRange = new FrameRange(25, 25, null, true);
            // Expected values for a zero-length frame range
            const expected = [25, 26];
            // Act
            const result = frameRange.getCutCenter();
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle frame range with negative values", () => {
            // Arrange
            const frameRange = new FrameRange(-10, 10);
            // Expected values for center and next frame
            const expected = [0, 1]; // Center is at 0
            // Act
            const result = frameRange.getCutCenter();
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle large frame range", () => {
            // Arrange
            const frameRange = new FrameRange(1000, 2000);
            // Expected values for center and next frame
            const expected = [1500, 1501]; // Center is at 1500
            // Act
            const result = frameRange.getCutCenter();
            // Assert
            expect(result).toEqual(expected);
        });
    });
    describe("getCutCenterInOutWithOther", () => {
        it("should handle non-overlapping centers", () => {
            // Arrange
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(30, 40);
            // Expected values for non-overlapping centers
            const expected = [15, 16, 35, 36]; // Centers at 15 and 35
            // Act
            const result = frameRange1.getCutCenterInOutWithOther(frameRange2);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should throw error with overlapping centers", () => {
            // Arrange
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(10, 20);
            // Act and Assert
            expect(() => {
                frameRange1.getCutCenterInOutWithOther(frameRange2);
            }).toThrow();
        });
        it("should handle zero-length frame ranges", () => {
            // Arrange
            const frameRange1 = new FrameRange(25, 25, null, true);
            const frameRange2 = new FrameRange(30, 30, null, true);
            // Expected values for zero-length frame ranges
            const expected = [25, 26, 30, 31];
            // Act
            const result = frameRange1.getCutCenterInOutWithOther(frameRange2);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle frame ranges with negative values", () => {
            // Arrange
            const frameRange1 = new FrameRange(-20, -10);
            const frameRange2 = new FrameRange(10, 20);
            // Expected values for ranges with negative values
            const expected = [-15, -14, 15, 16];
            // Act
            const result = frameRange1.getCutCenterInOutWithOther(frameRange2);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle large frame ranges", () => {
            // Arrange
            const frameRange1 = new FrameRange(1000, 2000);
            const frameRange2 = new FrameRange(3000, 4000);
            // Expected values for large frame ranges
            const expected = [1500, 1501, 3500, 3501];
            // Act
            const result = frameRange1.getCutCenterInOutWithOther(frameRange2);
            // Assert
            expect(result).toEqual(expected);
        });
    });
    describe("getRelativeFrame", () => {
        it("should return relative frame for a standard frame within the range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 30;
            // Expected relative frame
            const expected = 20; // 30 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle frame number before the start of the frame range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 5;
            // Expected relative frame
            const expected = -5; // 5 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle frame number after the end of the frame range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 60;
            // Expected relative frame
            const expected = 50; // 60 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle negative frame number", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = -20;
            // Expected relative frame
            const expected = -30; // -20 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle zero frame number", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 0;
            // Expected relative frame
            const expected = -10; // 0 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle very large frame number", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 100000;
            // Expected relative frame
            const expected = 99990; // 100000 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
    });
    describe("getRelativeFrame", () => {
        it("should return relative frame for a standard frame within the range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 30;
            // Expected relative frame
            const expected = 20; // 30 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle frame number before the start of the frame range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 5;
            // Expected relative frame
            const expected = -5; // 5 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle frame number after the end of the frame range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 60;
            // Expected relative frame
            const expected = 50; // 60 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle negative frame number", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = -20;
            // Expected relative frame
            const expected = -30; // -20 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle zero frame number", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 0;
            // Expected relative frame
            const expected = -10; // 0 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle very large frame number", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 100000;
            // Expected relative frame
            const expected = 99990; // 100000 - 10
            // Act
            const result = frameRange.getRelativeFrame(frame);
            // Assert
            expect(result).toEqual(expected);
        });
    });
    describe("getRelativeFrameClamp", () => {
        it("should return relative frame for a standard frame within the range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 30;
            // Expected relative frame
            const expected = 20; // 30 - 10
            // Act
            const result = frameRange.getRelativeFrameClamp(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle frame number before the start of the frame range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 5;
            // Expected relative frame
            const expected = 0;
            // Act
            const result = frameRange.getRelativeFrameClamp(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle frame number after the end of the frame range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 60;
            // Expected relative frame
            const expected = 40;
            // Act
            const result = frameRange.getRelativeFrameClamp(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle negative frame number", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = -20;
            // Expected relative frame
            const expected = 0;
            // Act
            const result = frameRange.getRelativeFrameClamp(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle zero frame number", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 0;
            // Expected relative frame
            const expected = 0; // 0 - 10
            // Act
            const result = frameRange.getRelativeFrameClamp(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle very large frame number", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 100000;
            // Expected relative frame
            const expected = 40;
            // Act
            const result = frameRange.getRelativeFrameClamp(frame);
            // Assert
            expect(result).toEqual(expected);
        });
    });
    describe("startAt", () => {
        it("should create a new range starting at 0", () => {
            // Arrange
            const originalRange = new FrameRange(10, 50);
            // Expected new range
            const expectedStart = 0;
            const expectedEnd = 40; // Original duration is 40
            // Act
            const newRange = originalRange.startAt(0);
            // Assert
            expect(newRange.start).toEqual(expectedStart);
            expect(newRange.end).toEqual(expectedEnd);
        });
        it("should create a new range starting at a positive frame number", () => {
            // Arrange
            const originalRange = new FrameRange(10, 50);
            const start = 20;
            // Expected new range
            const expectedStart = 20;
            const expectedEnd = 60; // Original duration is 40
            // Act
            const newRange = originalRange.startAt(start);
            // Assert
            expect(newRange.start).toEqual(expectedStart);
            expect(newRange.end).toEqual(expectedEnd);
        });
        it("should create a new range starting at a negative frame number", () => {
            // Arrange
            const originalRange = new FrameRange(10, 50);
            const start = -10;
            // Expected new range
            const expectedStart = -10;
            const expectedEnd = 30; // Original duration is 40
            // Act
            const newRange = originalRange.startAt(start);
            // Assert
            expect(newRange.start).toEqual(expectedStart);
            expect(newRange.end).toEqual(expectedEnd);
        });
        it("should create a new range starting at the current start frame", () => {
            // Arrange
            const originalRange = new FrameRange(10, 50);
            const start = 10;
            // Expected new range
            const expectedStart = 10;
            const expectedEnd = 50; // Original duration is 40
            // Act
            const newRange = originalRange.startAt(start);
            // Assert
            expect(newRange.start).toEqual(expectedStart);
            expect(newRange.end).toEqual(expectedEnd);
        });
        it("should handle very large start value", () => {
            // Arrange
            const originalRange = new FrameRange(10, 50);
            const start = 100000;
            // Expected new range
            const expectedStart = 100000;
            const expectedEnd = 100040; // Original duration is 40
            // Act
            const newRange = originalRange.startAt(start);
            // Assert
            expect(newRange.start).toEqual(expectedStart);
            expect(newRange.end).toEqual(expectedEnd);
        });
    });
    describe("endDuration", () => {
        it("should set a standard duration", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const newDuration = 30;
            // Expected end value
            const expectedEnd = 10 + newDuration; // Start is 10
            // Act
            frameRange.endDuration(newDuration);
            // Assert
            expect(frameRange.end).toEqual(expectedEnd);
        });
        it("should handle zero duration", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const newDuration = 0;
            // Expected end value
            const expectedEnd = 11; // Start is 10
            // Act
            frameRange.endDuration(newDuration);
            // Assert
            expect(frameRange.end).toEqual(expectedEnd);
        });
        it("should handle negative duration", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const newDuration = -20;
            // Expected end value
            // Note: Depending on implementation, this could either throw an error or set a specific value
            // Adjust the test based on the actual behavior of your implementation
            // Act and Assert
            expect(() => {
                frameRange.endDuration(newDuration);
            }).toThrow(); // If implementation throws an error
        });
        it("should handle very large duration", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const newDuration = 100000;
            // Expected end value
            const expectedEnd = 10 + newDuration;
            // Act
            frameRange.endDuration(newDuration);
            // Assert
            expect(frameRange.end).toEqual(expectedEnd);
        });
        it("should handle duration less than current duration", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const newDuration = 20; // Current duration is 40
            // Expected end value
            const expectedEnd = 10 + newDuration;
            // Act
            frameRange.endDuration(newDuration);
            // Assert
            expect(frameRange.end).toEqual(expectedEnd);
        });
        it("should handle duration equal to current duration", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const newDuration = 40; // Current duration is 40
            // Expected end value
            const expectedEnd = 50; // No change
            // Act
            frameRange.endDuration(newDuration);
            // Assert
            expect(frameRange.end).toEqual(expectedEnd);
        });
    });
    describe("calculateNormPosition", () => {
        it("should calculate normalized position for frame within the range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 30;
            // Expected normalized position
            const expected = (frame - 10) / 40; // (30 - 10) / (50 - 10)
            // Act
            const result = frameRange.calculateNormPosition(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should clamp frame number before the start of the range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 5;
            // Expected normalized position
            const expected = 0; // Clamped to 0
            // Act
            const result = frameRange.calculateNormPosition(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should clamp frame number after the end of the range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 60;
            // Expected normalized position
            const expected = 1; // Clamped to 1
            // Act
            const result = frameRange.calculateNormPosition(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle frame number exactly at start of the range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 10;
            // Expected normalized position
            const expected = 0;
            // Act
            const result = frameRange.calculateNormPosition(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle frame number exactly at end of the range", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 50;
            // Expected normalized position
            const expected = 1;
            // Act
            const result = frameRange.calculateNormPosition(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle negative frame number", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = -20;
            // Expected normalized position
            const expected = 0; // Clamped to 0
            // Act
            const result = frameRange.calculateNormPosition(frame);
            // Assert
            expect(result).toEqual(expected);
        });
        it("should handle very large frame number", () => {
            // Arrange
            const frameRange = new FrameRange(10, 50);
            const frame = 100000;
            // Expected normalized position
            const expected = 1; // Clamped to 1
            // Act
            const result = frameRange.calculateNormPosition(frame);
            // Assert
            expect(result).toEqual(expected);
        });
    });
    describe("getIntersection", () => {
        it("should return intersection for intersecting frame ranges", () => {
            const frameRange1 = new FrameRange(10, 30);
            const frameRange2 = new FrameRange(20, 40);
            const expectedIntersection = new FrameRange(20, 30);
            const result = frameRange1.getIntersection(frameRange2);
            expect(FrameRange.areEqual(expectedIntersection, result)).toBeTruthy();
        });
        it("should return null for non-intersecting frame ranges", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(20, 30);
            const result = frameRange1.getIntersection(frameRange2);
            expect(result).toBeNull();
        });
        it("should return correct intersection when one range is inside another", () => {
            const frameRange1 = new FrameRange(10, 50);
            const frameRange2 = new FrameRange(20, 30);
            const expectedIntersection = new FrameRange(20, 30);
            const result = frameRange1.getIntersection(frameRange2);
            expect(FrameRange.areEqual(expectedIntersection, result)).toBeTruthy();
        });
        it("should return null for adjacent frame ranges", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(20, 30);
            const result = frameRange1.getIntersection(frameRange2);
            expect(result).toBeNull();
        });
        it("should handle invalid frame ranges", () => {
            const frameRange1 = FrameRange.empty(); // Invalid
            const frameRange2 = new FrameRange(20, 30);
            // Depending on implementation, this could either throw an error or return a specific value
            expect(() => frameRange1.getIntersection(frameRange2)).toThrow(); // If implementation throws an error
        });
    });
    // #endregion DERIVED TESTS
    // #region COMPARISSON tests
    describe("isEmpty", () => {
        it("should return true for an empty frame range", () => {
            const frameRange = new FrameRange(0, 0, null, true);
            expect(frameRange.isEmpty()).toBe(true);
        });
        it("should return false for a non-empty frame range", () => {
            const frameRange = new FrameRange(10, 20);
            expect(frameRange.isEmpty()).toBe(false);
        });
        it("should handle negative values", () => {
            const frameRange = new FrameRange(-10, -20);
            expect(frameRange.isEmpty()).toBe(false);
        });
        it("should handle large values", () => {
            const frameRange = new FrameRange(100000, 200000);
            expect(frameRange.isEmpty()).toBe(false);
        });
    });
    describe("isDefault", () => {
        it("should return true for a default frame range", () => {
            const frameRange = new FrameRange(0, 1);
            expect(frameRange.isDefault()).toBe(true);
        });
        it("should return false for a non-default frame range", () => {
            const frameRange = new FrameRange(10, 20);
            expect(frameRange.isDefault()).toBe(false);
        });
        it("should return false for an empty frame range", () => {
            const frameRange = new FrameRange(0, 0, null, true);
            expect(frameRange.isDefault()).toBe(false);
        });
        it("should handle negative or large values", () => {
            const frameRange = new FrameRange(-10, 100000);
            expect(frameRange.isDefault()).toBe(false);
        });
    });
    describe("isValid", () => {
        it("should return true for a valid frame range", () => {
            const frameRange = new FrameRange(10, 50);
            expect(frameRange.isValid()).toBe(true);
        });
        it("should return false for non-finite values", () => {
            const frameRangeNaN = new FrameRange(NaN, 50);
            const frameRangeInfinity = new FrameRange(10, Infinity);
            expect(frameRangeNaN.isValid()).toBe(false);
            expect(frameRangeInfinity.isValid()).toBe(false);
        });
        it("should return false if start is greater than or equal to end", () => {
            const frameRange = new FrameRange(10, 10);
            frameRange.start = 50;
            expect(frameRange.isValid()).toBe(false);
        });
        it("should handle negative values", () => {
            const frameRange = new FrameRange(-20, -10);
            expect(frameRange.isValid()).toBe(true);
        });
        it("should return false for zero-length frame range", () => {
            const frameRange = new FrameRange(10, 10, null, true);
            expect(frameRange.isValid()).toBe(false);
        });
    });
    describe("isIntersecting", () => {
        it("should return true for intersecting frame ranges", () => {
            const frameRange1 = new FrameRange(10, 30);
            const frameRange2 = new FrameRange(20, 40);
            expect(frameRange1.isIntersecting(frameRange2)).toBe(true);
        });
        it("should return false for non-intersecting frame ranges", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(30, 40);
            expect(frameRange1.isIntersecting(frameRange2)).toBe(false);
        });
        it("should return true for one frame range inside another", () => {
            const frameRange1 = new FrameRange(10, 50);
            const frameRange2 = new FrameRange(20, 30);
            expect(frameRange1.isIntersecting(frameRange2)).toBe(true);
        });
        it("should return false for adjacent frame ranges", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(20, 30);
            expect(frameRange1.isIntersecting(frameRange2)).toBe(false);
        });
        it("should throw an error if self frame range is not valid", () => {
            const frameRange1 = new FrameRange(10, 10, null, true); // Invalid
            const frameRange2 = new FrameRange(20, 30);
            expect(() => frameRange1.isIntersecting(frameRange2)).toThrowError("self frames are not valid");
        });
        it("should throw an error if other frame range is not valid", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(30, 30, null, true); // Invalid
            expect(() => frameRange1.isIntersecting(frameRange2)).toThrowError("other frames are not valid");
        });
        it("should throw an error if both frame ranges are invalid", () => {
            const frameRange1 = new FrameRange(10, 10, null, true); // Invalid
            const frameRange2 = new FrameRange(30, 30, null, true); // Invalid
            expect(() => frameRange1.isIntersecting(frameRange2)).toThrowError("self frames are not valid");
        });
    });
    describe("isBefore", () => {
        it("should return true if current range ends before other range starts", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(30, 40);
            expect(frameRange1.isBefore(frameRange2)).toBe(true);
        });
        it("should return true if current range ends exactly at the start of other range", () => {
            const frameRange1 = new FrameRange(10, 30);
            const frameRange2 = new FrameRange(30, 40);
            expect(frameRange1.isBefore(frameRange2)).toBe(true);
        });
        it("should return false if current range overlaps with other range", () => {
            const frameRange1 = new FrameRange(10, 30);
            const frameRange2 = new FrameRange(20, 40);
            expect(frameRange1.isBefore(frameRange2)).toBe(false);
        });
        it("should throw an error if current frame range is not valid", () => {
            const frameRange1 = new FrameRange(30, 30, null, true); // Invalid
            const frameRange2 = new FrameRange(40, 50);
            expect(() => frameRange1.isBefore(frameRange2)).toThrowError("self frames are not valid");
        });
        it("should throw an error if other frame range is not valid", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(30, 30, null, true); // Invalid
            expect(() => frameRange1.isBefore(frameRange2)).toThrowError("other frames are not valid");
        });
        it("should throw an error if both frame ranges are invalid", () => {
            const frameRange1 = new FrameRange(30, 30, null, true); // Invalid
            const frameRange2 = new FrameRange(30, 30, null, true); // Invalid
            expect(() => frameRange1.isBefore(frameRange2)).toThrowError("self frames are not valid");
        });
    });
    describe("isInside", () => {
        it("should return true if current range is inside other range", () => {
            const frameRange1 = new FrameRange(20, 30);
            const frameRange2 = new FrameRange(10, 40);
            expect(frameRange1.isInside(frameRange2)).toBe(true);
        });
        it("should return false if current range partially overlaps other range", () => {
            const frameRange1 = new FrameRange(25, 45);
            const frameRange2 = new FrameRange(10, 40);
            expect(frameRange1.isInside(frameRange2)).toBe(false);
        });
        it("should return false if current range is outside other range", () => {
            const frameRange1 = new FrameRange(50, 60);
            const frameRange2 = new FrameRange(10, 40);
            expect(frameRange1.isInside(frameRange2)).toBe(false);
        });
        it("should return false if other frame range is undefined", () => {
            const frameRange1 = new FrameRange(20, 30);
            expect(frameRange1.isInside(undefined)).toBe(false);
        });
        it("should throw an error if current frame range is not valid", () => {
            const frameRange1 = FrameRange.empty(); // Invalid
            const frameRange2 = new FrameRange(10, 40);
            expect(() => frameRange1.isInside(frameRange2)).toThrowError(`self frames are not valid ${frameRange1}`);
        });
        it("should throw an error if other frame range is not valid", () => {
            const frameRange1 = new FrameRange(20, 30);
            const frameRange2 = FrameRange.empty(); // Invalid
            expect(() => frameRange1.isInside(frameRange2)).toThrowError("other frames are not valid");
        });
        it("should throw an error if both frame ranges are invalid", () => {
            const frameRange1 = FrameRange.empty(); // Invalid
            const frameRange2 = FrameRange.empty(); // Invalid
            expect(() => frameRange1.isInside(frameRange2)).toThrowError(`self frames are not valid ${frameRange1}`);
        });
    });
    describe("startIsEqual", () => {
        it("should return true if starts of both ranges are equal", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(10, 30);
            expect(frameRange1.startIsEqual(frameRange2)).toBe(true);
        });
        it("should return false if starts of ranges are not equal", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(15, 25);
            expect(frameRange1.startIsEqual(frameRange2)).toBe(false);
        });
        it("should return false if other frame range is undefined", () => {
            const frameRange1 = new FrameRange(10, 20);
            expect(frameRange1.startIsEqual(undefined)).toBe(false);
        });
    });
    describe("endIsEqual", () => {
        it("should return true if ends of both ranges are equal", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(15, 20);
            expect(frameRange1.endIsEqual(frameRange2)).toBe(true);
        });
        it("should return false if ends of ranges are not equal", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(15, 25);
            expect(frameRange1.endIsEqual(frameRange2)).toBe(false);
        });
        it("should return false if other frame range is undefined", () => {
            const frameRange1 = new FrameRange(10, 20);
            expect(frameRange1.endIsEqual(undefined)).toBe(false);
        });
    });
    describe("isEqual", () => {
        it("should return true if frame ranges are equal", () => {
            const frameRange1 = new FrameRange(10, 20, Anchor.Start);
            const frameRange2 = new FrameRange(10, 20, Anchor.Start);
            expect(frameRange1.isEqual(frameRange2)).toBe(true);
        });
        it("should return false if frame ranges have different start values", () => {
            const frameRange1 = new FrameRange(10, 20, Anchor.Start);
            const frameRange2 = new FrameRange(15, 20, Anchor.Start);
            expect(frameRange1.isEqual(frameRange2)).toBe(false);
        });
        it("should return false if frame ranges have different end values", () => {
            const frameRange1 = new FrameRange(10, 20, Anchor.Start);
            const frameRange2 = new FrameRange(10, 25, Anchor.Start);
            expect(frameRange1.isEqual(frameRange2)).toBe(false);
        });
        it("should return false if frame ranges have different anchor values", () => {
            const frameRange1 = new FrameRange(10, 20, Anchor.Start);
            const frameRange2 = new FrameRange(10, 20, Anchor.End);
            expect(frameRange1.isEqual(frameRange2)).toBe(false);
        });
        it("should return false if other frame range is undefined", () => {
            const frameRange1 = new FrameRange(10, 20, Anchor.Start);
            expect(frameRange1.isEqual(undefined)).toBe(false);
        });
    });
    describe("containsFrame", () => {
        it("should return true if frame is within the range", () => {
            const frameRange = new FrameRange(10, 20);
            const frame = 15;
            expect(frameRange.containsFrame(frame)).toBe(true);
        });
        it("should return false if frame is before the start of the range", () => {
            const frameRange = new FrameRange(10, 20);
            const frame = 5;
            expect(frameRange.containsFrame(frame)).toBe(false);
        });
        it("should return false if frame is after the end of the range", () => {
            const frameRange = new FrameRange(10, 20);
            const frame = 25;
            expect(frameRange.containsFrame(frame)).toBe(false);
        });
        it("should return true if frame is exactly at the start of the range", () => {
            const frameRange = new FrameRange(10, 20);
            const frame = 10;
            expect(frameRange.containsFrame(frame)).toBe(true);
        });
        it("should return true if frame is exactly at the end of the range", () => {
            const frameRange = new FrameRange(10, 20);
            const frame = 20;
            expect(frameRange.containsFrame(frame)).toBe(true);
        });
        it("should throw an error if the frame range is not valid", () => {
            const frameRange = FrameRange.empty(); // Invalid
            const frame = 15;
            expect(() => frameRange.containsFrame(frame)).toThrowError(`self frames are not valid ${frameRange.start} <= ${frameRange.end}`);
        });
    });
    describe("isInDurationRange", () => {
        it("should return true if duration is within the specified min and max range", () => {
            const frameRange = new FrameRange(10, 50); // Duration 40
            const min = 30;
            const max = 50;
            expect(frameRange.isInDurationRange(min, max)).toBe(true);
        });
        it("should return false if duration is below the specified minimum", () => {
            const frameRange = new FrameRange(10, 20); // Duration 10
            const min = 15;
            const max = 25;
            expect(frameRange.isInDurationRange(min, max)).toBe(false);
        });
        it("should return false if duration is above the specified maximum", () => {
            const frameRange = new FrameRange(10, 60); // Duration 50
            const min = 20;
            const max = 40;
            expect(frameRange.isInDurationRange(min, max)).toBe(false);
        });
        it("should return true if no minimum is specified", () => {
            const frameRange = new FrameRange(10, 40); // Duration 30
            const max = 50;
            expect(frameRange.isInDurationRange(undefined, max)).toBe(true);
        });
        it("should return true if no maximum is specified", () => {
            const frameRange = new FrameRange(10, 40); // Duration 30
            const min = 20;
            expect(frameRange.isInDurationRange(min, undefined)).toBe(true);
        });
        it("should return true if neither minimum nor maximum is specified", () => {
            const frameRange = new FrameRange(10, 40); // Duration 30
            expect(frameRange.isInDurationRange(undefined, undefined)).toBe(true);
        });
    });
    describe("deltaStart", () => {
        it("should return positive delta when current start is after other start", () => {
            const frameRange1 = new FrameRange(30, 50);
            const frameRange2 = new FrameRange(10, 40);
            const expectedDelta = 20; // 30 - 10
            expect(frameRange1.deltaStart(frameRange2)).toBe(expectedDelta);
        });
        it("should return negative delta when current start is before other start", () => {
            const frameRange1 = new FrameRange(10, 40);
            const frameRange2 = new FrameRange(30, 50);
            const expectedDelta = -20; // 10 - 30
            expect(frameRange1.deltaStart(frameRange2)).toBe(expectedDelta);
        });
        it("should return zero delta when starts of both ranges are equal", () => {
            const frameRange1 = new FrameRange(20, 40);
            const frameRange2 = new FrameRange(20, 50);
            const expectedDelta = 0; // 20 - 20
            expect(frameRange1.deltaStart(frameRange2)).toBe(expectedDelta);
        });
    });
    describe("deltaEnd", () => {
        it("should return positive delta when current end is after other end", () => {
            const frameRange1 = new FrameRange(10, 50);
            const frameRange2 = new FrameRange(20, 40);
            const expectedDelta = 10; // 50 - 40
            expect(frameRange1.deltaEnd(frameRange2)).toBe(expectedDelta);
        });
        it("should return negative delta when current end is before other end", () => {
            const frameRange1 = new FrameRange(10, 30);
            const frameRange2 = new FrameRange(20, 50);
            const expectedDelta = -20; // 30 - 50
            expect(frameRange1.deltaEnd(frameRange2)).toBe(expectedDelta);
        });
        it("should return zero delta when ends of both ranges are equal", () => {
            const frameRange1 = new FrameRange(10, 40);
            const frameRange2 = new FrameRange(20, 40);
            const expectedDelta = 0; // 40 - 40
            expect(frameRange1.deltaEnd(frameRange2)).toBe(expectedDelta);
        });
    });
    // #endregion comparison tests
    // #region edit
    describe("addPadding", () => {
        it("should add positive padding to both start and end", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 4;
            const expectedRange = new FrameRange(14, 16);
            const result = frameRange.addPadding(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should add negative padding to reduce the frame range", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = -3;
            const expectedRange = new FrameRange(7, 23);
            const result = frameRange.addPadding(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should handle zero padding", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 0;
            const result = frameRange.addPadding(padding);
            expect(result.isEqual(frameRange)).toBe(true);
        });
        it("should handle different start and end padding values", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 3;
            const paddingEnd = 5;
            const expectedRange = new FrameRange(13, 15);
            const result = frameRange.addPadding(padding, paddingEnd);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should use the same padding for both start and end when paddingEnd is null", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 4;
            const expectedRange = new FrameRange(14, 16);
            const result = frameRange.addPadding(padding, null);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
    });
    describe("addPaddingLeft", () => {
        it("should add positive padding to the start", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 5;
            const expectedRange = new FrameRange(15, 20); // Start increased by 5
            const result = frameRange.addPaddingLeft(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should add negative padding to reduce the start", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = -3;
            const expectedRange = new FrameRange(7, 20); // Start decreased by 3
            const result = frameRange.addPaddingLeft(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should handle zero padding", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 0;
            const result = frameRange.addPaddingLeft(padding);
            expect(frameRange.addPaddingLeft(padding).isEqual(frameRange)).toBe(true);
        });
        it("should not change the frame range if padding exceeds duration", () => {
            const frameRange = new FrameRange(10, 15);
            const padding = 10; // Equal to or greater than duration
            const result = frameRange.addPaddingLeft(padding);
            expect(result.isEqual(frameRange)).toBe(true);
        });
    });
    describe("tryAddPaddingLeft", () => {
        it("should successfully add padding to the start", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 5;
            const expectedRange = new FrameRange(15, 20); // Start increased by 5
            const result = frameRange.tryAddPaddingLeft(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should return undefined if padding exceeds duration", () => {
            const frameRange = new FrameRange(10, 15);
            const padding = 10; // Equal to or greater than duration
            const result = frameRange.tryAddPaddingLeft(padding);
            expect(result).toBeUndefined();
        });
        it("should handle zero padding", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 0;
            const expectedRange = new FrameRange(10, 20); // No change
            const result = frameRange.tryAddPaddingLeft(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should reduce the start with negative padding", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = -3;
            const expectedRange = new FrameRange(7, 20); // Start decreased by 3
            const result = frameRange.tryAddPaddingLeft(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
    });
    describe("addPaddingRight", () => {
        it("should subtract positive padding from the end", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 5;
            const expectedRange = new FrameRange(10, 15); // End decreased by 5
            const result = frameRange.addPaddingRight(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should increase the end with negative padding", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = -3;
            const expectedRange = new FrameRange(10, 23); // End increased by 3
            const result = frameRange.addPaddingRight(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should handle zero padding", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 0;
            const result = frameRange.addPaddingRight(padding);
            expect(result.isEqual(frameRange)).toBe(true);
        });
        it("should not change the frame range if padding exceeds duration", () => {
            const frameRange = new FrameRange(10, 15);
            const padding = 10; // Equal to or greater than duration
            const result = frameRange.addPaddingRight(padding);
            expect(result.isEqual(frameRange)).toBe(true);
        });
    });
    describe("tryAddPaddingRight", () => {
        it("should successfully subtract padding from the end", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 5;
            const expectedRange = new FrameRange(10, 15); // End decreased by 5
            const result = frameRange.tryAddPaddingRight(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should return undefined if padding exceeds duration", () => {
            const frameRange = new FrameRange(10, 15);
            const padding = 10; // Equal to or greater than duration
            const result = frameRange.tryAddPaddingRight(padding);
            expect(result).toBeUndefined();
        });
        it("should handle zero padding", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = 0;
            const expectedRange = new FrameRange(10, 20); // No change
            const result = frameRange.tryAddPaddingRight(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should increase the end with negative padding", () => {
            const frameRange = new FrameRange(10, 20);
            const padding = -3;
            const expectedRange = new FrameRange(10, 23); // End increased by 3
            const result = frameRange.tryAddPaddingRight(padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
    });
    describe("round", () => {
        it("should return the same range for whole number start and end", () => {
            const frameRange = new FrameRange(10, 20);
            const expectedRange = new FrameRange(10, 20);
            expect(frameRange.round().isEqual(expectedRange)).toBe(true);
        });
        it("should round fractional start and end values", () => {
            const frameRange = new FrameRange(10.4, 19.6);
            const expectedRange = new FrameRange(10, 20);
            expect(frameRange.round().isEqual(expectedRange)).toBe(true);
        });
        it("should correctly round negative fractional values", () => {
            const frameRange = new FrameRange(-10.5, -19.6);
            const expectedRange = new FrameRange(-10, -21);
            expect(frameRange.round().isEqual(expectedRange)).toBe(true);
        });
        it("should handle very large start and end values", () => {
            const frameRange = new FrameRange(1000000.4, 2000000.6);
            const expectedRange = new FrameRange(1000000, 2000001);
            expect(frameRange.round().isEqual(expectedRange)).toBe(true);
        });
    });
    describe("pushBehind", () => {
        it("should adjust other range to start after this range", () => {
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(15, 25);
            const expectedRange = new FrameRange(20, 30); // Starts right after frameRange1
            const result = frameRange1.pushOtherBehind(frameRange2);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should return other range if it is already behind this range", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(50, 60);
            const padding = 0;
            const expectedRange = new FrameRange(50, 60);
            const result = frameRange1.pushOtherBehind(frameRange2, padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should adjust other range that intersects with this range", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(35, 45);
            const padding = 0;
            const expectedRange = new FrameRange(40, 50);
            const result = frameRange1.pushOtherBehind(frameRange2, padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should adjust other range that is before but does not intersect this range", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(20, 25);
            const padding = 0;
            const expectedRange = new FrameRange(40, 45);
            const result = frameRange1.pushOtherBehind(frameRange2, padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should handle negative padding", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(10, 20);
            const padding = -5;
            const expectedRange = new FrameRange(45, 55); // Starts 5 frames before frameRange2 ends
            const result = frameRange1.pushOtherBehind(frameRange2, padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should handle positive padding", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(10, 20);
            const padding = 2;
            const expectedRange = new FrameRange(38, 48); // Starts 5 frames after frameRange2 ends
            const result = frameRange1.pushOtherBehind(frameRange2, padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
    });
    describe("pushOtherInFront", () => {
        it("should adjust other range to start before current range", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(40, 50);
            const padding = 0;
            const expectedRange = new FrameRange(20, 30); // Starts right before frameRange1
            const result = frameRange1.pushOtherInFront(frameRange2, padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should return other range if it is already in front of current range", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(10, 20);
            const padding = 0;
            const result = frameRange1.pushOtherInFront(frameRange2, padding);
            expect(result.isEqual(frameRange2)).toBe(true);
        });
        it("should adjust other range that intersects with current range", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(25, 35);
            const padding = 0;
            const expectedRange = new FrameRange(20, 30);
            const result = frameRange1.pushOtherInFront(frameRange2, padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should adjust other range that is behind current range", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(40, 50);
            const padding = 0;
            const expectedRange = new FrameRange(20, 30);
            const result = frameRange1.pushOtherInFront(frameRange2, padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should handle negative padding", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(40, 50);
            const padding = -5;
            const expectedRange = new FrameRange(15, 25); // Starts 5 frames before frameRange1 starts
            const result = frameRange1.pushOtherInFront(frameRange2, padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should handle zero padding", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(40, 50);
            const expectedRange = new FrameRange(20, 30);
            const result = frameRange1.pushOtherInFront(frameRange2);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should handle positive padding", () => {
            const frameRange1 = new FrameRange(30, 40);
            const frameRange2 = new FrameRange(40, 50);
            const padding = 2;
            const expectedRange = new FrameRange(22, 32); // Starts 5 frames before frameRange1 starts
            const result = frameRange1.pushOtherInFront(frameRange2, padding);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
    });
    describe("pushInside", () => {
        it("should return the same range if current range is already inside other range", () => {
            const frameRange1 = new FrameRange(20, 30);
            const frameRange2 = new FrameRange(10, 40);
            const result = frameRange1.pushInside(frameRange2);
            expect(result.isEqual(frameRange1)).toBe(true);
        });
        it("should return other range if current range is larger than other range", () => {
            const frameRange1 = new FrameRange(10, 50);
            const frameRange2 = new FrameRange(20, 30);
            const result = frameRange1.pushInside(frameRange2);
            expect(result.isEqual(frameRange2)).toBe(true);
        });
        it("should adjust start inside and end outside with trim action", () => {
            const frameRange1 = new FrameRange(15, 35);
            const frameRange2 = new FrameRange(10, 30);
            const result = frameRange1.pushInside(frameRange2, TimingAction.Trim);
            const expectedRange = new FrameRange(15, 30);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should adjust end inside and start outside with trim action", () => {
            const frameRange1 = new FrameRange(5, 25);
            const frameRange2 = new FrameRange(10, 30);
            const result = frameRange1.pushInside(frameRange2, TimingAction.Trim);
            const expectedRange = new FrameRange(10, 25);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should adjust range behind other range with shift action", () => {
            const frameRange1 = new FrameRange(35, 45);
            const frameRange2 = new FrameRange(10, 30);
            const result = frameRange1.pushInside(frameRange2, TimingAction.Shift);
            const expectedRange = new FrameRange(20, 30);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
        it("should adjust range before other range with shift action", () => {
            const frameRange1 = new FrameRange(0, 10);
            const frameRange2 = new FrameRange(20, 40);
            const result = frameRange1.pushInside(frameRange2, TimingAction.Shift);
            const expectedRange = new FrameRange(20, 30);
            expect(result.isEqual(expectedRange)).toBe(true);
        });
    });
    describe("pushApart", () => {
        it("should return the current range if other is undefined", () => {
            const frameRange = new FrameRange(10, 20);
            expect(frameRange.pushApart(undefined).isEqual(frameRange)).toBe(true);
        });
        it("should return the current range if other is inside it", () => {
            const frameRange = new FrameRange(10, 30);
            const other = new FrameRange(15, 25);
            expect(frameRange.pushApart(other).isEqual(frameRange)).toBe(true);
        });
        it("should expand to encompass other if current is inside other", () => {
            const frameRange = new FrameRange(15, 25);
            const other = new FrameRange(10, 30);
            const expectedRange = new FrameRange(10, 30);
            expect(frameRange.pushApart(other).isEqual(expectedRange)).toBe(true);
        });
        it("should expand to encompass other if ranges overlap", () => {
            const frameRange = new FrameRange(10, 25);
            const other = new FrameRange(20, 30);
            const expectedRange = new FrameRange(10, 30);
            expect(frameRange.pushApart(other).isEqual(expectedRange)).toBe(true);
        });
        it("should expand to encompass other if ranges are separate", () => {
            const frameRange = new FrameRange(10, 15);
            const other = new FrameRange(20, 25);
            const expectedRange = new FrameRange(10, 25);
            expect(frameRange.pushApart(other).isEqual(expectedRange)).toBe(true);
        });
    });
    describe("expand method", () => {
        it("should expand the frame range based on another frame range with padding", () => {
            // Arrange
            const baseFrameRange = new FrameRange(10, 20);
            const otherFrameRange = new FrameRange(6, 24);
            const padding = 2;
            // Act
            const result = baseFrameRange.expand(otherFrameRange, padding);
            // Assert
            expect(result.start).toBe(8); // Expect the start to remain the same as baseFrameRange
            expect(result.end).toBe(22); // Expect the end to be expanded to include otherFrameRange with padding
        });
        it("should not modify the frame range if the other frame range with padding is inside the current range", () => {
            // Arrange
            const baseFrameRange = new FrameRange(10, 30);
            const otherFrameRange = new FrameRange(15, 20);
            const padding = 2;
            // Act
            const result = baseFrameRange.expand(otherFrameRange, padding);
            // Assert
            expect(result.start).toBe(10); // Expect the start to remain the same as baseFrameRange
            expect(result.end).toBe(30); // Expect the end to remain the same as baseFrameRange
        });
    });
    // #endregion edit
    describe("clone method", () => {
        it("should create a new FrameRange instance with the same properties", () => {
            // Arrange
            const originalFrameRange = new FrameRange(5, 15, Anchor.Start);
            // Act
            const clonedFrameRange = originalFrameRange.clone();
            // Assert
            expect(clonedFrameRange).not.toBe(originalFrameRange); // Ensure it's a new instance
            expect(clonedFrameRange.start).toBe(originalFrameRange.start); // Check if the start property is cloned correctly
            expect(clonedFrameRange.end).toBe(originalFrameRange.end); // Check if the end property is cloned correctly
            expect(clonedFrameRange.anchor).toBe(originalFrameRange.anchor); // Check if the anchor property is cloned correctly
        });
    });
    describe("toString method", () => {
        it("should return a string representation of FrameRange with start, end, and duration", () => {
            // Arrange
            const frameRange = new FrameRange(5, 10);
            // Act
            const result = frameRange.toString();
            // Assert
            expect(result).toBe("frRng: 5 -> 10  [5]");
        });
    });
    // #region static methods
    describe("compareStartAndEnd static method", () => {
        it("should return -1 if the start of the first range is less than the start of the second range", () => {
            // Arrange
            const frameRangeA = new FrameRange(5, 10);
            const frameRangeB = new FrameRange(10, 15);
            // Act
            const result = FrameRange.compareStartAndEnd(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(-1);
        });
        it("should return 1 if the start of the first range is greater than the start of the second range", () => {
            // Arrange
            const frameRangeA = new FrameRange(10, 15);
            const frameRangeB = new FrameRange(5, 10);
            // Act
            const result = FrameRange.compareStartAndEnd(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(1);
        });
        it("should return -1 if the starts are equal and the end of the first range is less than the end of the second range", () => {
            // Arrange
            const frameRangeA = new FrameRange(5, 10);
            const frameRangeB = new FrameRange(5, 15);
            // Act
            const result = FrameRange.compareStartAndEnd(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(-1);
        });
        it("should return 1 if the starts are equal and the end of the first range is greater than the end of the second range", () => {
            // Arrange
            const frameRangeA = new FrameRange(5, 15);
            const frameRangeB = new FrameRange(5, 10);
            // Act
            const result = FrameRange.compareStartAndEnd(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(1);
        });
        it("should return 0 if both the start and end values are equal", () => {
            // Arrange
            const frameRangeA = new FrameRange(5, 10);
            const frameRangeB = new FrameRange(5, 10);
            // Act
            const result = FrameRange.compareStartAndEnd(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(0);
        });
    });
    describe("compareStart static method", () => {
        it("should return -1 if the start of the first range is less than the start of the second range", () => {
            // Arrange
            const frameRangeA = new FrameRange(5, 10);
            const frameRangeB = new FrameRange(10, 15);
            // Act
            const result = FrameRange.compareStart(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(-1);
        });
        it("should return 1 if the start of the first range is greater than the start of the second range", () => {
            // Arrange
            const frameRangeA = new FrameRange(15, 20);
            const frameRangeB = new FrameRange(10, 15);
            // Act
            const result = FrameRange.compareStart(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(1);
        });
        it("should return 0 if the start of both ranges are equal", () => {
            // Arrange
            const frameRangeA = new FrameRange(10, 20);
            const frameRangeB = new FrameRange(10, 30);
            // Act
            const result = FrameRange.compareStart(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(0);
        });
    });
    describe("compareEnd static method", () => {
        it("should return -1 if the end of the first range is less than the end of the second range", () => {
            // Arrange
            const frameRangeA = new FrameRange(5, 10);
            const frameRangeB = new FrameRange(8, 15);
            // Act
            const result = FrameRange.compareEnd(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(-1);
        });
        it("should return 1 if the end of the first range is greater than the end of the second range", () => {
            // Arrange
            const frameRangeA = new FrameRange(10, 20);
            const frameRangeB = new FrameRange(15, 18);
            // Act
            const result = FrameRange.compareEnd(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(1);
        });
        it("should return 0 if the end of both ranges are equal", () => {
            // Arrange
            const frameRangeA = new FrameRange(10, 20);
            const frameRangeB = new FrameRange(15, 20);
            // Act
            const result = FrameRange.compareEnd(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(0);
        });
    });
    describe("compareDuration static method", () => {
        it("should return -1 if the duration of the first range is less than the duration of the second range", () => {
            // Arrange
            const frameRangeA = new FrameRange(5, 10); // Duration: 5
            const frameRangeB = new FrameRange(10, 20); // Duration: 10
            // Act
            const result = FrameRange.compareDuration(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(-1);
        });
        it("should return 1 if the duration of the first range is greater than the duration of the second range", () => {
            // Arrange
            const frameRangeA = new FrameRange(10, 25); // Duration: 15
            const frameRangeB = new FrameRange(15, 20); // Duration: 5
            // Act
            const result = FrameRange.compareDuration(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(1);
        });
        it("should return 0 if the duration of both ranges are equal", () => {
            // Arrange
            const frameRangeA = new FrameRange(5, 15); // Duration: 10
            const frameRangeB = new FrameRange(10, 20); // Duration: 10
            // Act
            const result = FrameRange.compareDuration(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(0);
        });
    });
    describe("empty static method", () => {
        it("should return a new FrameRange instance with start and end as 0, anchor as null, and zeroLength as true", () => {
            // Act
            const emptyRange = FrameRange.empty();
            // Assert
            expect(emptyRange.start).toBe(0); // Check if start is 0
            expect(emptyRange.end).toBe(0); // Check if end is 0
            expect(emptyRange.anchor).toBeNull(); // Check if anchor is null
        });
    });
    describe("fromStart static method", () => {
        it("should create a new FrameRange instance with specified start frame and duration", () => {
            // Arrange
            const startFrame = 5;
            const duration = 10;
            // Act
            const result = FrameRange.fromStart(startFrame, duration);
            // Assert
            expect(result.start).toBe(startFrame); // Check if start is correctly set
            expect(result.end).toBe(startFrame + duration); // Check if end is correctly set
        });
        it("should set the anchor if provided", () => {
            // Arrange
            const startFrame = 5;
            const duration = 10;
            const anchor = Anchor.Start;
            // Act
            const result = FrameRange.fromStart(startFrame, duration, anchor);
            // Assert
            expect(result.anchor).toBe(anchor); // Check if anchor is correctly set
        });
        it("should handle negative start frame", () => {
            // Arrange
            const startFrame = -5;
            const duration = 10;
            // Act
            const result = FrameRange.fromStart(startFrame, duration);
            // Assert
            expect(result.start).toBe(startFrame); // Check if start is correctly set for negative values
            expect(result.end).toBe(startFrame + duration); // Check if end is correctly set for negative duration
        });
    });
    describe("fromCenter static method", () => {
        it("should create a new FrameRange instance centered around the specified center with the given duration", () => {
            // Arrange
            const center = 10;
            const duration = 6; // Even number to ensure symmetric range
            // Act
            const result = FrameRange.fromCenter(center, duration);
            // Assert
            const halfDuration = duration / 2;
            expect(result.start).toBe(center - halfDuration); // Check if start is correctly set
            expect(result.end).toBe(center + halfDuration); // Check if end is correctly set
        });
        it("should set the anchor if provided", () => {
            // Arrange
            const center = 10;
            const duration = 6;
            const anchor = Anchor.Center;
            // Act
            const result = FrameRange.fromCenter(center, duration, anchor);
            // Assert
            expect(result.anchor).toBe(anchor); // Check if anchor is correctly set
        });
        it("should handle odd duration", () => {
            // Arrange
            const center = 10;
            const duration = 5; // Odd number to test asymmetric range
            // Act
            const result = FrameRange.fromCenter(center, duration);
            // Assert
            const halfDuration = duration / 2;
            expect(result.start).toBe(Math.floor(center - halfDuration)); // Check if start is correctly set for odd duration
            expect(result.end).toBe(Math.ceil(center + halfDuration)); // Check if end is correctly set for odd duration
        });
    });
    describe("fromEnd static method", () => {
        it("should create a new FrameRange instance ending at the specified last frame with the given duration", () => {
            // Arrange
            const lastFrame = 20;
            const duration = 10;
            // Act
            const result = FrameRange.fromEnd(lastFrame, duration);
            // Assert
            expect(result.start).toBe(lastFrame - duration); // Check if start is correctly set
            expect(result.end).toBe(lastFrame); // Check if end is correctly set
        });
        it("should set the anchor if provided", () => {
            // Arrange
            const lastFrame = 20;
            const duration = 10;
            const anchor = Anchor.End;
            // Act
            const result = FrameRange.fromEnd(lastFrame, duration, anchor);
            // Assert
            expect(result.anchor).toBe(anchor); // Check if anchor is correctly set
        });
        it("should handle negative last frame and duration", () => {
            // Arrange
            const lastFrame = -5;
            const duration = 10;
            // Act
            const result = FrameRange.fromEnd(lastFrame, duration);
            // Assert
            expect(result.start).toBe(lastFrame - duration); // Check if start is correctly set for negative values
            expect(result.end).toBe(lastFrame); // Check if end is correctly set for negative last frame
        });
    });
    describe("first static method", () => {
        it("should return the first FrameRange based on the start value", () => {
            // Arrange
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(5, 15);
            const frameRange3 = new FrameRange(8, 18);
            const frames = [frameRange1, frameRange2, frameRange3];
            // Act
            const result = FrameRange.first(frames);
            // Assert
            expect(result).toBe(frameRange2); // Expect frameRange2 to be the first based on start value
        });
        it("should return null if the array is empty", () => {
            // Arrange
            const frames = [];
            // Act
            const result = FrameRange.first(frames);
            // Assert
            expect(result).toBeNull(); // Expect null for empty array
        });
        it("should handle an array with a single FrameRange", () => {
            // Arrange
            const singleFrameRange = new FrameRange(10, 20);
            const frames = [singleFrameRange];
            // Act
            const result = FrameRange.first(frames);
            // Assert
            expect(result).toBe(singleFrameRange); // Expect the single frame range to be returned
        });
    });
    describe("last static method", () => {
        it("should return the last FrameRange based on the start value", () => {
            // Arrange
            const frameRange1 = new FrameRange(10, 20);
            const frameRange2 = new FrameRange(5, 15);
            const frameRange3 = new FrameRange(15, 25);
            const frames = [frameRange1, frameRange2, frameRange3];
            // Act
            const result = FrameRange.last(frames);
            // Assert
            expect(result).toBe(frameRange3); // Expect frameRange3 to be the last based on start value
        });
        it("should return null if the array is empty", () => {
            // Arrange
            const frames = [];
            // Act
            const result = FrameRange.last(frames);
            // Assert
            expect(result).toBeNull(); // Expect null for empty array
        });
        it("should handle an array with a single FrameRange", () => {
            // Arrange
            const singleFrameRange = new FrameRange(10, 20);
            const frames = [singleFrameRange];
            // Act
            const result = FrameRange.last(frames);
            // Assert
            expect(result).toBe(singleFrameRange); // Expect the single frame range to be returned as the last
        });
    });
    describe("withStart static method", () => {
        it("should return the first FrameRange with the specified start value", () => {
            // Arrange
            const frameRange1 = new FrameRange(5, 10);
            const frameRange2 = new FrameRange(10, 15);
            const frameRange3 = new FrameRange(15, 20);
            const frames = [frameRange1, frameRange2, frameRange3];
            const startValue = 10;
            // Act
            const result = FrameRange.withStart(frames, startValue);
            // Assert
            expect(result).toBe(frameRange2); // Expect frameRange2 to be the first with start value 10
        });
        it("should return null if no FrameRange matches the specified start value", () => {
            // Arrange
            const frameRange1 = new FrameRange(5, 10);
            const frameRange2 = new FrameRange(11, 15);
            const frameRange3 = new FrameRange(15, 20);
            const frames = [frameRange1, frameRange2, frameRange3];
            const startValue = 10;
            // Act
            const result = FrameRange.withStart(frames, startValue);
            // Assert
            expect(result).toBeNull(); // Expect null as no FrameRange starts at 10
        });
        it("should return null if the array is empty", () => {
            // Arrange
            const frames = [];
            const startValue = 10;
            // Act
            const result = FrameRange.withStart(frames, startValue);
            // Assert
            expect(result).toBeNull(); // Expect null for empty array
        });
    });
    describe("withEnd static method", () => {
        it("should return the first FrameRange with the specified end value", () => {
            // Arrange
            const frameRange1 = new FrameRange(5, 10);
            const frameRange2 = new FrameRange(10, 15);
            const frameRange3 = new FrameRange(15, 20);
            const frames = [frameRange1, frameRange2, frameRange3];
            const endValue = 15;
            // Act
            const result = FrameRange.withEnd(frames, endValue);
            // Assert
            expect(result).toBe(frameRange2); // Expect frameRange2 to be the first with end value 15
        });
        it("should return null if no FrameRange matches the specified end value", () => {
            // Arrange
            const frameRange1 = new FrameRange(5, 10);
            const frameRange2 = new FrameRange(10, 14);
            const frameRange3 = new FrameRange(15, 20);
            const frames = [frameRange1, frameRange2, frameRange3];
            const endValue = 16;
            // Act
            const result = FrameRange.withEnd(frames, endValue);
            // Assert
            expect(result).toBeNull(); // Expect null as no FrameRange ends at 16
        });
        it("should return null if the array is empty", () => {
            // Arrange
            const frames = [];
            const endValue = 10;
            // Act
            const result = FrameRange.withEnd(frames, endValue);
            // Assert
            expect(result).toBeNull(); // Expect null for empty array
        });
    });
    describe("areEqual static method", () => {
        it("should return true when two FrameRange instances are equal", () => {
            // Arrange
            const frameRangeA = new FrameRange(10, 20, Anchor.Center);
            const frameRangeB = new FrameRange(10, 20, Anchor.Center);
            // Act
            const result = FrameRange.areEqual(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(true); // Expect true as both frame ranges are equal
        });
        it("should return false when two FrameRange instances are not equal", () => {
            // Arrange
            const frameRangeA = new FrameRange(10, 20, Anchor.Center);
            const frameRangeB = new FrameRange(10, 20, Anchor.End);
            // Act
            const result = FrameRange.areEqual(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(false); // Expect true as both frame ranges are equal
        });
        it("should return false when two FrameRange instances are not equal", () => {
            // Arrange
            const frameRangeA = new FrameRange(10, 20);
            const frameRangeB = new FrameRange(15, 25);
            // Act
            const result = FrameRange.areEqual(frameRangeA, frameRangeB);
            // Assert
            expect(result).toBe(false); // Expect false as the frame ranges are not equal
        });
        // It("should return false when one of the FrameRange instances is null", () => {
        //   // Arrange
        //   const frameRangeA = new FrameRange(10, 20);
        //   const frameRangeB = null;
        //   // Act
        //   const result = FrameRange.areEqual(
        //     frameRangeA,
        //     frameRangeB as FrameRange
        //   );
        //   // Assert
        //   expect(result).toBe(false); // Expect false as one of the frame ranges is null
        // });
        //   it("should return false when both FrameRange instances are null", () => {
        //     // Arrange
        //     const frameRangeA = null;
        //     const frameRangeB = null;
        //     // Act
        //     const result = FrameRange.areEqual(frameRangeA, frameRangeB);
        //     // Assert
        //     expect(result).toBe(false); // Expect false as both frame ranges are null
        //   });
        // });
        describe("arrayEquals static method", () => {
            it("should return true when two arrays of FrameRange instances are equal", () => {
                // Arrange
                const arrayA = [new FrameRange(5, 10), new FrameRange(10, 15)];
                const arrayB = [new FrameRange(5, 10), new FrameRange(10, 15)];
                // Act
                const result = FrameRange.arrayEquals(arrayA, arrayB);
                // Assert
                expect(result).toBe(true); // Expect true as both arrays are equal
            });
            it("should return false when two arrays of FrameRange instances are not equal in length", () => {
                // Arrange
                const arrayA = [new FrameRange(5, 10)];
                const arrayB = [new FrameRange(5, 10), new FrameRange(10, 15)];
                // Act
                const result = FrameRange.arrayEquals(arrayA, arrayB);
                // Assert
                expect(result).toBe(false); // Expect false as the arrays are not equal in length
            });
            it("should return false when two arrays of FrameRange instances are not equal in content", () => {
                // Arrange
                const arrayA = [new FrameRange(5, 10), new FrameRange(10, 15)];
                const arrayB = [new FrameRange(5, 10), new FrameRange(10, 20)];
                // Act
                const result = FrameRange.arrayEquals(arrayA, arrayB);
                // Assert
                expect(result).toBe(false); // Expect false as the arrays are not equal in content
            });
        });
        // #endregion static methods
    });
});
//# sourceMappingURL=frameRange.spec.js.map