import { describe, expect, it } from "vitest";
import { Anchor } from "../enums/index.js";
import { FrameRange } from "./frameRange.js";
import { FrameRangeCollection } from "./frameRangeCollection.js";
describe("FrameRangeCollection", () => {
    it("testAppendWithValidFrameRanges", () => {
        // Arrange
        const collection = new FrameRangeCollection();
        const range = new FrameRange(0, 10, Anchor.Start);
        const rangeScaled = new FrameRange(0, 20, Anchor.Start);
        // // Act
        collection.append(range, rangeScaled);
        // // Assert
        expect(collection.collection).toHaveLength(1);
        expect(collection.collection[0].isEqual(range)).toBeTruthy();
        expect(collection.collectionScaled).toHaveLength(1);
        expect(collection.collectionScaled[0].isEqual(rangeScaled)).toBeTruthy();
    });
    it("testLastFrameWithEmptyCollection", () => {
        // Arrange
        const collection = new FrameRangeCollection();
        // Act
        const { lastFrame } = collection;
        // Assert
        expect(lastFrame).toBe(0);
    });
    it("testLastFrameWithNonEmptyCollection", () => {
        // Arrange
        const collection = new FrameRangeCollection();
        collection.append(new FrameRange(0, 10, Anchor.Start), new FrameRange(0, 20, Anchor.Start));
        collection.append(new FrameRange(10, 15, Anchor.End), new FrameRange(20, 30, Anchor.End));
        // Act
        const { lastFrame } = collection;
        // Assert
        expect(lastFrame).toBe(15); // Assuming the end of the second range is 25
    });
    it("testLastFrameScaledWithEmptyCollection", () => {
        // Arrange
        const collection = new FrameRangeCollection();
        // Act
        const { lastFrameScaled } = collection;
        // Assert
        expect(lastFrameScaled).toBe(0);
    });
    it("testLastFrameScaledWithNonEmptyCollection", () => {
        // Arrange
        const collection = new FrameRangeCollection();
        collection.append(new FrameRange(0, 10, Anchor.Start), new FrameRange(0, 20, Anchor.Start));
        collection.append(new FrameRange(10, 15, Anchor.End), new FrameRange(20, 25, Anchor.End));
        // Act
        const { lastFrameScaled } = collection;
        // Assert
        expect(lastFrameScaled).toBe(25); // Assuming the end of the second scaled range is 45
    });
    it("testFindIndexContainsFrameWithFrameInCollection", () => {
        // Arrange
        const collection = new FrameRangeCollection();
        collection.append(new FrameRange(0, 10, Anchor.Start), new FrameRange(0, 20, Anchor.Start));
        collection.append(new FrameRange(10, 15, Anchor.End), new FrameRange(20, 25, Anchor.End));
        const testFrame = 12; // Frame that should be within the second FrameRange
        // Act
        const index = collection.findIndexContainsFrame(testFrame);
        // Assert
        expect(index).toBe(1); // Expecting the frame to be in the second FrameRange
    });
    it("testFindIndexContainsFrameWithFrameNotInCollection", () => {
        // Arrange
        const collection = new FrameRangeCollection();
        collection.append(new FrameRange(0, 10, Anchor.Start), new FrameRange(0, 20, Anchor.Start));
        collection.append(new FrameRange(10, 15, Anchor.End), new FrameRange(20, 25, Anchor.End));
        const testFrame = 30; // Frame that is not in any FrameRange
        // Act
        const index = collection.findIndexContainsFrame(testFrame);
        // Assert
        expect(index).toBe(-1); // Expecting -1 as the frame is not in the collection
    });
    it("testFindScaledIndexContainsFrameWithFrameInScaledCollection", () => {
        // Arrange
        const collection = new FrameRangeCollection();
        collection.append(new FrameRange(0, 10, Anchor.Start), new FrameRange(0, 20, Anchor.Start));
        collection.append(new FrameRange(10, 15, Anchor.End), new FrameRange(20, 25, Anchor.End));
        const testFrame = 22; // Frame that should be within the second scaled FrameRange
        // Act
        const index = collection.findIndexContainsFrame(testFrame, true);
        // Assert
        expect(index).toBe(1); // Expecting the frame to be in the second scaled FrameRange
    });
    it("testFindScaledIndexContainsFrameWithFrameNotInScaledCollection", () => {
        // Arrange
        const collection = new FrameRangeCollection();
        collection.append(new FrameRange(0, 10, Anchor.Start), new FrameRange(0, 20, Anchor.Start));
        collection.append(new FrameRange(10, 15, Anchor.End), new FrameRange(20, 25, Anchor.End));
        const testFrame = 50; // Frame that is not in any scaled FrameRange
        // Act
        const index = collection.findIndexContainsFrame(testFrame, true);
        // Assert
        expect(index).toBe(-1); // Expecting -1 as the frame is not in the scaled collection
    });
    describe("includingFrame", () => {
        it("should return FrameRange objects that include the specified frame in collection", () => {
            // Arrange
            const collection = new FrameRangeCollection();
            const frameRange1 = new FrameRange(0, 10); // Frame range from 0 to 10
            const frameRange2 = new FrameRange(20, 30); // Frame range from 11 to 20
            collection.append(frameRange1, frameRange1); // Adding frameRange1 to collection
            collection.append(frameRange2, frameRange1); // Adding frameRange2 to collection
            const targetFrame = 5; // Frame to check inclusion for
            // Act
            const result = collection.includingFrame(targetFrame);
            // Assert
            expect(result.length).to.equal(1); // Expecting one FrameRange object
            expect(result[0].isEqual(frameRange1)).toBeTruthy(); // The frame is included in frameRange1
        });
        it("should return empty array", () => {
            // Arrange
            const collection = new FrameRangeCollection();
            const frameRange1 = new FrameRange(0, 10); // Frame range from 0 to 10
            const frameRange2 = new FrameRange(20, 30); // Frame range from 11 to 20
            collection.append(frameRange1, frameRange1); // Adding frameRange1 to collection
            collection.append(frameRange2, frameRange2); // Adding frameRange2 to collection
            const targetFrame = 11; // Frame to check inclusion for
            // Act
            const result = collection.includingFrame(targetFrame);
            // Assert
            expect(result.length).to.equal(0); // Expecting one FrameRange object
        });
    });
    describe("leftOfFrame", () => {
        it("should return all FrameRange objects to the left of the specified frame", () => {
            // Arrange
            const collection = new FrameRangeCollection();
            const frameRange1 = new FrameRange(0, 5); // Frame range from 0 to 5
            const frameRange2 = new FrameRange(6, 10); // Frame range from 6 to 10
            const frameRange3 = new FrameRange(11, 15); // Frame range from 11 to 15
            collection.append(frameRange1, frameRange1); // Adding frameRange1 to collection
            collection.append(frameRange2, frameRange2); // Adding frameRange2 to collection
            collection.append(frameRange3, frameRange3); // Adding frameRange3 to collection
            const targetFrame = 16; // Frame to search left of
            // Act
            const result = collection.leftOfFrame(targetFrame);
            // Assert
            expect(result.length).to.equal(3); // Expecting three FrameRange objects
            expect(result).to.deep.include.members([
                frameRange1,
                frameRange2,
                frameRange3,
            ]); // All ranges are to the left of targetFrame
        });
        it("should return all FrameRange objects to the left of the specified frame Scaled", () => {
            // Arrange
            const collection = new FrameRangeCollection();
            const frameRange1 = new FrameRange(0, 5); // Frame range from 0 to 5
            const frameRange2 = new FrameRange(6, 10); // Frame range from 6 to 10
            const frameRange3 = new FrameRange(11, 15); // Frame range from 11 to 15
            collection.append(frameRange1, frameRange1); // Adding frameRange1 to collection
            collection.append(frameRange2, frameRange2); // Adding frameRange2 to collection
            collection.append(frameRange3, frameRange3); // Adding frameRange3 to collection
            const targetFrame = 16; // Frame to search left of
            // Act
            const result = collection.leftOfFrame(targetFrame, true);
            // Assert
            expect(result.length).to.equal(3); // Expecting three FrameRange objects
            expect(result).to.deep.include.members([
                frameRange1,
                frameRange2,
                frameRange3,
            ]); // All ranges are to the left of targetFrame
        });
    });
});
//# sourceMappingURL=frameRangeCollection.spec.js.map