/* eslint-disable capitalized-comments */
import { beforeEach, describe, expect, it } from "vitest";
import { LottieLayer } from "./lottie.layer.js";
import { lottieLayerData } from "../../data";
import { DEFAULT_VIDEO_SETTINGS } from "../../interfaces/index.js";
import { FrameRange } from "../frameRange.js";
describe("LottieLayer", () => {
    let lottieLayer;
    let data;
    const assets = [];
    beforeEach(() => {
        data = Object.assign({}, lottieLayerData);
        lottieLayer = LottieLayer.fromILayer(lottieLayerData, assets, DEFAULT_VIDEO_SETTINGS, 150);
    });
    it("should return a ILayer", () => {
        const iLayer = lottieLayer.getILayer();
        expect(iLayer).toBeDefined();
        expect(iLayer.id).toBe(data.id);
        expect(iLayer.name).toBe(data.name);
        expect(iLayer.type).toBe(data.type);
    });
    describe("freeze Frame calculation", () => {
        it("should return the correct lottieAnimation properties", () => {
            expect(lottieLayer.lottieAnimation.frameDuration).toBe(150);
            expect(lottieLayer.lottieAnimation.sumOfBrakes).toBe(31);
        });
        it("should return break range", () => {
            const breaks = lottieLayer.getBreakRanges();
            expect(breaks).toBeDefined();
            expect(breaks).toHaveLength(1);
            expect(breaks[0].isEqual(new FrameRange(69, 100))).toBeTruthy();
        });
        it("should return clipPauseSequenceScaled without scale", () => {
            const clipPauseSequenceScaled = lottieLayer.clipPauseSequenceScaled(lottieLayer.absoluteRange);
            const i1 = clipPauseSequenceScaled === null || clipPauseSequenceScaled === void 0 ? void 0 : clipPauseSequenceScaled.collection[0];
            const i2 = clipPauseSequenceScaled === null || clipPauseSequenceScaled === void 0 ? void 0 : clipPauseSequenceScaled.collection[1];
            const i3 = clipPauseSequenceScaled === null || clipPauseSequenceScaled === void 0 ? void 0 : clipPauseSequenceScaled.collection[2];
            expect(i1 === null || i1 === void 0 ? void 0 : i1.type).toBe("animation");
            expect(i1 === null || i1 === void 0 ? void 0 : i1.range.isEqual(new FrameRange(0, 68))).toBeTruthy();
            expect(i2 === null || i2 === void 0 ? void 0 : i2.type).toBe("pause");
            expect(i2 === null || i2 === void 0 ? void 0 : i2.range.isEqual(new FrameRange(69, 100))).toBeTruthy();
            expect(i3 === null || i3 === void 0 ? void 0 : i3.type).toBe("animation");
            expect(i3 === null || i3 === void 0 ? void 0 : i3.range.isEqual(new FrameRange(101, 150))).toBeTruthy();
            expect(clipPauseSequenceScaled).toBeDefined();
        });
        it("should return clipPauseSequenceScaled with scale", () => {
            const streched = lottieLayer.absoluteRange.addEnd(10);
            const clipPauseSequenceScaled = lottieLayer.clipPauseSequenceScaled(streched);
            const i1 = clipPauseSequenceScaled === null || clipPauseSequenceScaled === void 0 ? void 0 : clipPauseSequenceScaled.collection[0];
            const i2 = clipPauseSequenceScaled === null || clipPauseSequenceScaled === void 0 ? void 0 : clipPauseSequenceScaled.collection[1];
            const i3 = clipPauseSequenceScaled === null || clipPauseSequenceScaled === void 0 ? void 0 : clipPauseSequenceScaled.collection[2];
            expect(i1 === null || i1 === void 0 ? void 0 : i1.type).toBe("animation");
            expect(i1 === null || i1 === void 0 ? void 0 : i1.scaledRange.isEqual(new FrameRange(0, 68))).toBeTruthy();
            expect(i2 === null || i2 === void 0 ? void 0 : i2.type).toBe("pause");
            expect(i2 === null || i2 === void 0 ? void 0 : i2.scaledRange.isEqual(new FrameRange(69, 110))).toBeTruthy();
            expect(i3 === null || i3 === void 0 ? void 0 : i3.type).toBe("animation");
            expect(i3 === null || i3 === void 0 ? void 0 : i3.scaledRange.isEqual(new FrameRange(111, 160))).toBeTruthy();
            expect(clipPauseSequenceScaled).toBeDefined();
        });
        describe("nextFrame calculation, NO Scaling No Shift", () => {
            it("no pause in the past, nextFrame should return the input frame", () => {
                lottieLayer.relativeRange = new FrameRange(0, 150);
                lottieLayer.absoluteRange = new FrameRange(0, 150);
                const lottieDuration = lottieLayer.lottieAnimation.frameDuration;
                const breaks = lottieLayer.lottieAnimation.getBreaksRanges();
                const frame = 0;
                const nextFrame = lottieLayer.nextFrame(frame, lottieLayer.absoluteRange, lottieDuration, breaks);
                expect(nextFrame).toBe(frame);
            });
            it("no pause in the past, nextFrame should return the input frame", () => {
                lottieLayer.relativeRange = new FrameRange(0, 150);
                lottieLayer.absoluteRange = new FrameRange(0, 150);
                const lottieDuration = lottieLayer.lottieAnimation.frameDuration;
                const breaks = lottieLayer.lottieAnimation.getBreaksRanges();
                const frame = 69;
                const nextFrame = lottieLayer.nextFrame(frame, lottieLayer.absoluteRange, lottieDuration, breaks);
                expect(nextFrame).toBe(frame);
            });
            it("enter the pause, nextFrame should return the the start of the pause", () => {
                lottieLayer.relativeRange = new FrameRange(0, 150);
                lottieLayer.absoluteRange = new FrameRange(0, 150);
                const lottieDuration = lottieLayer.lottieAnimation.frameDuration;
                const breaks = lottieLayer.lottieAnimation.getBreaksRanges();
                const frame = 70;
                const startOfPause = 69;
                const nextFrame = lottieLayer.nextFrame(frame, lottieLayer.absoluteRange, lottieDuration, breaks);
                expect(nextFrame).toBe(startOfPause);
            });
            it("leave the pause, nextFrame should return the the input frame. NO SCALING", () => {
                lottieLayer.relativeRange = new FrameRange(0, 150);
                lottieLayer.absoluteRange = new FrameRange(0, 150);
                const lottieDuration = lottieLayer.lottieAnimation.frameDuration;
                const breaks = lottieLayer.lottieAnimation.getBreaksRanges();
                const frame = 110;
                const nextFrame = lottieLayer.nextFrame(frame, lottieLayer.absoluteRange, lottieDuration, breaks);
                expect(nextFrame).toBe(frame);
            });
        });
        describe("nextFrame calculation, with 10 Scaling No Shift", () => {
            it("no pause in the past, nextFrame should return the input frame", () => {
                lottieLayer.relativeRange = new FrameRange(0, 160);
                lottieLayer.absoluteRange = new FrameRange(0, 160);
                const lottieDuration = lottieLayer.lottieAnimation.frameDuration;
                const breaks = lottieLayer.lottieAnimation.getBreaksRanges();
                const frame = 69;
                const nextFrame = lottieLayer.nextFrame(frame, lottieLayer.absoluteRange, lottieDuration, breaks);
                expect(nextFrame).toBe(frame);
            });
            it("enter the pause, nextFrame should return the the start of the pause", () => {
                lottieLayer.relativeRange = new FrameRange(0, 160);
                lottieLayer.absoluteRange = new FrameRange(0, 160);
                const lottieDuration = lottieLayer.lottieAnimation.frameDuration;
                const breaks = lottieLayer.lottieAnimation.getBreaksRanges();
                const frame = 70;
                const startOfPause = 69;
                const nextFrame = lottieLayer.nextFrame(frame, lottieLayer.absoluteRange, lottieDuration, breaks);
                expect(nextFrame).toBe(startOfPause);
            });
            it("leave the pause, nextFrame should return sum of history - delta. with SCALING", () => {
                lottieLayer.relativeRange = new FrameRange(0, 160);
                lottieLayer.absoluteRange = new FrameRange(0, 160);
                const lottieDuration = lottieLayer.lottieAnimation.frameDuration;
                const breaks = lottieLayer.lottieAnimation.getBreaksRanges();
                const frame = 120;
                const target = 110;
                const nextFrame = lottieLayer.nextFrame(frame, lottieLayer.absoluteRange, lottieDuration, breaks);
                expect(nextFrame).toBe(target);
            });
        });
        describe("nextFrame calculation, with 10 Scaling and 10 Shift", () => {
            beforeEach(() => {
                lottieLayer.relativeRange = new FrameRange(10, 170);
                lottieLayer.absoluteRange = new FrameRange(10, 170);
            });
            it("no pause in the past, nextFrame should return the input frame", () => {
                const lottieDuration = lottieLayer.lottieAnimation.frameDuration;
                const breaks = lottieLayer.lottieAnimation.getBreaksRanges();
                const frame = 79;
                const target = 69;
                const nextFrame = lottieLayer.nextFrame(frame, lottieLayer.absoluteRange, lottieDuration, breaks);
                expect(nextFrame).toBe(target);
            });
            it("enter the pause, nextFrame should return the the start of the pause", () => {
                const lottieDuration = lottieLayer.lottieAnimation.frameDuration;
                const breaks = lottieLayer.lottieAnimation.getBreaksRanges();
                const frame = 80;
                const startOfPause = 69;
                const nextFrame = lottieLayer.nextFrame(frame, lottieLayer.absoluteRange, lottieDuration, breaks);
                expect(nextFrame).toBe(startOfPause);
            });
            it("leave the pause, nextFrame should return sum of history - delta. with SCALING", () => {
                const lottieDuration = lottieLayer.lottieAnimation.frameDuration;
                const breaks = lottieLayer.lottieAnimation.getBreaksRanges();
                const frame = 130;
                const target = 110;
                const nextFrame = lottieLayer.nextFrame(frame, lottieLayer.absoluteRange, lottieDuration, breaks);
                expect(nextFrame).toBe(target);
            });
        });
    });
});
//# sourceMappingURL=lottie.layer.spec.js.map