var _a;
import "reflect-metadata";
import { immerable } from "immer";
import { clamp } from "../index.js";
import { AnchorType, PositionType, CssUnit } from "../enums/index.js";
export class Position {
    // eslint-disable-next-line no-useless-constructor, max-params
    constructor(x, y, anchorX = AnchorType.Center, anchorY = AnchorType.Center, positionType = PositionType.Absolute) {
        this.x = x;
        this.y = y;
        this.anchorX = anchorX;
        this.anchorY = anchorY;
        this.positionType = positionType;
        this[_a] = true; // Option 1
        this.xUnit = CssUnit.percent;
        this.yUnit = CssUnit.percent;
        this.isNull = false;
        this.toString = () => {
            return `Position:  x:${this.x} y:${this.y}  originX:${this.anchorX} originY:${this.anchorY}  position:${this.positionType}`;
        };
        this.textAlign = () => {
            switch (this.anchorX) {
                case AnchorType.Left:
                    return "left";
                case AnchorType.Center:
                    return "center";
                case AnchorType.Right:
                    return "right";
                default:
                    return "center";
            }
        };
    }
    setXInPercent(percent) {
        this.x = clamp(percent, -100, 100) * 0.01;
        this.xUnit = CssUnit.percent;
    }
    setYInPercent(percent) {
        this.y = clamp(percent, -100, 100) * 0.01;
        this.yUnit = CssUnit.percent;
    }
    getOffsetX(width) {
        switch (this.anchorX) {
            case AnchorType.Left:
                return 0;
            case AnchorType.Right:
                return 0 - width.value;
            case AnchorType.Center:
                return 0 - width.value / 2;
            default:
                return 0;
        }
    }
    getOffsetY(height) {
        switch (this.anchorY) {
            case AnchorType.Top:
                return 0;
            case AnchorType.Bottom:
                return 0 - height.value;
            case AnchorType.Center:
                return 0 - height.value / 2;
            default:
                return 0;
        }
    }
    toPercent() {
        const isX = this.x || this.x === 0;
        const isY = this.y || this.y === 0;
        const posInPercent = Position.leftTopZero();
        if (isX)
            posInPercent.x = clamp(this.x * 100, -100, 100);
        if (isY)
            posInPercent.y = clamp(this.y * 100, -100, 100);
        return posInPercent;
    }
    static() {
        this.positionType = PositionType.Static;
        return this;
    }
    relative() {
        this.positionType = PositionType.Relative;
        return this;
    }
    fixed() {
        this.positionType = PositionType.Fixed;
        return this;
    }
    absolute() {
        this.positionType = PositionType.Absolute;
        return this;
    }
    sticky() {
        this.positionType = PositionType.Sticky;
        return this;
    }
    left() {
        this.anchorX = AnchorType.Left;
        return this;
    }
    right() {
        this.anchorX = AnchorType.Right;
        return this;
    }
    top() {
        this.anchorY = AnchorType.Top;
        return this;
    }
    bottom() {
        this.anchorY = AnchorType.Bottom;
        return this;
    }
    xPixel() {
        this.xUnit = CssUnit.pixel;
        return this;
    }
    xPercent() {
        this.xUnit = CssUnit.percent;
        return this;
    }
    toIPosition() {
        if (this.isNull)
            return null;
        return {
            x: this.x,
            y: this.y,
            anchorX: this.anchorX,
            anchorY: this.anchorY,
        };
    }
    static leftTopZero() {
        return new Position(0, 0, AnchorType.Left, AnchorType.Top, PositionType.Absolute);
    }
    static centerBottom() {
        return new Position(0, 1, AnchorType.Center, AnchorType.Bottom, PositionType.Absolute);
    }
    static leftTop(x, y) {
        return new Position(x, y, AnchorType.Left, AnchorType.Top, PositionType.Absolute);
    }
    static centerZero() {
        return new Position(0, 0, AnchorType.Center, AnchorType.Center, PositionType.Absolute);
    }
    static centerCenter() {
        return new Position(0.5, 0.5, AnchorType.Center, AnchorType.Center, PositionType.Absolute);
    }
    static centerTop() {
        return new Position(0.5, 0, AnchorType.Center, AnchorType.Top, PositionType.Absolute);
    }
    static center(x, y) {
        return new Position(x, y, AnchorType.Center, AnchorType.Center, PositionType.Absolute);
    }
    static fromIPosition(value, defaultValue = Position.leftTopZero()) {
        const position = defaultValue;
        position.isNull = value === null;
        position.xUnit = CssUnit.percent;
        position.yUnit = CssUnit.percent;
        if (value === null || value === void 0 ? void 0 : value.x)
            position.x = value === null || value === void 0 ? void 0 : value.x;
        if (value === null || value === void 0 ? void 0 : value.y)
            position.y = value === null || value === void 0 ? void 0 : value.y;
        if (value === null || value === void 0 ? void 0 : value.anchorX)
            position.anchorX = value === null || value === void 0 ? void 0 : value.anchorX;
        if (value === null || value === void 0 ? void 0 : value.anchorY)
            position.anchorY = value === null || value === void 0 ? void 0 : value.anchorY;
        return position;
    }
    static fromUnitPositionRelative(value, anchorPoint) {
        const position = Position.fromIPosition({
            x: value.x,
            y: value.y,
            anchorX: anchorTypeXFromUnitSpace(anchorPoint.x),
            anchorY: anchorTypeYFromUnitSpace(anchorPoint.y),
        });
        position.x = value.x;
        position.y = value.y;
        return position;
    }
    static aspectFit(widthInput, heightInput, widthBoundary, heightBoundary) {
        if (widthInput <= 0 ||
            heightInput <= 0 ||
            widthBoundary <= 0 ||
            heightBoundary <= 0)
            throw new Error("Input parameter must be positive numbers");
        // RatioBoundary > ratioInput
        // ? (widthInput * heightBoundary / heightInput, heightBoundary)
        // : (widthBoundary, heightInput * widthBoundary / widthInput);
        const ratioInput = widthInput / heightInput;
        const ratioBoundary = widthBoundary / heightBoundary;
        const width = ratioBoundary > ratioInput
            ? (widthInput * heightBoundary) / heightInput
            : widthBoundary;
        const height = ratioBoundary > ratioInput
            ? heightBoundary
            : (heightInput * widthBoundary) / widthInput;
        return { width, height };
    }
    static aspectFill(widthInput, heightInput, widthBoundary, heightBoundary) {
        if (widthInput <= 0 ||
            heightInput <= 0 ||
            widthBoundary <= 0 ||
            heightBoundary <= 0)
            throw new Error("Input parameter must be positive numbers");
        // RatioBoundary > ratioInput
        // ? (widthInput * heightBoundary / heightInput, heightBoundary)
        // : (widthBoundary, heightInput * widthBoundary / widthInput);
        const ratioInput = widthInput / heightInput;
        const ratioBoundary = widthBoundary / heightBoundary;
        const width = ratioBoundary < ratioInput
            ? (widthInput * heightBoundary) / heightInput
            : widthBoundary;
        const height = ratioBoundary < ratioInput
            ? heightBoundary
            : (heightInput * widthBoundary) / widthInput;
        return { width, height };
    }
}
_a = immerable;
export function positionAnchorToUnitSpace(position) {
    return {
        x: position.anchorX ? anchorTypeXToUnitSpace(position.anchorX) : 0,
        y: position.anchorY ? anchorTypeYToUnitSpace(position.anchorY) : 0,
    };
}
export function anchorTypeXToUnitSpace(anchor) {
    switch (anchor) {
        case AnchorType.Left:
            return 0;
        case AnchorType.Center:
            return 0.5;
        case AnchorType.Right:
            return 1;
        default:
            return 0;
    }
}
export function anchorTypeYToUnitSpace(anchor) {
    switch (anchor) {
        case AnchorType.Top:
            return 0;
        case AnchorType.Center:
            return 0.5;
        case AnchorType.Bottom:
            return 1;
        default:
            return 0;
    }
}
export function anchorTypeToUnitSpace(x, y) {
    return {
        x: anchorTypeXToUnitSpace(x),
        y: anchorTypeYToUnitSpace(y),
    };
}
export function anchorTypeFromUnitSpace(x, y) {
    return {
        x: anchorTypeXFromUnitSpace(x),
        y: anchorTypeYFromUnitSpace(y),
    };
}
function anchorTypeXFromUnitSpace(x) {
    if (x === 0)
        return AnchorType.Left;
    if (x === 0.5)
        return AnchorType.Center;
    if (x === 1)
        return AnchorType.Right;
    return AnchorType.Center;
}
function anchorTypeYFromUnitSpace(y) {
    if (y === 0)
        return AnchorType.Top;
    if (y === 0.5)
        return AnchorType.Center;
    if (y === 1)
        return AnchorType.Bottom;
    return AnchorType.Center;
}
//# sourceMappingURL=position.js.map