import { describe, it, expect, beforeEach } from "vitest";
import { NumberRange } from "./numberRange";
describe("Range class tests", () => {
    let range;
    beforeEach(() => {
        range = new NumberRange(1, 5);
    });
    it("should initialize start and end correctly", () => {
        expect(range.start).toBe(1);
        expect(range.end).toBe(5);
    });
    it("should validate range correctly", () => {
        expect(range.isValid()).toBe(true);
        range = new NumberRange(5, 1);
        expect(range.isValid()).toBe(false);
    });
    it("should check if range is empty", () => {
        expect(range.isEmpty()).toBe(false);
        range = new NumberRange(5, 5);
        expect(range.isEmpty()).toBe(true);
    });
    it("should check if ranges are equal", () => {
        const other = new NumberRange(1, 5);
        expect(range.isEqual(other)).toBe(true);
        const different = new NumberRange(2, 6);
        expect(range.isEqual(different)).toBe(false);
    });
    it("should check if range is before another range", () => {
        const other = new NumberRange(6, 10);
        expect(range.isBefore(other)).toBe(true);
        expect(other.isBefore(range)).toBe(false);
    });
    it("should check if range is behind another range", () => {
        const other = new NumberRange(6, 10);
        expect(range.isBehind(other)).toBe(false);
        expect(other.isBehind(range)).toBe(true);
    });
    it("should check if ranges are intersecting", () => {
        const other = new NumberRange(4, 6);
        expect(range.isIntersecting(other)).toBe(true);
        const nonIntersecting = new NumberRange(6, 10);
        expect(range.isIntersecting(nonIntersecting)).toBe(false);
    });
    it("should check if range is inside another range", () => {
        const other = new NumberRange(0, 10);
        expect(range.isInside(other)).toBe(true);
        expect(other.isInside(range)).toBe(false);
    });
    it("should check if start points are equal", () => {
        const other = new NumberRange(1, 10);
        expect(range.startIsEqual(other)).toBe(true);
        const different = new NumberRange(2, 10);
        expect(range.startIsEqual(different)).toBe(false);
    });
    it("should check if end points are equal", () => {
        const other = new NumberRange(0, 5);
        expect(range.endIsEqual(other)).toBe(true);
        const different = new NumberRange(0, 6);
        expect(range.endIsEqual(different)).toBe(false);
    });
    it("should check if range contains a point", () => {
        expect(range.containsPoint(3)).toBe(true);
        expect(range.containsPoint(0)).toBe(false);
    });
    it("should calculate duration correctly", () => {
        expect(range.duration()).toBe(4);
    });
    it("should calculate delta start correctly", () => {
        const other = new NumberRange(2, 6);
        expect(range.deltaStart(other)).toBe(-1);
    });
    it("should calculate delta end correctly", () => {
        const other = new NumberRange(2, 6);
        expect(range.deltaEnd(other)).toBe(-1);
    });
    it("should get intersection of ranges", () => {
        const other = new NumberRange(3, 7);
        const intersection = range.getIntersection(other);
        expect(intersection === null || intersection === void 0 ? void 0 : intersection.start).toBe(3);
        expect(intersection === null || intersection === void 0 ? void 0 : intersection.end).toBe(5);
    });
    it("should union ranges correctly", () => {
        const other = new NumberRange(3, 7);
        const union = range.union(other);
        expect(union.length).toBe(1);
        expect(union[0].start).toBe(1);
        expect(union[0].end).toBe(7);
    });
    it("should calculate difference of ranges", () => {
        const other = new NumberRange(3, 7);
        const difference = range.difference(other);
        expect(difference.length).toBe(1);
        expect(difference[0].start).toBe(1);
        expect(difference[0].end).toBe(3);
    });
    it("should shift range correctly", () => {
        const shifted = range.shift(2);
        expect(shifted.start).toBe(3);
        expect(shifted.end).toBe(7);
    });
    it("should scale range correctly", () => {
        const scaled = range.scale(2);
        expect(scaled.start).toBe(1);
        expect(scaled.end).toBe(9);
    });
});
//# sourceMappingURL=numberRange.spec.js.map