/* eslint-disable no-useless-constructor */
import "reflect-metadata";
import { vec2 } from "gl-matrix";
import { anchorTypeToUnitSpace } from "./position.js";
export class PositionRelative {
    get vector() {
        return vec2.fromValues(this.x, this.y);
    }
    static zero() {
        return new PositionRelative(0, 0);
    }
    static fromVector(vector) {
        return new PositionRelative(vector[0], vector[1]);
    }
    static fromPositionAnchor(position) {
        const point = anchorTypeToUnitSpace(position.anchorX, position.anchorY);
        return new PositionRelative(point.x, point.y);
    }
    // eslint-disable-next-line no-useless-constructor
    constructor(x, y) {
        this.x = x;
        this.y = y;
    }
    add(other) {
        const differenceVector = vec2.create();
        vec2.add(differenceVector, this.vector, other.vector);
        return PositionRelative.fromVector(differenceVector);
    }
    subtract(other) {
        return this.difference(other);
    }
    difference(other) {
        const differenceVector = vec2.create();
        vec2.subtract(differenceVector, this.vector, other.vector);
        return PositionRelative.fromVector(differenceVector);
    }
    clone() {
        return new PositionRelative(this.x, this.y);
    }
}
// Define a class for Size
export class SizeRelative {
    get vector() {
        return vec2.fromValues(this.width, this.height);
    }
    static fromVector(vector) {
        return new SizeRelative(vector[0], vector[1]);
    }
    constructor(width, height) {
        this.width = width;
        this.height = height;
    }
    difference(other) {
        const differenceVector = vec2.create();
        vec2.subtract(differenceVector, this.vector, other.vector);
        return PositionRelative.fromVector(differenceVector);
    }
    clone() {
        return new SizeRelative(this.width, this.height);
    }
}
// Define a class for Rectangle
export class RectangleRelative {
    static fromCoordinates(x, y, width, height, anchorX, anchorY) {
        return new RectangleRelative(new SizeRelative(width, height), new PositionRelative(x, y), new PositionRelative(anchorX !== null && anchorX !== void 0 ? anchorX : 0, anchorY !== null && anchorY !== void 0 ? anchorY : 0));
    }
    static unit() {
        return new RectangleRelative(new SizeRelative(1, 1), new PositionRelative(0, 0), new PositionRelative(0, 0));
    }
    // Getters Setters
    get position() {
        return this._position;
    }
    set position(position) {
        this._position = position;
    }
    get positionVector() {
        return vec2.fromValues(this._position.x, this._position.y);
    }
    get center() {
        return new PositionRelative(this.right + this.left / 2, this.bottom + this.top / 2);
    }
    get size() {
        return this._size;
    }
    set size(newSize) {
        const oldSize = this._size.clone();
        this._size = newSize;
        this.setAnchorOffset();
        // Recalculate position based on the new size
        this._position.x += (newSize.width - oldSize.width) * this.anchorPoint.x;
        this._position.y += (newSize.height - oldSize.height) * this.anchorPoint.y;
    }
    get anchorPoint() {
        return this._anchorPoint;
    }
    set anchorPoint(newAnchorPoint) {
        // Const oldAnchorPoint = this._anchorPoint.clone();
        this._anchorPoint = newAnchorPoint;
        this.setAnchorOffset();
        // Recalculate position based on the new anchor point
        // this._position.x += (newAnchorPoint.x - oldAnchorPoint.x) * this.size.width;
        // this._position.y +=
        //   (newAnchorPoint.y - oldAnchorPoint.y) * this.size.height;
    }
    moveAnchorPoint(newAnchorPoint) {
        const oldAnchorPoint = this._anchorPoint.clone();
        this._anchorPoint = newAnchorPoint;
        this.setAnchorOffset();
        // Recalculate position based on the new anchor point
        this._position.x += (newAnchorPoint.x - oldAnchorPoint.x) * this.size.width;
        this._position.y +=
            (newAnchorPoint.y - oldAnchorPoint.y) * this.size.height;
    }
    get anchorOffsetVector() {
        return this.anchorOffset.vector;
    }
    get anchorOffsetInvertedVector() {
        return this.anchorOffsetInverted.vector;
    }
    get parent() {
        return this._parent;
    }
    set parent(parent) {
        if (this.isDescendantOf(parent)) {
            throw new Error("Cannot set parent to a descendant rectangle, it would create a circular reference.");
        }
        this._parent = parent;
    }
    get boundary() {
        return this._boundary;
    }
    set boundary(boundary) {
        this._boundary = boundary;
    }
    // Getters for edges
    get left() {
        return this.position.x - this.anchorOffset.x;
    }
    getHorizontal(unit) {
        return this.left + this.size.width * unit;
    }
    get horizontalCenter() {
        return this.left + this.size.width / 2;
    }
    get right() {
        return this.left + this.size.width;
    }
    get top() {
        return this.position.y - this.anchorOffset.y;
    }
    getVertical(unit) {
        return this.top + this.size.height * unit;
    }
    get verticalCenter() {
        return this.top + this.size.height / 2;
    }
    get bottom() {
        return this.top + this.size.height;
    }
    get topLeft() {
        return new PositionRelative(this.left, this.top);
    }
    get topRight() {
        return new PositionRelative(this.right, this.top);
    }
    get bottomLeft() {
        return new PositionRelative(this.left, this.bottom);
    }
    get bottomRight() {
        return new PositionRelative(this.right, this.bottom);
    }
    get edges() {
        return {
            left: this.left,
            top: this.top,
            right: this.right,
            bottom: this.bottom,
        };
    }
    get ratioFraction() {
        return this._ratioFraction;
    }
    set ratioFraction(ratioFraction) {
        if (ratioFraction[1] === 0 || ratioFraction[0] === 0) {
            throw new Error("Ratio height or width cannot be zero.");
        }
        this._ratioFraction = ratioFraction;
        const [ratioWidth, ratioHeight] = this._ratioFraction;
        this._aspectRatio = ratioWidth / ratioHeight;
        this.resizeFromAspectRatio();
    }
    get aspectRatio() {
        return this._aspectRatio;
    }
    get keepRatio() {
        return this._keepRatio;
    }
    set keepRatio(keepRatio) {
        this._keepRatio = keepRatio;
    }
    constructor(size, position, anchorPoint = new PositionRelative(0, 0), parent = null, ratioFraction = [1, 1], keepRatio = false) {
        this._size = size;
        this._position = position;
        this._anchorPoint = anchorPoint;
        this.anchorOffset = new PositionRelative(this._size.width * this._anchorPoint.x, this._size.height * this._anchorPoint.y);
        this.anchorOffsetInverted = new PositionRelative(this._size.width * (1 - this._anchorPoint.x), this._size.height * (1 - this._anchorPoint.y));
        this._parent = parent;
        this._boundary = null;
        this._children = [];
        this._ratioFraction = ratioFraction;
        this._aspectRatio = ratioFraction[0] / ratioFraction[1];
        this._keepRatio = keepRatio;
    }
    setAnchorOffset() {
        this.anchorOffset = new PositionRelative(this._size.width * this._anchorPoint.x, this._size.height * this._anchorPoint.y);
        this.anchorOffsetInverted = new PositionRelative(this._size.width * (1 - this._anchorPoint.x), this._size.height * (1 - this._anchorPoint.y));
    }
    addChild(child) {
        if (child.isDescendantOf(this)) {
            throw new Error("Cannot add a descendant rectangle as a child, it would create a circular reference.");
        }
        child.parent = this;
        this._children.push(child);
    }
    removeChild(child) {
        const index = this._children.indexOf(child);
        if (index !== -1) {
            this._children.splice(index, 1);
            child.parent = null;
        }
    }
    getChildren() {
        return this._children;
    }
    move(deltaX, deltaY, snapActive = {
        x: false,
        y: false,
        left: false,
        top: false,
        right: false,
        bottom: false,
    }, snapDistance = 10, snapSiblings = []) {
        const xMoved = deltaX !== 0;
        const yMoved = deltaY !== 0;
        let newX = this.position.x + deltaX;
        let newY = this.position.y + deltaY;
        // Snapping logic
        snapSiblings.forEach((sibling) => {
            if (xMoved) {
                if (snapActive.x) {
                    const snapResult = this.applyMoveSnapX(newX, newY, deltaX, sibling, snapDistance);
                    newX = snapResult.x;
                }
                if (snapActive.left) {
                    const snapResult1 = this.applyMoveSnapLeftEdge(newX, newY, deltaX, sibling, snapDistance);
                    newX = snapResult1.x;
                }
                if (snapActive.right) {
                    const snapResult2 = this.applyMoveSnapRightEdge(newX, newY, deltaX, sibling, snapDistance);
                    newX = snapResult2.x;
                }
            }
            if (yMoved) {
                if (snapActive.y) {
                    const snapResult = this.applyMoveSnapY(newX, newY, deltaY, sibling, snapDistance);
                    newY = snapResult.y;
                }
                if (snapActive.top) {
                    const snapResult1 = this.applyMoveSnapTopEdge(newX, newY, deltaY, sibling, snapDistance);
                    newY = snapResult1.y;
                }
                if (snapActive.bottom) {
                    const snapResult2 = this.applyMoveSnapBottomEdge(newX, newY, deltaY, sibling, snapDistance);
                    newY = snapResult2.y;
                }
            }
        });
        if (this._boundary) {
            // Const rectLeft = newX - this.anchorOffset.x;
            // const rectTop = newY - this.anchorOffset.y;
            // const rectRight = rectLeft + this._size.width;
            // const rectBottom = rectTop + this._size.height;
            // if (rectLeft < this._boundary.left) {
            //   newX = this._boundary.left + this.anchorOffset.x;
            // }
            // if (rectRight > this._boundary.right) {
            //   newX = this._boundary.right - this.anchorOffsetInverted.x;
            // }
            // if (rectTop < this._boundary.top) {
            //   newY = this._boundary.top + this.anchorOffset.y;
            // }
            // if (rectBottom > this._boundary.bottom) {
            //   newY = this._boundary.bottom - this.anchorOffsetInverted.y;
            // }
        }
        this.position = new PositionRelative(newX, newY);
    }
    applyMoveSnapX(x, y, delta, sibling, snapDistance) {
        let newX = x;
        const newY = y;
        if (this.insideSnapDistance(x, sibling.right, snapDistance)) {
            newX = sibling.right;
        }
        if (this.insideSnapDistance(x, sibling.left, snapDistance)) {
            newX = sibling.left;
        }
        if (this.insideSnapDistance(x, sibling.horizontalCenter, snapDistance)) {
            newX = sibling.horizontalCenter;
        }
        if (this.insideSnapDistance(x, sibling.position.x, snapDistance)) {
            newX = sibling.position.x;
        }
        return { x: newX, y: newY };
    }
    applyMoveSnapY(x, y, delta, sibling, snapDistance) {
        const newX = x;
        let newY = y;
        if (this.insideSnapDistance(y, sibling.bottom, snapDistance)) {
            newY = sibling.bottom;
        }
        if (this.insideSnapDistance(y, sibling.verticalCenter, snapDistance)) {
            newY = sibling.verticalCenter;
        }
        if (this.insideSnapDistance(y, sibling.top, snapDistance)) {
            newY = sibling.top;
        }
        if (this.insideSnapDistance(y, sibling.position.y, snapDistance)) {
            newY = sibling.position.y;
        }
        return { x: newX, y: newY };
    }
    applyMoveSnapLeftEdge(x, y, delta, sibling, snapDistance) {
        let newX = x;
        const newY = y;
        if (this.insideSnapDistance(this.left + delta, sibling.right, snapDistance)) {
            newX = sibling.right + this.anchorOffset.x;
        }
        if (this.insideSnapDistance(this.left + delta, sibling.horizontalCenter, snapDistance)) {
            newX = sibling.horizontalCenter + this.anchorOffset.x;
        }
        if (this.insideSnapDistance(this.left + delta, sibling.left, snapDistance)) {
            newX = sibling.left + this.anchorOffset.x;
        }
        return { x: newX, y: newY };
    }
    applyMoveSnapRightEdge(x, y, delta, sibling, snapDistance) {
        let newX = x;
        const newY = y;
        if (this.insideSnapDistance(this.right + delta, sibling.left, snapDistance)) {
            newX = sibling.left - this.anchorOffsetInverted.x;
        }
        if (this.insideSnapDistance(this.right + delta, sibling.horizontalCenter, snapDistance)) {
            newX = sibling.horizontalCenter - this.anchorOffsetInverted.x;
        }
        if (this.insideSnapDistance(this.right + delta, sibling.right, snapDistance)) {
            newX = sibling.right - this.anchorOffsetInverted.x;
        }
        return { x: newX, y: newY };
    }
    applyMoveSnapTopEdge(x, y, delta, sibling, snapDistance) {
        const newX = x;
        let newY = y;
        if (this.insideSnapDistance(this.top + delta, sibling.bottom, snapDistance)) {
            newY = sibling.bottom + this.anchorOffset.y;
        }
        if (this.insideSnapDistance(this.top + delta, sibling.verticalCenter, snapDistance)) {
            newY = sibling.verticalCenter + this.anchorOffset.y;
        }
        if (this.insideSnapDistance(this.top + delta, sibling.top, snapDistance)) {
            newY = sibling.top + this.anchorOffset.y;
        }
        return { x: newX, y: newY };
    }
    applyMoveSnapBottomEdge(x, y, delta, sibling, snapDistance) {
        const newX = x;
        let newY = y;
        if (this.insideSnapDistance(this.bottom + delta, sibling.top, snapDistance)) {
            newY = sibling.top - this.anchorOffsetInverted.y;
        }
        if (this.insideSnapDistance(this.bottom + delta, sibling.verticalCenter, snapDistance)) {
            newY = sibling.verticalCenter - this.anchorOffsetInverted.y;
        }
        if (this.insideSnapDistance(this.bottom + delta, sibling.bottom, snapDistance)) {
            newY = sibling.bottom - this.anchorOffsetInverted.y;
        }
        return { x: newX, y: newY };
    }
    resizeFromAspectRatio() {
        if (this._aspectRatio > 1) {
            this.size.height = this.size.width / this._aspectRatio;
        }
        else {
            this.size.width = this.size.height * this._aspectRatio;
        }
    }
    resize(deltaLeft, deltaTop, deltaRight, deltaBottom, snapActive = {
        x: false,
        y: false,
        left: false,
        top: false,
        right: false,
        bottom: false,
    }, snapDistance = 10, snapSiblings = []) {
        const leftChanged = deltaLeft !== 0;
        const rightChanged = deltaRight !== 0;
        const topChanged = deltaTop !== 0;
        const bottomChanged = deltaBottom !== 0;
        if (this._keepRatio) {
            if (leftChanged || rightChanged) {
                deltaTop = deltaLeft / this._aspectRatio;
                deltaBottom = deltaRight / this._aspectRatio;
            }
            else if (topChanged || bottomChanged) {
                deltaLeft = deltaTop * this._aspectRatio;
                deltaRight = deltaBottom * this._aspectRatio;
            }
        }
        let newX = this.position.x - deltaLeft;
        let newY = this.position.y - deltaTop;
        let newWidth = this.size.width + deltaLeft + deltaRight;
        let newHeight = this.size.height + deltaTop + deltaBottom;
        let newRect = RectangleRelative.fromCoordinates(newX, newY, newWidth, newHeight);
        // Snapping logic
        snapSiblings.forEach((sibling) => {
            // Snap to sibling edges
            if (leftChanged && snapActive.left) {
                const snapResult = this.applyResizeSnapLeftEdge(newX, newY, newWidth, newHeight, deltaLeft, sibling, snapDistance);
                newX = snapResult.x;
                newY = snapResult.y;
                newWidth = snapResult.width;
                newHeight = snapResult.height;
            }
            if (rightChanged && snapActive.right) {
                const snapResult = this.applyResizeSnapRightEdge(newX, newY, newWidth, newHeight, deltaRight, sibling, snapDistance);
                newWidth = snapResult.width;
            }
            if (topChanged && snapActive.top) {
                const snapResult = this.applyResizeSnapTopEdge(newX, newY, newWidth, newHeight, deltaTop, sibling, snapDistance);
                newY = snapResult.y;
                newHeight = snapResult.height;
            }
            if (bottomChanged && snapActive.bottom) {
                const snapResult = this.applyResizeSnapBottomEdge(newX, newY, newWidth, newHeight, deltaBottom, sibling, snapDistance);
                newHeight = snapResult.height;
            }
        });
        newRect = RectangleRelative.fromCoordinates(newX, newY, newWidth, newHeight);
        if (this.boundary) {
            if (!this.boundary.contains(newRect)) {
                const limited = this.pushInto(this.boundary);
                newRect = limited;
            }
        }
        this.position = newRect.position;
        this.size = newRect.size;
    }
    applyResizeSnapLeftEdge(x, y, width, height, delta, sibling, snapDistance) {
        let newX = x;
        const newY = y;
        let newWidth = width;
        const newHeight = height;
        if (this.insideSnapDistance(this.left - delta, sibling.right, snapDistance)) {
            newX = sibling.right + this.anchorOffset.x;
            newWidth = this.right - newX;
        }
        if (this.insideSnapDistance(this.left - delta, sibling.horizontalCenter, snapDistance)) {
            newX = sibling.horizontalCenter + this.anchorOffset.x;
            newWidth = this.right - newX;
        }
        if (this.insideSnapDistance(this.left - delta, sibling.left, snapDistance)) {
            newX = sibling.left + this.anchorOffset.x;
            newWidth = this.right - newX;
        }
        return { x: newX, y: newY, width: newWidth, height: newHeight };
    }
    applyResizeSnapRightEdge(x, y, width, height, delta, sibling, snapDistance) {
        const newX = x;
        const newY = y;
        let newWidth = width;
        const newHeight = height;
        if (this.insideSnapDistance(this.right + delta, sibling.left, snapDistance)) {
            newWidth = sibling.left - this.left;
        }
        if (this.insideSnapDistance(this.right + delta, sibling.horizontalCenter, snapDistance)) {
            newWidth = sibling.horizontalCenter - this.left;
        }
        if (this.insideSnapDistance(this.right + delta, sibling.right, snapDistance)) {
            newWidth = sibling.right - this.left;
        }
        return { x: newX, y: newY, width: newWidth, height: newHeight };
    }
    applyResizeSnapTopEdge(x, y, width, height, delta, sibling, snapDistance) {
        const newX = x;
        let newY = y;
        const newWidth = width;
        let newHeight = height;
        if (this.insideSnapDistance(this.top - delta, sibling.bottom, snapDistance)) {
            newY = sibling.bottom + this.anchorOffset.y;
            newHeight = this.bottom - newY;
        }
        if (this.insideSnapDistance(this.top - delta, sibling.verticalCenter, snapDistance)) {
            newY = sibling.verticalCenter + this.anchorOffset.y;
            newHeight = this.bottom - newY;
        }
        if (this.insideSnapDistance(this.top - delta, sibling.top, snapDistance)) {
            newY = sibling.top + this.anchorOffset.y;
            newHeight = this.bottom - newY;
        }
        return { x: newX, y: newY, width: newWidth, height: newHeight };
    }
    applyResizeSnapBottomEdge(x, y, width, height, delta, sibling, snapDistance) {
        const newX = x;
        const newY = y;
        const newWidth = width;
        let newHeight = height;
        if (this.insideSnapDistance(this.bottom + delta, sibling.top, snapDistance)) {
            newHeight = sibling.top - this.top;
        }
        if (this.insideSnapDistance(this.bottom + delta, sibling.verticalCenter, snapDistance)) {
            newHeight = sibling.verticalCenter - this.top;
        }
        if (this.insideSnapDistance(this.bottom + delta, sibling.bottom, snapDistance)) {
            newHeight = sibling.bottom - this.top;
        }
        return { x: newX, y: newY, width: newWidth, height: newHeight };
    }
    positionDifference(other) {
        return this.position.difference(other.position);
    }
    sizeDifference(other) {
        return this.size.difference(other.size);
    }
    edgeDifference(other) {
        return {
            left: this.left - other.left,
            top: this.top - other.top,
            right: this.right - other.right,
            bottom: this.bottom - other.bottom,
        };
    }
    edgeDifferenceAbsolute(other) {
        const positionDifference = this.positionDifference(other);
        return {
            left: this.left - other.left + positionDifference.x,
            top: this.top - other.top + positionDifference.y,
            right: this.right - other.right + positionDifference.x,
            bottom: this.bottom - other.bottom + positionDifference.y,
        };
    }
    isIntersecting(other) {
        // Const thisTopLeft = this.topLeft.vector;
        // const thisBottomRight = this.bottomRight.vector;
        // const otherTopLeft = other.topLeft.vector;
        // const otherBottomRight = other.bottomRight.vector;
        // return (
        //   vec2.distance(thisTopLeft, otherBottomRight) > 0 &&
        //   vec2.distance(thisBottomRight, otherTopLeft) > 0
        // );
        return (this.left < other.right &&
            this.right > other.left &&
            this.top < other.bottom &&
            this.bottom > other.top);
    }
    contains(other) {
        return (this.left <= other.left &&
            this.right >= other.right &&
            this.top <= other.top &&
            this.bottom >= other.bottom);
    }
    isInside(other) {
        return other.contains(this);
    }
    intersection(other) {
        if (!this.isIntersecting(other)) {
            return null;
        }
        const left = Math.max(this.left, other.left);
        const top = Math.max(this.top, other.top);
        const right = Math.min(this.right, other.right);
        const bottom = Math.min(this.bottom, other.bottom);
        return RectangleRelative.fromCoordinates(left, top, right - left, bottom - top);
    }
    pushInto(other) {
        if (this.isInside(other)) {
            return this.clone();
        }
        const thisCenter = this.center.vector;
        const otherCenter = other.center.vector;
        const direction = vec2.create();
        vec2.subtract(direction, otherCenter, thisCenter);
        vec2.normalize(direction, direction);
        // Const distanceToMove = Math.max(
        //   Math.abs(other.right - this.right),
        //   Math.abs(other.left - this.left),
        //   Math.abs(other.top - this.top),
        //   Math.abs(other.bottom - this.bottom)
        // );
        const dx = Math.max(other.left - this.right, this.left - other.right, 0);
        const dy = Math.max(other.top - this.bottom, this.bottom - other.top, 0);
        const distanceToMove = Math.sqrt(dx * dx + dy * dy);
        const newPosition = vec2.create();
        vec2.scaleAndAdd(newPosition, thisCenter, direction, distanceToMove);
        return new RectangleRelative(this.size.clone(), PositionRelative.fromVector(newPosition), this.anchorPoint.clone(), this.parent);
    }
    // Method to project this rectangle into the coordinate system of another rectangle
    projectTo(target) {
        // Calculate the relative position of this rectangle in the target's coordinate system
        const projectedPositionVector = vec2.create();
        vec2.subtract(projectedPositionVector, this.position.vector, target.position.vector);
        // Adjust the position by considering the anchor points
        vec2.subtract(projectedPositionVector, projectedPositionVector, this.anchorOffset.vector);
        vec2.add(projectedPositionVector, projectedPositionVector, target.anchorOffset.vector);
        const projectedAnchorPoint = new PositionRelative(this.anchorPoint.x, this.anchorPoint.y);
        // Return the new projected rectangle
        return new RectangleRelative(this.size, PositionRelative.fromVector(projectedPositionVector), projectedAnchorPoint, target);
    }
    calculateSubRectangle(subX, subY, subWidth, subHeight) {
        // Calculate the new position relative to this rectangle's coordinate system
        const newVector = vec2.create();
        vec2.add(newVector, this.position.vector, vec2.fromValues(subX, subY));
        const newPosition = PositionRelative.fromVector(newVector);
        // Calculate the size of the sub-rectangle
        const newSize = new SizeRelative(subWidth, subHeight);
        // Create and return the sub-rectangle
        return new RectangleRelative(newSize, newPosition, new PositionRelative(0, 0) // Assuming the sub-rectangle's anchor point is (0,0)
        );
    }
    subRectangleFromUnitSpace(subX, subY, subWidth, subHeight, anchorPoint = PositionRelative.zero()) {
        // Convert unit space to pixel space
        const subXInPixels = subX * this.size.width;
        const subYInPixels = subY * this.size.height;
        const subWidthInPixels = subWidth * this.size.width;
        const subHeightInPixels = subHeight * this.size.height;
        // Calculate the new position relative to this rectangle's coordinate system
        const newPosition = new PositionRelative(subXInPixels, subYInPixels);
        // Calculate the size of the sub-rectangle
        const newSize = new SizeRelative(subWidthInPixels, subHeightInPixels);
        // Create and return the sub-rectangle
        return new RectangleRelative(newSize, newPosition, anchorPoint);
    }
    subRectangleInUnitSpace(input) {
        // Calculate the relative position of the input rectangle in unit space
        const subXUnit = input.position.x / this.size.width;
        const subYUnit = input.position.y / this.size.height;
        const subWidthUnit = input.size.width / this.size.width;
        const subHeightUnit = input.size.height / this.size.height;
        const newPosition = new PositionRelative(subXUnit, subYUnit);
        const newSize = new SizeRelative(subWidthUnit, subHeightUnit);
        return new RectangleRelative(newSize, newPosition, input.anchorPoint.clone());
    }
    clone() {
        return new RectangleRelative(this.size.clone(), this.position.clone(), this.anchorPoint.clone(), this.parent, [...this.ratioFraction], this.keepRatio);
    }
    isDescendantOf(rectangle) {
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        let current = this;
        while (current) {
            if (current === rectangle) {
                return true;
            }
            current = current.parent;
        }
        return false;
    }
    insideSnapDistance(a, b, snapDistance) {
        return Math.abs(a - b) <= snapDistance;
    }
}
//# sourceMappingURL=rectangle-relative.js.map