import { describe, it, expect, beforeEach } from "vitest";
import { Trie } from "./trie.js";
describe("Trie", () => {
    let trie;
    beforeEach(() => {
        trie = new Trie();
    });
    describe("Contains", () => {
        it("should return false if the trie is empty", () => {
            expect(trie.Contains("word")).toBe(false);
        });
        it("should return false if the word is not in the trie", () => {
            trie.Insert("hello");
            expect(trie.Contains("word")).toBe(false);
        });
        it("should return true if the word is in the trie", () => {
            trie.Insert("hello");
            expect(trie.Contains("hello")).toBe(true);
        });
        it("should return true if the word is in the trie with a reference", () => {
            trie.Insert("hello", "reference");
            expect(trie.Contains("hello", "reference")).toBe(true);
        });
        it("should return false if the word is in the trie but with a different reference", () => {
            trie.Insert("hello", "reference1");
            expect(trie.Contains("hello", "reference2")).toBe(false);
        });
    });
    describe("GetReferences", () => {
        it("should return an empty array if the word is not in the trie", () => {
            trie.Insert("hello", "reference1");
            expect(trie.GetReferences("word")).toEqual([]);
        });
        it("should return an empty array if the word is in the trie but has no references", () => {
            trie.Insert("hello");
            expect(trie.GetReferences("hello")).toEqual([]);
        });
        it("should return an array with the references if the word is in the trie and has references", () => {
            trie.Insert("hello", "reference1");
            trie.Insert("hello", "reference2");
            expect(trie.GetReferences("hello")).toEqual(["reference1", "reference2"]);
        });
    });
    describe("Insert", () => {
        it("should insert the word into the trie", () => {
            trie.Insert("hello");
            expect(trie.Contains("hello")).toBe(true);
        });
        it("should insert the reference into the trie", () => {
            trie.Insert("hello", "reference");
            expect(trie.Contains("hello", "reference")).toBe(true);
        });
        it("should insert multiple references for the same word into the trie", () => {
            trie.Insert("hello", "reference1");
            trie.Insert("hello", "reference2");
            expect(trie.Contains("hello", "reference1")).toBe(true);
            expect(trie.Contains("hello", "reference2")).toBe(true);
        });
    });
    describe("Remove", () => {
        it("should do nothing if the word is not in the trie", () => {
            trie.Insert("hello", "reference1");
            trie.Remove("word");
            expect(trie.Contains("hello", "reference1")).toBe(true);
        });
        it("should remove the word from the trie if it has no references", () => {
            trie.Insert("hello");
            trie.Remove("hello");
            expect(trie.Contains("hello")).toBe(false);
        });
        it("should remove the reference from the word if it has references", () => {
            trie.Insert("hello", "reference1");
            trie.Insert("hello", "reference2");
            trie.Remove("hello", "reference1");
            expect(trie.Contains("hello", "reference1")).toBe(false);
            expect(trie.Contains("hello", "reference2")).toBe(true);
        });
        it("should remove the word from the trie if all references are removed", () => {
            trie.Insert("hello", "reference1");
            trie.Insert("hello", "reference2");
            trie.Remove("hello", "reference1");
            trie.Remove("hello", "reference2");
            expect(trie.Contains("hello")).toBe(false);
        });
    });
    describe("FindWords", () => {
        const trie = new Trie();
        trie.Insert("cat");
        trie.Insert("car");
        trie.Insert("rat");
        trie.Insert("bat");
        trie.Insert("dog");
        it("should return all words that start with the given prefix", () => {
            const prefix = "c";
            const expected = ["cat", "car"];
            const actual = trie.FindWords(prefix);
            expect(actual).toEqual(expected);
        });
        it("should return an empty array if there are no words that start with the given prefix", () => {
            const prefix = "x";
            const actual = trie.FindWords(prefix);
            expect(actual).toEqual([]);
        });
    });
    describe("FindWordsWithReferences", () => {
        const trie = new Trie();
        trie.Insert("cat", "pet");
        trie.Insert("car", "vehicle");
        trie.Insert("rat", "rodent");
        trie.Insert("bat", "mammal");
        trie.Insert("dog", "pet");
        it("should return all words that start with the given prefix and have at least one reference", () => {
            const prefix = "c";
            const expected = [
                ["cat", ["pet"]],
                ["car", ["vehicle"]],
            ];
            const actual = trie.FindWordsWithReferences(prefix);
            expect(actual).toEqual(expected);
        });
        it("should return an empty array if there are no words that start with the given prefix and have at least one reference", () => {
            const prefix = "x";
            const expected = [];
            const actual = trie.FindWordsWithReferences(prefix);
            expect(actual).toEqual(expected);
        });
    });
    describe("TraversePreOrder", () => {
        it("should traverse all the nodes of the trie in pre-order", () => {
            trie.Insert("hello");
            trie.Insert("help");
            trie.Insert("helping");
            trie.Insert("helps");
            trie.Insert("helped");
            const nodes = [];
            trie.TraversePreOrder((node) => nodes.push(node.Value));
            expect(nodes).toEqual([
                " ",
                "h",
                "e",
                "l",
                "l",
                "o",
                "p",
                "i",
                "n",
                "g",
                "s",
                "e",
                "d",
            ]);
        });
    });
    describe("TraverseInOrder", () => {
        it("should traverse all the nodes of the trie in-order", () => {
            trie.Insert("hello");
            trie.Insert("help");
            trie.Insert("helping");
            trie.Insert("helps");
            trie.Insert("helped");
            const nodes = [];
            trie.TraverseInOrder((node) => nodes.push(node.Value));
            expect(nodes).toEqual([
                "o",
                "l",
                "g",
                "n",
                "i",
                "s",
                "d",
                "e",
                "p",
                "l",
                "e",
                "h",
                " ",
            ]);
        });
    });
    describe("TraversePostOrder", () => {
        it("should traverse all the nodes of the trie in post-order", () => {
            trie.Insert("hello");
            trie.Insert("help");
            trie.Insert("helping");
            trie.Insert("helps");
            trie.Insert("helped");
            const nodes = [];
            trie.TraversePostOrder((node) => nodes.push(node.Value));
            expect(nodes).toEqual([
                "o",
                "l",
                "g",
                "n",
                "i",
                "s",
                "d",
                "e",
                "p",
                "l",
                "e",
                "h",
                " ",
            ]);
        });
    });
});
//# sourceMappingURL=trie.spec.js.map