import "reflect-metadata";
import { immerable } from "immer";
import type { IVariant, LanguageKey, IProperty } from "../interfaces/index.js";
export declare const LOCKED = "LOCKED";
export declare class VariantMap<K, T> {
    [immerable]: boolean;
    collection: IVariant<K, T>[];
    private _default?;
    get default(): T | undefined;
    get keys(): K[];
    get values(): T[];
    get hasValues(): boolean;
    get length(): number;
    onChange: (collection: IVariant<K, T>[]) => void;
    constructor(map?: IVariant<K, T>[]);
    setDefault(defaultValue: T): void;
    initMap(map: IVariant<K, T>[]): void;
    exists(key: K): boolean;
    find(expression: (key: K, item: IVariant<K, T>) => boolean): IVariant<K, T> | undefined;
    get(key: K, createIfMissing?: boolean): T | null;
    /**
     * Adds a new key-value pair to the collection.
     *
     * @param key - The key to add to the collection.
     * @param value - The value associated with the key.
     * @returns A boolean indicating whether the addition was successful.
     * @throws {Error} If the key already exists in the collection.
     */
    add(key: K, value: T): boolean;
    /**
     * Updates the value associated with the specified key in the collection.
     * If the key is found and the associated value is not locked, the value is updated.
     * If the key is found and the associated value is locked, the update is not performed.
     * If the key is not found, the update is not performed.
     *
     * @param key - The key of the item to update.
     * @param value - The new value to associate with the specified key.
     * @returns `true` if the update was successful, `false` otherwise.
     */
    update(key: K, value: T): boolean;
    set(key: K, value: T): boolean;
    upsert(key: K, value: T): boolean;
    remove(key: K): boolean;
    cloneItem(key: K, newKey: K): boolean;
    cloneFromDefault(newKey: K): boolean;
    toIVariantArray(): IVariant<K, T>[];
    static fromArray<Key, Type extends {
        name: string;
    }>(variants: IVariant<Key, Type[]>[]): VariantMap<Key, Map<string, Type>>;
    /**
     * Creates a `VariantMap` instance from an array of `IVariant` objects.
     *
     * @template K - The type of the key in the `VariantMap`.
     * @template T - The type of the value in the `VariantMap`.
     * @template KDto - The type of the key in the DTO `IVariant`.
     * @template TDto - The type of the value in the DTO `IVariant`.
     *
     * @param {IVariant<KDto, TDto>[]} variants - An array of `IVariant` objects to be converted.
     * @param {(item: IVariant<KDto, TDto>) => IVariant<K, T>} [dtoToMap=(item) => item as unknown as IVariant<K, T>] - A function to convert a DTO `IVariant` to a `VariantMap` `IVariant`.
     * @param {(item: IVariant<K, T>) => IVariant<KDto, TDto>} [mapToDto=(item) => item as unknown as IVariant<KDto, TDto>] - A function to convert a `VariantMap` `IVariant` to a DTO `IVariant`.
     * @returns {VariantMap<K, T>} A new `VariantMap` instance containing the converted `IVariant` objects.
     */
    static fromIVariantArray<K, T, KDto, TDto>(variants: IVariant<KDto, TDto>[], onChange?: (variants: IVariant<KDto, TDto>[]) => void, dtoToMap?: (item: IVariant<KDto, TDto>) => IVariant<K, T>, mapToDto?: (item: IVariant<K, T>) => IVariant<KDto, TDto>): VariantMap<K, T>;
}
export declare function fromIVariantArrayToVariantMap(variants: IVariant<LanguageKey, IProperty[]>[]): VariantMap<LanguageKey, Map<string, IProperty>>;
//# sourceMappingURL=variantMap.d.ts.map