import { describe, it, expect, beforeEach } from "vitest";
import { VariantMapMapper, LOCKED } from "./variantMapMapper";
describe("VariantMapMapper", () => {
    let mapper;
    const fromDto = (dto) => ({
        key: { id: dto.key.id },
        value: Object.assign({}, dto.value),
    });
    const toDto = (item) => ({
        key: { id: item.key.id },
        value: Object.assign({}, item.value),
    });
    beforeEach(() => {
        mapper = new VariantMapMapper([], fromDto, toDto);
    });
    it("should initialize with an empty collection", () => {
        expect(mapper.length).toBe(0);
        expect(mapper.hasValues).toBe(false);
    });
    it("should add a new item", () => {
        const key = { id: 1 };
        const value = { name: "Test" };
        mapper.add(key, value);
        expect(mapper.length).toBe(1);
        expect(mapper.get(key)).toEqual(value);
    });
    it("should not add an item with an existing key", () => {
        const key = { id: 1 };
        const value = { name: "Test" };
        mapper.add(key, value);
        expect(() => mapper.add(key, value)).toThrow(`Key ${key} already exists`);
    });
    it("should update an existing item", () => {
        const key = { id: 1 };
        const value = { name: "Test" };
        mapper.add(key, value);
        const newValue = { name: "Updated" };
        const updated = mapper.update(key, newValue);
        expect(updated).toBe(true);
        expect(mapper.get(key)).toEqual(newValue);
    });
    it("should not update a locked item", () => {
        const key = { id: 1 };
        const value = { name: "Test", [LOCKED]: true };
        mapper.add(key, value);
        const newValue = { name: "Updated" };
        const updated = mapper.update(key, newValue);
        expect(updated).toBe(false);
        expect(mapper.get(key)).toEqual(value);
    });
    it("should remove an existing item", () => {
        const key = { id: 1 };
        const value = { name: "Test" };
        mapper.add(key, value);
        const removed = mapper.remove(key);
        expect(removed).toBe(true);
        expect(mapper.get(key)).toBeNull();
    });
    it("should not remove a locked item", () => {
        const key = { id: 1 };
        const value = { name: "Test", [LOCKED]: true };
        mapper.add(key, value);
        const removed = mapper.remove(key);
        expect(removed).toBe(false);
        expect(mapper.get(key)).toEqual(value);
    });
    it("should clone an item", () => {
        const key = { id: 1 };
        const newKey = { id: 2 };
        const value = { name: "Test" };
        mapper.add(key, value);
        const cloned = mapper.cloneItem(key, newKey);
        expect(cloned).toBe(true);
        expect(mapper.get(newKey)).toEqual(value);
    });
    it("should not clone an item to an existing key", () => {
        const key = { id: 1 };
        const newKey = { id: 2 };
        const value = { name: "Test" };
        mapper.add(key, value);
        mapper.add(newKey, value);
        expect(() => mapper.cloneItem(key, newKey)).toThrow(`Key ${newKey} already exists`);
    });
    it("should find an item by expression", () => {
        const key = { id: 1 };
        const value = { name: "Test" };
        mapper.add(key, value);
        const found = mapper.find((k, item) => k.id === 1 && item.value.name === "Test");
        expect(found).toEqual({ key, value });
    });
    it("should return null if item not found", () => {
        const key = { id: 1 };
        expect(mapper.get(key)).toBeNull();
    });
    it("should set a default value", () => {
        const defaultValue = { name: "Default" };
        mapper.setDefault(defaultValue);
        expect(mapper.default).toEqual(defaultValue);
    });
    it("should get default value if createIfMissing is true", () => {
        const key = { id: 1 };
        const defaultValue = { name: "Default" };
        mapper.setDefault(defaultValue);
        const value = mapper.get(key, true);
        expect(value).toEqual(defaultValue);
    });
});
//# sourceMappingURL=variantMapMapper.spec.js.map